/*
    si - Version 0.6 - By Joe Turgeon (jturgeon@isd.net)
    loadinfo.c

    si - System Information Viewer
    Copyright (C) 1999  Joe Turgeon (jturgeon@isd.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "main.h"


// Global Variables
time_t BOOTTIME;
long int UPTIME;
// PROCFS Filename Defines
struct STRING_ARRAY  PROCINFO  [PROCINFO_COUNT_MAX];
struct STRING_ARRAY  MODULES   [MODULES_COUNT_MAX];
struct STRING_ARRAY  CHARDEV   [CHARDEV_COUNT_MAX];
struct STRING_ARRAY  BLOCKDEV  [BLOCKDEV_COUNT_MAX];
struct STRING_ARRAY  ISADMA    [ISADMA_COUNT_MAX];
struct STRING_ARRAY  FILESYS   [FILESYS_COUNT_MAX];
struct STRING_ARRAY  IRQINT    [IRQINT_COUNT_MAX];
struct STRING_ARRAY  IOPORTS   [IOPORTS_COUNT_MAX];
struct STRING_ARRAY  PARTS     [PARTS_COUNT_MAX];
struct STRING_ARRAY  MOUNTS    [MOUNTS_COUNT_MAX];
struct STRING_ARRAY  SWAPS     [SWAPS_COUNT_MAX];
struct CPU_STRUCT   CPU         [CPU_COUNT_MAX];
struct PROC_STRUCT  PROC        [PROC_COUNT_MAX];
struct SCSIDEV_STRUCT  SCSIDEV   [SCSIDEV_COUNT_MAX];
struct SCSIDRV_STRUCT  SCSIDRV   [SCSIDRV_COUNT_MAX];
struct TTYDRIVERS_STRUCT  TTYDRIVERS  [TTYDRIVERS_COUNT_MAX];
struct TTYLDISCS_STRUCT   TTYLDISCS   [TTYLDISCS_COUNT_MAX];
struct STRING_ARRAY  SNDDRV     [SNDDRV_COUNT_MAX];
struct STRING_ARRAY  SNDCONF    [SNDCONF_COUNT_MAX];
struct STRING_ARRAY  SNDAUDIO   [SNDAUDIO_COUNT_MAX];
struct STRING_ARRAY  SNDSYNTH   [SNDAUDIO_COUNT_MAX];
struct STRING_ARRAY  SNDMIDI    [SNDMIDI_COUNT_MAX];
struct STRING_ARRAY  SNDTIMERS  [SNDTIMERS_COUNT_MAX];
struct STRING_ARRAY  SNDMIXERS  [SNDMIXERS_COUNT_MAX];
struct MTRR_STRUCT  MTRR  [MTRR_COUNT_MAX];
struct IDEDRV_STRUCT  IDEDRV  [IDEDRV_COUNT_MAX];
struct IDEDEV_STRUCT  IDEDEV  [IDEDEV_COUNT_MAX];
struct PARPORT_STRUCT  PARPORT  [PARPORT_COUNT_MAX];
struct STRING_ARRAY  MISC       [MISC_COUNT_MAX];
int NUM_MODULES=-1;
int NUM_CHARDEV=-1;
int NUM_BLOCKDEV=-1;
int NUM_ISADMA=-1;
int NUM_FILESYS=-1;
int NUM_IRQINT=-1;
int NUM_IOPORTS=-1;
int NUM_PARTS=-1;
int NUM_MOUNTS=-1;
int NUM_SWAPS=-1;
int NUM_CPU=-1;
int NUM_PROC=-1;
int NUM_SCSIDEV=-1;
int NUM_SCSIDRV=-1;
int NUM_TTYDRIVERS=-1;
int NUM_TTYLDISCS=-1; 
int NUM_SNDDRV=-1;
int NUM_SNDCONF=-1;
int NUM_SNDAUDIO=-1;
int NUM_SNDSYNTH=-1;
int NUM_SNDMIDI=-1;
int NUM_SNDTIMERS=-1;
int NUM_SNDMIXERS=-1;
int NUM_MTRR=-1;
int NUM_IDEDRV=-1;
int NUM_IDEDEV=-1;
int NUM_PARPORT=-1;
int NUM_MISC=-1;


// loadinfo() Function
int loadinfo () {
	// Display Status
	fprintf (fstdout, "si:  Loading Information\n");
	// Load All proc Filesystem Information
	// si
		sprintf (PROCINFO[PROC_EXINFO].STRING, "si - Version 0.6 - By Joe Turgeon (jturgeon@isd.net)");
		PROCINFO[PROC_EXINFO].LENGTH=strlen(PROCINFO[PROC_EXINFO].STRING);
	// cmdline
		handleproc(PROC_CMDLINE, "cmdline");
	// domainname
		handleproc(PROC_DOMAINNAME, "sys/kernel/domainname");
	// hostname
		handleproc(PROC_HOSTNAME, "sys/kernel/hostname");
	// modprobe
		if (KERNEL_VERSION != TWOPOINTZERO)
			handleproc(PROC_MODPROBE, "sys/kernel/modprobe");
	// osrelease
		handleproc(PROC_OSRELEASE, "sys/kernel/osrelease");
	// ostype
		handleproc(PROC_OSTYPE, "sys/kernel/ostype");
	// version
		handleproc(PROC_VERSION, "sys/kernel/version");
	// modules
		handlemodules();
	// loadavg and uptime
		handletime();
	// cpu statistics
		if (KERNEL_VERSION == TWOPOINTZERO) handlecpuv20();
		else handlecpuv22();
	// block and character devices
		handledevices();
	// isa dma channels
		NUM_ISADMA=readprocline("dma", ISADMA);
	// filesystems
		NUM_FILESYS=readprocline("filesystems", FILESYS);
	// interrupts
		if (KERNEL_VERSION == TWOPOINTZERO) handleinterruptsv20();
		else handleinterruptsv22();
	// ioports
		NUM_IOPORTS=readprocline("ioports", IOPORTS);
	// partitions
		if (KERNEL_VERSION != TWOPOINTZERO) {
			NUM_PARTS=readprocline("partitions", PARTS);
			sprintf (PARTS[0].STRING, "  Maj   Min    Bytes  Name");
		}
	// stat
		if (KERNEL_VERSION == TWOPOINTZERO) handlestatv20();
		else handlestatv22();
	// meminfo
		handlememinfo();
	// mounts
		handlemounts();
	// swaps
		handleswaps();
	// processes
		handleprocesses();
	// scsi devices and drivers
		handlescsi();
	// tty drivers and ldiscs
		handletty();
	// sound
		handlesound();
	// mtrr
		handlemtrr();
	// ide
		handleide();
	// parport
		handleparport();
	// misc
		handlemisc();
	// Default Return Value
	return 0;
}

// handleproc() Function
void handleproc(int procnum, char *filename) {
	char tmpstring [INPUT_STRING];
	int tmplength;

	tmplength=readproc(filename, tmpstring);
	tmplength=killcrlf(tmpstring, tmplength);
	strcpy(PROCINFO[procnum].STRING, tmpstring);
	PROCINFO[procnum].LENGTH=tmplength;
	return;
}

// handlemodules() Function
void handlemodules () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int c=0;
	int tmplength;
	
	// Open /proc/modules
	sprintf (filename, "%smodules", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	// Fill Modules Structure
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		tmplength=killcrlf(tmpstring, strlen(tmpstring));
		if (strlen(tmpstring)==1) continue;
		strcpy(MODULES[c].STRING, tmpstring);
		MODULES[c].LENGTH=tmplength;
		c++;
	} while (0==0);
	NUM_MODULES=c-1;
	// Close and Cleanup
	fclose (filep);
	return;
}

// handletime() Function
void handletime () {
	FILE *filep;
	char filename [INPUT_STRING];
	char strsec [128];
	long int numsec;
	long int nummin;
	long int numhour;
	long int numday;

	// Open and Handle /proc/loadavg
	sprintf (filename, "%sloadavg", PROCLOC);
	filep=fopen(filename, "r");
	if (filep!=NULL) {
		fscanf (filep, "%s %s %s", PROCINFO[PROC_LOADAVG_1].STRING,
			PROCINFO[PROC_LOADAVG_5].STRING,
			PROCINFO[PROC_LOADAVG_15].STRING);
		PROCINFO[PROC_LOADAVG_1].LENGTH=strlen(PROCINFO[PROC_LOADAVG_1].STRING);
		PROCINFO[PROC_LOADAVG_5].LENGTH=strlen(PROCINFO[PROC_LOADAVG_5].STRING);
		PROCINFO[PROC_LOADAVG_15].LENGTH=strlen(PROCINFO[PROC_LOADAVG_15].STRING);
		fclose (filep);
	}
	// Open and Handle /proc/uptime
	sprintf (filename, "%suptime", PROCLOC);
	filep=fopen(filename, "r");
	if (filep!=NULL) {
		fscanf (filep, "%s", strsec);
		UPTIME=atol(strsec);
		numsec=UPTIME;
		numday=numsec/86400;
		numhour=(numsec-(numday*86400))/3600;
		nummin=(numsec-(numday*86400)-(numhour*3600))/60;
		numsec=(numsec-(numday*86400)-(numhour*3600)-(nummin*60));
		sprintf (PROCINFO[PROC_UPTIME].STRING, "%ld Days  %ld Hours  %ld Minutes  %ld Seconds (%s Seconds)", numday, numhour, nummin, numsec, strsec);
		PROCINFO[PROC_UPTIME].LENGTH=strlen(PROCINFO[PROC_UPTIME].STRING);
		fclose (filep);
	}
	return;
}

// handlecpuv20 ()
// 	code specific to the Linux Kernel 2.0
void handlecpuv20 () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int colpos;
	int c=-1;

	sprintf (filename, "%scpuinfo", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		if (strncmp(tmpstring, "processor", 9)==0) {
			c++;
			CPU[c].SPEED[0]=0;
			CPU[c].CACHE[0]=0;
		}
		if (strncmp(tmpstring, "vendor_id", 9)==0) {
			killcrlf(tmpstring, strlen(tmpstring));
			colpos=instr(tmpstring, ':');
			rstring(tmpstring, colpos+2, CPU[c].VENDOR);
		}
		if (strncmp(tmpstring, "model", 5)==0) {
			killcrlf(tmpstring, strlen(tmpstring));
			colpos=instr(tmpstring, ':');
			rstring(tmpstring, colpos+2, CPU[c].MODEL);
		}
	} while (0==0);
	NUM_CPU=c;
	fclose (filep);
	return;
}

// handlecpuv22 ()
//	code specific to the Linux Kernel 2.2
void handlecpuv22 () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int colpos;
	int c=-1;

	sprintf (filename, "%scpuinfo", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		if (strncmp(tmpstring, "processor", 9)==0) c++;
		if (strncmp(tmpstring, "vendor_id", 9)==0) {
			killcrlf(tmpstring, strlen(tmpstring));
			colpos=instr(tmpstring, ':');
			rstring(tmpstring, colpos+2, CPU[c].VENDOR);
		}
		if (strncmp(tmpstring, "model name", 10)==0) {
			killcrlf(tmpstring, strlen(tmpstring));
			colpos=instr(tmpstring, ':');
			rstring(tmpstring, colpos+2, CPU[c].MODEL);
		}
		if (strncmp(tmpstring, "cpu MHz", 7)==0) {
			killcrlf(tmpstring, strlen(tmpstring));
			colpos=instr(tmpstring, ':');
			rstring(tmpstring, colpos+2, CPU[c].SPEED);
		}
		if (strncmp(tmpstring, "cache size", 10)==0) {
			killcrlf(tmpstring, strlen(tmpstring));
			colpos=instr(tmpstring, ':');
			rstring(tmpstring, colpos+2, CPU[c].CACHE);
		}
	} while (0==0);
	NUM_CPU=c;
	fclose (filep);
	return;
}

// handledevices() Function
void handledevices () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int c;

	sprintf (filename, "%sdevices", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	// Character Devices
	c=0;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (strlen(tmpstring)==1) continue;
		if (strncmp(tmpstring, "Character devices", 17)==0) continue;
		if (strncmp(tmpstring, "Block devices", 13)==0) break;
		killcrlf (tmpstring, strlen(tmpstring));
		strcpy(CHARDEV[c].STRING, tmpstring);
		CHARDEV[c].LENGTH=strlen(CHARDEV[c].STRING);
		c++;
	} while (0==0);
	NUM_CHARDEV=c-1;
	// Block Devices
	c=0;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		if (strncmp(tmpstring, "Block devices", 13)==0) continue;
		killcrlf (tmpstring, strlen(tmpstring));
		strcpy(BLOCKDEV[c].STRING, tmpstring);
		BLOCKDEV[c].LENGTH=strlen(BLOCKDEV[c].STRING);
		c++;
	} while (0==0);
	NUM_BLOCKDEV=c-1;
	// Cleanup
	fclose (filep);
	return;
}

// handleinterruptsv20() Function
// 	code specific to the Linux Kernel 2.0
void handleinterruptsv20 () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int intnum;
	char intname [INPUT_STRING];
	char intcalls [INPUT_STRING];
	int c=0;

	sprintf (filename, "%sinterrupts", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		if (strlen(tmpstring)==1) continue;
		sscanf (tmpstring, "%i %*1s %11s", &intnum, intcalls);
		rstring (tmpstring, 17, intname);
		killcrlf (intname, strlen(intname));
		sprintf (IRQINT[c].STRING, "%3i  %12s  %s", intnum, intcalls, intname);
		IRQINT[c].LENGTH=strlen(IRQINT[c].STRING);
		c++;
	} while (0==0);
	c--;
	NUM_IRQINT=c;
	fclose (filep);
	return;
}

// handleinterruptsv22() Function
// 	code specific to the Linux Kernel 2.2
void handleinterruptsv22 () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int intnum;
	char intname [INPUT_STRING];
	char intcalls [INPUT_STRING];
	char intdesc [INPUT_STRING];
	int c=0;
	int i=0;

	sprintf (filename, "%sinterrupts", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		if (strlen(tmpstring)==1) continue;
		if (strncmp(tmpstring, "           CPU0", 15)==0) continue;
		if (strncmp(tmpstring, "NMI", 3)==0) continue;
		if (strncmp(tmpstring, "ERR", 3)==0) continue;
		sscanf (tmpstring, "%i %*1s", &intnum);
		sprintf (IRQINT[c].STRING, "%3i", intnum);
		rstring (tmpstring, 5, tmpstring);
		for (i=0; i<=NUM_CPU; i++) {
			sscanf (tmpstring, "%11s", intcalls);
			sprintf (IRQINT[c].STRING, "%s  %12s", IRQINT[c].STRING, intcalls);
			rstring (tmpstring, 12, tmpstring);
		}
		killcrlf (tmpstring, strlen(tmpstring));
		sscanf (tmpstring, "%s %s", intdesc, intname);
		sprintf (IRQINT[c].STRING, "%s  %15s  %s", IRQINT[c].STRING, intdesc, intname);
		IRQINT[c].LENGTH=strlen(IRQINT[c].STRING);
		c++;
	} while (0==0);
	c--;
	NUM_IRQINT=c;
	fclose (filep);
	return;
}

// handlestatv20() Function
// 	code specific to the Linux Kernel 2.0
void handlestatv20 () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	char field [INPUT_STRING];
	char strsec [INPUT_STRING];
	long int totalsec;
	long int numsec;
	long int nummin;
	long int numhour;
	long int cpuuser, cpunice, cpusys, cpuidle;

	sprintf (filename, "%sstat", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		sscanf (tmpstring, "%s", field);
		if (strcmp(field, "cpu")==0) {
			sscanf (tmpstring, "%s %s %s %s %s", field, PROCINFO[PROC_CPUUSER].STRING, PROCINFO[PROC_CPUNICE].STRING, PROCINFO[PROC_CPUSYS].STRING, PROCINFO[PROC_CPUIDLE].STRING);
			// PROC_CPUUSER
			strcpy (strsec, PROCINFO[PROC_CPUUSER].STRING);
			cpuuser=atol(strsec);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUUSER].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUUSER].LENGTH=strlen(PROCINFO[PROC_CPUUSER].STRING);
			// PROC_CPUNICE
			strcpy (strsec, PROCINFO[PROC_CPUNICE].STRING);
			cpunice=atol(strsec);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUNICE].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUNICE].LENGTH=strlen(PROCINFO[PROC_CPUNICE].STRING);
			// PROC_CPUSYS
			strcpy (strsec, PROCINFO[PROC_CPUSYS].STRING);
			cpusys=atol(strsec);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUSYS].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUSYS].LENGTH=strlen(PROCINFO[PROC_CPUSYS].STRING);
			// PROC_CPUIDLE
			cpuidle = (((UPTIME*100)*(NUM_CPU+1)) - (cpuuser + cpunice + cpusys));
			totalsec = cpuidle / 100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUIDLE].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUIDLE].LENGTH=strlen(PROCINFO[PROC_CPUIDLE].STRING);
		}
		if (strcmp(field, "disk")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISK].STRING);
			PROCINFO[PROC_DISK].LENGTH=strlen(PROCINFO[PROC_DISK].STRING);
		}
		if (strcmp(field, "disk_rio")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKRIO].STRING);
			PROCINFO[PROC_DISKRIO].LENGTH=strlen(PROCINFO[PROC_DISKRIO].STRING);
		}
		if (strcmp(field, "disk_wio")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKWIO].STRING);
			PROCINFO[PROC_DISKWIO].LENGTH=strlen(PROCINFO[PROC_DISKWIO].STRING);
		}
		if (strcmp(field, "disk_rblk")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKRBLK].STRING);
			PROCINFO[PROC_DISKRBLK].LENGTH=strlen(PROCINFO[PROC_DISKRBLK].STRING);
		}
		if (strcmp(field, "disk_wblk")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKWBLK].STRING);
			PROCINFO[PROC_DISKWBLK].LENGTH=strlen(PROCINFO[PROC_DISKWBLK].STRING);
		}
		if (strcmp(field, "page")==0) {
			sscanf (tmpstring, "%s %s %s", field, PROCINFO[PROC_PAGE_IN].STRING, PROCINFO[PROC_PAGE_OUT].STRING);
			PROCINFO[PROC_PAGE_IN].LENGTH=strlen(PROCINFO[PROC_PAGE_IN].STRING);
			PROCINFO[PROC_PAGE_OUT].LENGTH=strlen(PROCINFO[PROC_PAGE_OUT].STRING);
		}
		if (strcmp(field, "swap")==0) {
			sscanf (tmpstring, "%s %s %s", field, PROCINFO[PROC_SWAP_IN].STRING, PROCINFO[PROC_SWAP_OUT].STRING);
			PROCINFO[PROC_SWAP_IN].LENGTH=strlen(PROCINFO[PROC_SWAP_IN].STRING);
			PROCINFO[PROC_SWAP_OUT].LENGTH=strlen(PROCINFO[PROC_SWAP_OUT].STRING);
		}
		if (strcmp(field, "intr")==0) {
		}
		if (strcmp(field, "ctxt")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_CTXT].STRING);
			PROCINFO[PROC_CTXT].LENGTH=strlen(PROCINFO[PROC_CTXT].STRING);
		}
		if (strcmp(field, "btime")==0) {
			sscanf (tmpstring, "%s %ld", field, &BOOTTIME);
			strcpy (tmpstring, ctime(&BOOTTIME));
			killcrlf (tmpstring, strlen(tmpstring));
			strcpy(PROCINFO[PROC_BTIME].STRING, tmpstring);
			PROCINFO[PROC_BTIME].LENGTH=strlen(PROCINFO[PROC_BTIME].STRING);
		}
		if (strcmp(field, "processes")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_PROCESS].STRING);
			PROCINFO[PROC_PROCESS].LENGTH=strlen(PROCINFO[PROC_PROCESS].STRING);
		}
	} while (0==0);
	// Cleanup
	fclose (filep);
	return;
}

// handlestatv22() Function
// 	code specific to the Linux Kernel 2.2
void handlestatv22 () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	char field [INPUT_STRING];
	char strsec [INPUT_STRING];
	long int totalsec;
	long int numsec;
	long int nummin;
	long int numhour;
	int i=0;

	sprintf (filename, "%sstat", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		sscanf (tmpstring, "%s", field);
		if (strcmp(field, "cpu")==0) {
			sscanf (tmpstring, "%s %s %s %s %s", field, PROCINFO[PROC_CPUUSER].STRING, PROCINFO[PROC_CPUNICE].STRING, PROCINFO[PROC_CPUSYS].STRING, PROCINFO[PROC_CPUIDLE].STRING);
			// PROC_CPUUSER
			strcpy (strsec, PROCINFO[PROC_CPUUSER].STRING);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUUSER].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUUSER].LENGTH=strlen(PROCINFO[PROC_CPUUSER].STRING);
			// PROC_CPUNICE
			strcpy (strsec, PROCINFO[PROC_CPUNICE].STRING);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUNICE].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUNICE].LENGTH=strlen(PROCINFO[PROC_CPUNICE].STRING);
			// PROC_CPUSYS
			strcpy (strsec, PROCINFO[PROC_CPUSYS].STRING);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUSYS].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUSYS].LENGTH=strlen(PROCINFO[PROC_CPUSYS].STRING);
			// PROC_CPUIDLE
			strcpy (strsec, PROCINFO[PROC_CPUIDLE].STRING);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (PROCINFO[PROC_CPUIDLE].STRING, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			PROCINFO[PROC_CPUIDLE].LENGTH=strlen(PROCINFO[PROC_CPUIDLE].STRING);
			// Single-CPU Computers
			if (NUM_CPU==0) {
				strcpy (CPU[0].USER, PROCINFO[PROC_CPUUSER].STRING);
				strcpy (CPU[0].NICE, PROCINFO[PROC_CPUNICE].STRING);
				strcpy (CPU[0].SYS,  PROCINFO[PROC_CPUSYS].STRING);
				strcpy (CPU[0].IDLE, PROCINFO[PROC_CPUIDLE].STRING);
			}
		}
		if ((strncmp(field, "cpu", 3)==0) & (isdigit(field[3])!=0)) {
			sscanf (field, "%*3s%d", &i);
			sscanf (tmpstring, "%s %s %s %s %s", field, CPU[i].USER, CPU[i].NICE, CPU[i].SYS, CPU[i].IDLE);
			// CPU.USER
			strcpy (strsec, CPU[i].USER);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (CPU[i].USER, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			// CPU.NICE
			strcpy (strsec, CPU[i].NICE);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (CPU[i].NICE, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			// CPU.SYS
			strcpy (strsec, CPU[i].SYS);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (CPU[i].SYS, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
			// CPU.IDLE
			strcpy (strsec, CPU[i].IDLE);
			totalsec=atol(strsec)/100;
			numhour=totalsec/3600;
			nummin=(totalsec-(numhour*3600))/60;
			numsec=(totalsec-(numhour*3600)-(nummin*60));
			sprintf (CPU[i].IDLE, "%7ld Seconds (%3ld:%2ld:%2ld)", totalsec, numhour, nummin, numsec);
		}
		if (strcmp(field, "disk")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISK].STRING);
			PROCINFO[PROC_DISK].LENGTH=strlen(PROCINFO[PROC_DISK].STRING);
		}
		if (strcmp(field, "disk_rio")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKRIO].STRING);
			PROCINFO[PROC_DISKRIO].LENGTH=strlen(PROCINFO[PROC_DISKRIO].STRING);
		}
		if (strcmp(field, "disk_wio")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKWIO].STRING);
			PROCINFO[PROC_DISKWIO].LENGTH=strlen(PROCINFO[PROC_DISKWIO].STRING);
		}
		if (strcmp(field, "disk_rblk")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKRBLK].STRING);
			PROCINFO[PROC_DISKRBLK].LENGTH=strlen(PROCINFO[PROC_DISKRBLK].STRING);
		}
		if (strcmp(field, "disk_wblk")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_DISKWBLK].STRING);
			PROCINFO[PROC_DISKWBLK].LENGTH=strlen(PROCINFO[PROC_DISKWBLK].STRING);
		}
		if (strcmp(field, "page")==0) {
			sscanf (tmpstring, "%s %s %s", field, PROCINFO[PROC_PAGE_IN].STRING, PROCINFO[PROC_PAGE_OUT].STRING);
			PROCINFO[PROC_PAGE_IN].LENGTH=strlen(PROCINFO[PROC_PAGE_IN].STRING);
			PROCINFO[PROC_PAGE_OUT].LENGTH=strlen(PROCINFO[PROC_PAGE_OUT].STRING);
		}
		if (strcmp(field, "swap")==0) {
			sscanf (tmpstring, "%s %s %s", field, PROCINFO[PROC_SWAP_IN].STRING, PROCINFO[PROC_SWAP_OUT].STRING);
			PROCINFO[PROC_SWAP_IN].LENGTH=strlen(PROCINFO[PROC_SWAP_IN].STRING);
			PROCINFO[PROC_SWAP_OUT].LENGTH=strlen(PROCINFO[PROC_SWAP_OUT].STRING);
		}
		if (strcmp(field, "intr")==0) {
		}
		if (strcmp(field, "ctxt")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_CTXT].STRING);
			PROCINFO[PROC_CTXT].LENGTH=strlen(PROCINFO[PROC_CTXT].STRING);
		}
		if (strcmp(field, "btime")==0) {
			sscanf (tmpstring, "%s %ld", field, &BOOTTIME);
			strcpy (tmpstring, ctime(&BOOTTIME));
			killcrlf (tmpstring, strlen(tmpstring));
			strcpy(PROCINFO[PROC_BTIME].STRING, tmpstring);
			PROCINFO[PROC_BTIME].LENGTH=strlen(PROCINFO[PROC_BTIME].STRING);
		}
		if (strcmp(field, "processes")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_PROCESS].STRING);
			PROCINFO[PROC_PROCESS].LENGTH=strlen(PROCINFO[PROC_PROCESS].STRING);
		}
	} while (0==0);
	// Cleanup
	fclose (filep);
	return;
}

// handlememinfo() Function
void handlememinfo () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	char field [INPUT_STRING];

	sprintf (filename, "%smeminfo", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		sscanf (tmpstring, "%s", field);
		if (strcmp(field, "MemTotal:")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_MEMTOTAL].STRING);
			PROCINFO[PROC_MEMTOTAL].LENGTH=strlen(PROCINFO[PROC_MEMTOTAL].STRING);
		}
		if (strcmp(field, "MemFree:")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_MEMFREE].STRING);
			PROCINFO[PROC_MEMFREE].LENGTH=strlen(PROCINFO[PROC_MEMFREE].STRING);
		}
		if (strcmp(field, "MemShared:")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_MEMSHARED].STRING);
			PROCINFO[PROC_MEMSHARED].LENGTH=strlen(PROCINFO[PROC_MEMSHARED].STRING);
		}
		if (strcmp(field, "Buffers:")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_BUFFERS].STRING);
			PROCINFO[PROC_BUFFERS].LENGTH=strlen(PROCINFO[PROC_BUFFERS].STRING);
		}
		if (strcmp(field, "Cached:")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_CACHED].STRING);
			PROCINFO[PROC_CACHED].LENGTH=strlen(PROCINFO[PROC_CACHED].STRING);
		}
		if (strcmp(field, "SwapTotal:")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_SWAPTOTAL].STRING);
			PROCINFO[PROC_SWAPTOTAL].LENGTH=strlen(PROCINFO[PROC_SWAPTOTAL].STRING);
		}
		if (strcmp(field, "SwapFree:")==0) {
			sscanf (tmpstring, "%s %s", field, PROCINFO[PROC_SWAPFREE].STRING);
			PROCINFO[PROC_SWAPFREE].LENGTH=strlen(PROCINFO[PROC_SWAPFREE].STRING);
		}
	} while (0==0);
	// Cleanup
	fclose (filep);
	return;
}

// handlemounts() Function
void handlemounts () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	char device [INPUT_STRING];
	char mountpoint [INPUT_STRING];
	char filesystem [INPUT_STRING];
	char mode [INPUT_STRING];
	int c=0;

	sprintf (filename, "%smounts", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		sscanf (tmpstring, "%s %s %s %s", device, mountpoint, filesystem, mode);
		sprintf (MOUNTS[c].STRING, "%-20s%-20s%-10s%-10s", device, mountpoint, filesystem, mode);
		MOUNTS[c].LENGTH=strlen(MOUNTS[c].STRING);
		c++;
	} while (0==0);
	NUM_MOUNTS=c-1;
	// Cleanup
	fclose (filep);
	return;
}

// handleswaps() Function
void handleswaps () {
	FILE *filep;
	char tmpstring [INPUT_STRING];
	char filename [INPUT_STRING];
	char size [INPUT_STRING];
	char used [INPUT_STRING];
	int c=0;

	if (KERNEL_VERSION==TWOPOINTZERO) return;
	sprintf (filename, "%sswaps", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		sscanf (tmpstring, "%s %*s %s %s %*s", filename, size, used);
		if (strcmp(filename, "Filename")==0) continue;
		sprintf (SWAPS[c].STRING, "%-20s%-10s%-10s", filename, size, used);
		SWAPS[c].LENGTH=strlen(SWAPS[c].STRING);
		c++;
	} while (0==0);
	NUM_SWAPS=c-1;
	// Cleanup
	fclose (filep);
	return;
}

// handleprocesses() Function
void handleprocesses () {
	DIR *procdir;
	struct dirent *dirname;
	char tmpstring [INPUT_STRING];
	int i=0;
	int c=0;
	char filename [INPUT_STRING];
	char dirstr [INPUT_STRING];
	FILE *filep;
	int space;
	time_t starttime;
	time_t curtime;
	long int totaltime;
	long int numday;
	long int numhour;
	long int nummin;
	long int numsec;
	char tty [INPUT_STRING];

	// Open Directory
	procdir=opendir(PROCLOC);
	if (procdir==NULL) return;
	// Read Entries
	do {
		dirname=readdir(procdir);
		if (dirname==NULL) break;
		strcpy (tmpstring, dirname->d_name);
		if (isdigit(tmpstring[0])) {
			PROC[c].PID=atoi(tmpstring);
			c++;
		}
	} while (0==0);
	NUM_PROC=c-1;
	// Close Directory
	closedir (procdir);
	// Retrieve Basic Information on Each Process
	for (i=0; i<=NUM_PROC; i++) {
		// cmdline
		sprintf (filename, "%s%ld/cmdline", PROCLOC, PROC[i].PID);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fgets (PROC[i].CMDLINE, STORAGE_STRING, filep);
			fclose (filep);
		}
		// cwd
		sprintf (filename, "%s%ld/cwd", PROCLOC, PROC[i].PID);
		readlink (filename, PROC[i].CWD, STORAGE_STRING);
		// exe
		sprintf (filename, "%s%ld/exe", PROCLOC, PROC[i].PID);
		readlink (filename, PROC[i].EXE, STORAGE_STRING);
		// stat
		sprintf (filename, "%s%ld/stat", PROCLOC, PROC[i].PID);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fscanf (filep, "%*d %s %s %d %d %d %s %d %*u %*u", PROC[i].NAME, PROC[i].STATE, &PROC[i].PPID, &PROC[i].PGRP, &PROC[i].SESSION, tty, &PROC[i].TPGID);
			fscanf (filep, "%*u %*u %*u %d %d %*d %*d %*d %d %*u %*u %ld %u %*u %*u %u %u", &PROC[i].UTIME, &PROC[i].STIME, &PROC[i].PRIORITY, &starttime, &PROC[i].VSIZE, &PROC[i].STARTCODE, &PROC[i].ENDCODE);
			fscanf (filep, "%*u %*u %*u %*u %*u %*u %*u %*u %*u %*u");
			fclose (filep);
			// Drop Parentheses on Name
			rstring (PROC[i].NAME, 1, PROC[i].NAME);
			PROC[i].NAME[strlen(PROC[i].NAME)-1]=0;
			// Convert State
			if (strcmp(PROC[i].STATE, "R")) sprintf(PROC[i].STATE, "Running");
			if (strcmp(PROC[i].STATE, "S")) sprintf(PROC[i].STATE, "Uninterruptible");
			if (strcmp(PROC[i].STATE, "D")) sprintf(PROC[i].STATE, "Swapping");
			if (strcmp(PROC[i].STATE, "Z")) sprintf(PROC[i].STATE, "Zombie");
			if (strcmp(PROC[i].STATE, "T")) sprintf(PROC[i].STATE, "Stopped");
			// Fix Priority Level
			PROC[i].PRIORITY = PROC[i].PRIORITY + 1;
			// Fix Start Time
			starttime = (starttime / 100) + BOOTTIME;
			strcpy (PROC[i].STARTTIME, ctime(&starttime));
			killcrlf (PROC[i].STARTTIME, strlen(PROC[i].STARTTIME));
			// Fix TTY
			if (strcmp(tty, "0")==0)
				sprintf (PROC[i].TTY, "unknown (%s)", tty);
			else
				sprintf (PROC[i].TTY, "tty%d (%s)", (atoi(tty)-1024), tty);
			// Create Total Time
			time (&curtime);
			totaltime = curtime - starttime;
			numday=totaltime/86400;
			numhour=(totaltime-(numday*86400))/3600;
			nummin=(totaltime-(numday*86400)-(numhour*3600))/60;
			numsec=(totaltime-(numday*86400)-(numhour*3600)-(nummin*60));
			sprintf (PROC[i].TOTALTIME, "%ld Days  %ld Hours  %ld Minutes  %ld Seconds", numday, numhour, nummin, numsec);
		}
		// files in use
		c=0;
		sprintf (filename, "%s%ld/fd", PROCLOC, PROC[i].PID);
		procdir=opendir(filename);
		if (procdir!=NULL) {
			do {
				fillstring (tmpstring, 0, INPUT_STRING);
				dirname=readdir(procdir);
				if (dirname==NULL) break;
				strcpy (tmpstring, dirname->d_name);
				if (strcmp(tmpstring, ".")==0) continue;
				if (strcmp(tmpstring, "..")==0) continue;
				if (strcmp(tmpstring, "0")==0) strcpy(tmpstring, "(stdin)");
				if (strcmp(tmpstring, "1")==0) strcpy(tmpstring, "(stdout)");
				if (strcmp(tmpstring, "2")==0) strcpy(tmpstring, "(stderr)");
				sprintf (PROC[i].FILES[c], "%8s : ", tmpstring);
				sprintf (dirstr, "%s/%s", filename, dirname->d_name);
				fillstring (tmpstring, 0, INPUT_STRING);
				readlink (dirstr, tmpstring, INPUT_STRING);
				space=instr(tmpstring, ' ');
				if (space==0) strcat (PROC[i].FILES[c], tmpstring);
				if (space!=0) strncat (PROC[i].FILES[c], tmpstring, space);
				c++;
			} while (0==0);
			closedir (procdir);
		}
		PROC[i].NUM_FILES=c-1;
	}
	// Return
	return;
}

// handlescsi() Function
void handlescsi () {
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	FILE *filep;
	DIR *scsidir;
	struct dirent *dirname;
	int c=-1;

	// Open /proc/scsi
	sprintf (filename, "%sscsi/scsi", PROCLOC);
	filep=fopen(filename, "r");
	if (filep!=NULL) {
		do {
			fgets (tmpstring, INPUT_STRING, filep);
			if (feof(filep)!=0) break;
			killcrlf (tmpstring, strlen(tmpstring));
			if (strncmp(tmpstring, "Attached devices:", 17)==0) continue;
			if (strncmp(tmpstring, "Host:", 5)==0) {
				c++;
				sscanf (tmpstring, "%*5s %5s %*8s %2d %*3s %2d %*4s %2d", SCSIDEV[c].HOST, &SCSIDEV[c].CHANNEL, &SCSIDEV[c].ID, &SCSIDEV[c].LUN);
			}
			if (strncmp(tmpstring, "  Vendor:", 9)==0) {
				trim (SCSIDEV[c].VENDOR);
				strncpy (SCSIDEV[c].MODEL, tmpstring + 26, 17);
				trim (SCSIDEV[c].MODEL);
				strcpy (SCSIDEV[c].REV, tmpstring + 48);
			}
			if (strncmp(tmpstring, "  Type:", 7)==0) {
				strncpy (SCSIDEV[c].TYPE, tmpstring + 7, 33);
				trim (SCSIDEV[c].TYPE);
				strcpy (SCSIDEV[c].ANSIREV, tmpstring + 62);
				trim (SCSIDEV[c].ANSIREV);
			}
		} while (0==0);
		NUM_SCSIDEV=c;
		fclose (filep);
	}
	// Find SCSI Drivers
	c=0;
	sprintf (filename, "%sscsi", PROCLOC);
	scsidir=opendir(filename);
	if (scsidir==NULL) return;
	// Read Entries
	do {
		dirname=readdir(scsidir);
		if (dirname==NULL) break;
		strcpy (tmpstring, dirname->d_name);
		if (strcmp(tmpstring, "scsi")==0) continue;
		if (tmpstring[0]=='.') continue;
		strcpy (SCSIDRV[c].NAME, tmpstring);
		c++;
	} while (0==0);
	NUM_SCSIDRV=c-1;
	// Read Descriptions
	for (c=0; c<=NUM_SCSIDRV; c++) {
		sprintf (filename, "%sscsi/%s/0", PROCLOC, SCSIDRV[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fgets (SCSIDRV[c].DESC, STORAGE_STRING, filep);
			killcrlf (SCSIDRV[c].DESC, strlen(SCSIDRV[c].DESC));
			fclose (filep);
		}
	}
	// Return
	return;
}

// handletty() Function
void handletty () {
	char filename [INPUT_STRING];
	FILE *filep;
	char tmpstring [INPUT_STRING];
	int c=0;

	// Open and Process /proc/tty/drivers
	sprintf (filename, "%stty/drivers", PROCLOC);
	filep=fopen(filename, "r");
	if (filep!=NULL) {
		do {
			fgets (tmpstring, INPUT_STRING, filep);
			if (feof(filep)!=0) break;
			sscanf (tmpstring, "%20s %13s %3s %7s %s", TTYDRIVERS[c].DRIVER, TTYDRIVERS[c].NAME, TTYDRIVERS[c].MAJOR, TTYDRIVERS[c].RANGE, TTYDRIVERS[c].TYPE);
			c++;
		} while (0==0);
		NUM_TTYDRIVERS=c-1;
		fclose (filep);
	}
	// Open and Process /proc/tty/ldiscs
	c=0;
	sprintf (filename, "%stty/ldiscs", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		sscanf (tmpstring, "%10s %2d", TTYLDISCS[c].NAME, &TTYLDISCS[c].COUNT);
		c++;
	} while (0==0);
	NUM_TTYLDISCS=c-1;
	fclose (filep);
	// Return
	return;
}

// handlesound() Function
void handlesound () {
	char filename [INPUT_STRING];
	FILE *filep;
	char tmpstring [INPUT_STRING];
	int c=0;
	int col=0;
	char type [INPUT_STRING];
	char desc [INPUT_STRING];

	// Open /proc/sound
	sprintf (filename, "%ssound", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	// Find PROC_SOUNDDRV
	fgets (tmpstring, INPUT_STRING, filep);
	killcrlf (tmpstring, strlen(tmpstring));
	strcpy (PROCINFO[PROC_SOUNDDRV].STRING, tmpstring);
	PROCINFO[PROC_SOUNDDRV].LENGTH=strlen(PROCINFO[PROC_SOUNDDRV].STRING);
	// Enter Loop
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		if (strncmp(tmpstring, "Installed drivers:", 18)==0) {
			c=0;
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				killcrlf (tmpstring, strlen(tmpstring));
				col=instr(tmpstring, ':');
				if (col==-1) break;
				fillstring (type, 0, INPUT_STRING);
				strncpy (type, tmpstring, col);
				rstring (tmpstring, col+1, desc);
				sprintf (SNDDRV[c].STRING, "%-10s %s", type, desc);
				SNDDRV[c].LENGTH=strlen(SNDDRV[c].STRING);
				c++;
			} while (0==0);
			NUM_SNDDRV=c-1;
			continue;
		}
		if (strncmp(tmpstring, "Card config:", 12)==0) {
			c=0;
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (strlen(tmpstring)==1) break;
				killcrlf (tmpstring, strlen(tmpstring));
				strcpy (SNDCONF[c].STRING, tmpstring);
				SNDCONF[c].LENGTH=strlen(SNDCONF[c].STRING);
				c++;
			} while (0==0);
			NUM_SNDCONF=c-1;
			continue;
		}
		if (strncmp(tmpstring, "Audio devices:", 14)==0) {
			c=0;
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (strlen(tmpstring)==1) break;
				killcrlf (tmpstring, strlen(tmpstring));
				strcpy (SNDAUDIO[c].STRING, tmpstring);
				SNDAUDIO[c].LENGTH=strlen(SNDAUDIO[c].STRING);
				c++;
			} while (0==0);
			NUM_SNDAUDIO=c-1;
			continue;
		}
		if (strncmp(tmpstring, "Synth devices:", 14)==0) {
			c=0;
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (strlen(tmpstring)==1) break;
				killcrlf (tmpstring, strlen(tmpstring));
				strcpy (SNDSYNTH[c].STRING, tmpstring);
				SNDSYNTH[c].LENGTH=strlen(SNDSYNTH[c].STRING);
				c++;
			} while (0==0);
			NUM_SNDSYNTH=c-1;
			continue;
		}
		if (strncmp(tmpstring, "Midi devices:", 13)==0) {
			c=0;
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (strlen(tmpstring)==1) break;
				killcrlf (tmpstring, strlen(tmpstring));
				strcpy (SNDMIDI[c].STRING, tmpstring);
				SNDMIDI[c].LENGTH=strlen(SNDMIDI[c].STRING);
				c++;
			} while (0==0);
			NUM_SNDMIDI=c-1;
			continue;
		}
		if (strncmp(tmpstring, "Timers:", 7)==0) {
			c=0;
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (strlen(tmpstring)==1) break;
				killcrlf (tmpstring, strlen(tmpstring));
				strcpy (SNDTIMERS[c].STRING, tmpstring);
				SNDTIMERS[c].LENGTH=strlen(SNDTIMERS[c].STRING);
				c++;
			} while (0==0);
			NUM_SNDTIMERS=c-1;
			continue;
		}
		if (strncmp(tmpstring, "Mixers:", 7)==0) {
			c=0;
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (feof(filep)!=0) break;
				if (strlen(tmpstring)==1) break;
				killcrlf (tmpstring, strlen(tmpstring));
				strcpy (SNDMIXERS[c].STRING, tmpstring);
				SNDMIXERS[c].LENGTH=strlen(SNDMIXERS[c].STRING);
				c++;
			} while (0==0);
			NUM_SNDMIXERS=c-1;
			continue;
		}
	} while (0==0);
	fclose (filep);
	// Return
	return;
}

// handlemtrr() Function
void handlemtrr () {
	char filename [INPUT_STRING];
	FILE *filep;
	char tmpstring [INPUT_STRING];
	int c=0;
	int comma=0;

	NUM_MTRR=-1;
	// Open /proc/mtrr
	sprintf (filename, "%smtrr", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		sscanf (tmpstring, "%*3s%2d%*1s %*5s%10s %*1s%4d%*4s %*5s%4d%*3s %s %*6s%d", &MTRR[c].REG, MTRR[c].OFFHEX, &MTRR[c].OFFMB, &MTRR[c].SIZE, tmpstring, &MTRR[c].COUNT);
		comma=instr(tmpstring, ',');
		strncpy (MTRR[c].OPTION, tmpstring, comma);
		c++;
	} while (0==0);
	fclose (filep);
	NUM_MTRR=c-1;
	// Return;
	return;
}

// handleide() Function
void handleide () {
	FILE *filep;
	DIR *dirp;
	DIR *dirq;
	DIR *dirr;
	struct dirent *dir;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	char idename [INPUT_STRING];
	char hdname [INPUT_STRING];
	int c=0;
	int space=0;

	// IDE Drivers
	sprintf (filename, "%side/drivers", PROCLOC);
	filep=fopen(filename, "r");
	if (filep!=NULL) {
		do {
			fgets (tmpstring, INPUT_STRING, filep);
			if (feof(filep)!=0) break;
			space=instr(tmpstring, ' ');
			strncpy (IDEDRV[c].NAME, tmpstring, space);
			rstring (tmpstring, space+8, IDEDRV[c].VERSION);
			killcrlf (IDEDRV[c].VERSION, strlen(IDEDRV[c].VERSION));
			c++;
		} while (0==0);
		NUM_IDEDRV=c-1;
		fclose (filep);
	}
	// IDE Devices
	c=0;
	sprintf (filename, "%side/", PROCLOC);
	dirp=opendir(filename);
	if (dirp==NULL) return;
	do {
		dir=readdir(dirp);
		if (dir==NULL) break;
		strcpy (idename, dir->d_name);
		// Test for Directory
		if (strncmp(idename, ".", 1)==0) continue;
		sprintf (filename, "%side/%s/", PROCLOC, idename);
		dirq=opendir(filename);
		if (dirq==NULL) continue;
		do {
			dir=readdir(dirq);
			if (dir==NULL) break;
			strcpy (hdname, dir->d_name);
			if (strncmp(hdname, ".", 1)==0) continue;
			sprintf (filename, "%side/%s/%s", PROCLOC, idename, hdname);
			dirr=opendir(filename);
			if (dirr==NULL) continue;
			strcpy (IDEDEV[c].NAME, hdname);
			strcpy (IDEDEV[c].PARENT, idename);
			c++;
			closedir (dirr);
		} while (0==0);
		closedir (dirq);
	} while (0==0);
	closedir (dirp);
	NUM_IDEDEV=c-1;
	// Information on Each Device
	for (c=0; c<=NUM_IDEDEV; c++) {
		// driver
		sprintf (filename, "%side/%s/%s/driver", PROCLOC, IDEDEV[c].PARENT, IDEDEV[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fgets (IDEDEV[c].DRIVER, STORAGE_STRING, filep);
			killcrlf (IDEDEV[c].DRIVER, strlen(IDEDEV[c].DRIVER));
			fclose (filep);
		}
		// media
		sprintf (filename, "%side/%s/%s/media", PROCLOC, IDEDEV[c].PARENT, IDEDEV[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fgets (IDEDEV[c].MEDIA, STORAGE_STRING, filep);
			killcrlf (IDEDEV[c].MEDIA, strlen(IDEDEV[c].MEDIA));
			fclose (filep);
		}
		// model
		sprintf (filename, "%side/%s/%s/model", PROCLOC, IDEDEV[c].PARENT, IDEDEV[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fgets (IDEDEV[c].MODEL, STORAGE_STRING, filep);
			killcrlf (IDEDEV[c].MODEL, strlen(IDEDEV[c].MODEL));
			fclose (filep);
		}
		// settings
		sprintf (filename, "%side/%s/%s/settings", PROCLOC, IDEDEV[c].PARENT, IDEDEV[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (feof(filep)!=0) break;
				if (strncmp(tmpstring, "name", 4)==0) continue;
				if (strncmp(tmpstring, "----", 4)==0) continue;
				if (strncmp(tmpstring, "bios_cyl", 7)==0)
					sscanf (tmpstring, "%*24s %d", &IDEDEV[c].CYL);
				if (strncmp(tmpstring, "bios_head", 8)==0)
					sscanf (tmpstring, "%*24s %d", &IDEDEV[c].HEAD);
				if (strncmp(tmpstring, "bios_sect", 8)==0)
					sscanf (tmpstring, "%*24s %d", &IDEDEV[c].SECT);
			} while (0==0);
		}	
	}
	// Return
	return;
}

// handleparport() Function
void handleparport () {
	DIR *dirp;
	struct dirent *dirlist;
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int c=0;
	int len;

	// Open and Read /proc/parport
	sprintf (filename, "%sparport", PROCLOC);
	dirp=opendir(filename);
	if (dirp==NULL) return;
	do {
		dirlist=readdir(dirp);
		if (dirlist==NULL) break;
		strcpy (tmpstring, dirlist->d_name);
		if (strncmp(tmpstring, ".", 1)==0) continue;
		strcpy (PARPORT[c].NAME, tmpstring);
		// autoprobe
		sprintf (filename, "%sparport/%s/autoprobe", PROCLOC, PARPORT[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fgets(tmpstring, INPUT_STRING, filep);
			len=killcrlf(tmpstring, strlen(tmpstring));
			strncpy (PARPORT[c].AUTOPROBE, tmpstring, len);
			fclose (filep);
		}
		// devices
		sprintf (filename, "%sparport/%s/devices", PROCLOC, PARPORT[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			fgets (tmpstring, INPUT_STRING, filep);
			killcrlf (tmpstring, strlen(tmpstring));
			strcpy (PARPORT[c].DEVICES, tmpstring+1);
			fclose (filep);
		}
		// hardware
		sprintf (filename, "%sparport/%s/hardware", PROCLOC, PARPORT[c].NAME);
		filep=fopen(filename, "r");
		if (filep!=NULL) {
			do {
				fgets (tmpstring, INPUT_STRING, filep);
				if (feof(filep)!=0) break;
				killcrlf (tmpstring, strlen(tmpstring));
				if (strncmp(tmpstring, "base:", 5)==0)
					sscanf (tmpstring, "base:\t%s", PARPORT[c].IOPORT);
				if (strncmp(tmpstring, "irq:", 4)==0)
					sscanf (tmpstring, "irq:\t%s", PARPORT[c].IRQ);
				if (strncmp(tmpstring, "dma:", 4)==0)
					sscanf (tmpstring, "dma:\t%s", PARPORT[c].DMA);
				if (strncmp(tmpstring, "modes:", 6)==0)
					sscanf (tmpstring, "modes:\t%s", PARPORT[c].PORTTYPE);
			} while (0==0);
			fclose (filep);
		}
		c++;
	} while (0==0);
	NUM_PARPORT=c-1;
	closedir (dirp);
	// Return
	return;
}

// handlemisc() Function
void handlemisc () {
	FILE *filep;
	char filename [INPUT_STRING];
	char tmpstring [INPUT_STRING];
	int c=0;

	// Open and Handle /proc/misc
	sprintf (filename, "%smisc", PROCLOC);
	filep=fopen(filename, "r");
	if (filep==NULL) return;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		killcrlf (tmpstring, strlen(tmpstring));
		strcpy (MISC[c].STRING, tmpstring);
		c++;
	} while (0==0);
	fclose (filep);
	NUM_MISC=c-1;
	// Return
	return;
}

// readprocline() Function
int readprocline (char *shortname, struct STRING_ARRAY dataarray[READPROCLINE_COUNT]) {
	char filename [1024];
	FILE *filep;
	char tmpstring [INPUT_STRING];
	int c=0;

	sprintf (filename, "%s%s", PROCLOC, shortname);
	filep=fopen(filename, "r");
	if (filep==NULL) return 0;
	do {
		fgets (tmpstring, INPUT_STRING, filep);
		if (feof(filep)!=0) break;
		if (strlen(tmpstring)==1) continue;
		killcrlf (tmpstring, strlen(tmpstring));
		strcpy (dataarray[c].STRING, tmpstring);
		dataarray[c].LENGTH=strlen(dataarray[c].STRING);
		c++;
	} while (0==0);
	c--;
	fclose (filep);
	return c;
}

// readproc() Function
int readproc (char *shortname, char *string) {
	char filename[1024];
	FILE *filep;
	char tmpstring [INPUT_STRING];
	int tmplength;
	int sizeread;

	sprintf (filename, "%s%s", PROCLOC, shortname);
	filep=fopen(filename, "r");
	if (filep==NULL) {
		tmpstring[0]=0;
		tmplength=strlen(tmpstring);
	}
	else {
		sizeread=fgetstr(filep, tmpstring, INPUT_STRING-1);
		tmplength=sizeread;
		fclose (filep);
	}
	strcpy(string, tmpstring);
	return tmplength;
}

