/*
    si - Version 0.6 - By Joe Turgeon (jturgeon@isd.net)
    main.c

    si - System Information Viewer
    Copyright (C) 1999  Joe Turgeon (jturgeon@isd.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "main.h"


// Global Variables
// File Pointers
FILE *fstdout;
FILE *pipes;
// Kernel Variables
int KERNEL_VERSION;
const int TWOPOINTTHREE = 23;
const int TWOPOINTTWO   = 22;
const int TWOPOINTONE   = 21;
const int TWOPOINTZERO  = 20;
// Command-Line Argument Variables
int OPT_SHOW_SYSTEM	= 1;
int OPT_SHOW_HARDWARE   = 1;
int OPT_SHOW_MEMORY	= 1;
int OPT_SHOW_KERNEL	= 1;
int OPT_SHOW_FILESYS	= 1;
int OPT_SHOW_NETWORK	= 1;
int OPT_SHOW_PROCESS	= 1;
int OPT_OUTPUT_PAGER	= 0;
int OPT_OUTPUT_FILE	= 0;
char OUTPUT_FILE [INPUT_STRING];


// main() Function
int main (int argc, char **argv, char **argp) {
	int retval;

	// Set Default Output
	fstdout = stdout;
	// Banner
	fprintf (fstdout, "si - Version 0.6 - By Joe Turgeon (jturgeon@isd.net)\n");
	// Handle Command-Line Arguments
	if (command_line(argc, argv, argp)!=0) {
		fprintf (fstdout, "\n");
		fprintf (fstdout, "Usage:  %s  [-h | -?]  [-o smkfnp]\n", argv[0]);
		fprintf (fstdout, "\n");
		fprintf (fstdout, "Command Line Arguments:\n");
		fprintf (fstdout, "  -h          Display This Help Screen\n");
		fprintf (fstdout, "  -?                (same as -h)      \n");
		fprintf (fstdout, "  -f <file>   Send Output To File\n");
		fprintf (fstdout, "  -o          Display Specified Sections  (default: all)\n");
		fprintf (fstdout, "              s   System Section\n");
		fprintf (fstdout, "              h   Hardware Section\n");
		fprintf (fstdout, "              m   Memory Section\n");
		fprintf (fstdout, "              k   Kernel Section\n");
		fprintf (fstdout, "              f   File-System Section\n");
		fprintf (fstdout, "              n   Network Section\n");
		fprintf (fstdout, "              p   Process Section\n");
		fprintf (fstdout, "\n");
		exit_program (0);
	}
	// Run Through Pager or Send Output to File
	if (OPT_OUTPUT_FILE==1) {
		fprintf (fstdout, "si:  Sending Output to:  %s\n", OUTPUT_FILE);
		fstdout=fopen(OUTPUT_FILE, "w");
		fprintf (fstdout, "si - Version 0.6 - By Joe Turgeon (jturgeon@isd.net)\n");
	}
	if ((isatty(1)) && (OPT_OUTPUT_FILE==0)) {
		fprintf (fstdout, "si:  Using Pager...\n");
		pipes=popen("less", "w");
		fstdout=pipes;
		OPT_OUTPUT_PAGER=1;
		fprintf (fstdout, "si - Version 0.6 - By Joe Turgeon (jturgeon@isd.net)\n");
	}
	// Find Linux Kernel Version
	retval=kernelversion(0);
	if (retval!=0) exit_program (1);
	// Load Information
	retval=loadinfo();
	if (retval!=0) exit_program (1);
	// Dump Information
	display_dump();
	// Exit Program / Return
	exit_program (0);
	return 0;
}

// exit_program() Function
void exit_program (int retval) {
	// Clean-Up File Handlers
	if (OPT_OUTPUT_FILE==1) fclose(fstdout);
	if (OPT_OUTPUT_PAGER==1) pclose (pipes);
	fstdout=stdout;
	// Exit
	fprintf (fstdout, "si:  Clean Exit\n");
	exit (retval);
	// Return
	return;
}

// kernelversion() Function
int kernelversion (int force) {

	// We assume Kernel 2.3 is compatible with Kernel 2.2
	// We assume Kernel 2.1 is compatible with Kernel 2.0

	if (force!=0) {
		KERNEL_VERSION=force;
		fprintf (fstdout, "si:  Using Debug Code\n");
		return 0;
	}
	if (strncmp("2.3", UTS_RELEASE, 3)==0) {
		KERNEL_VERSION=TWOPOINTTWO;
		fprintf (fstdout, "si:  Using Linux Kernel 2.3 Code\n");
	}
	if (strncmp("2.2", UTS_RELEASE, 3)==0) {
		KERNEL_VERSION=TWOPOINTTWO;
		fprintf (fstdout, "si:  Using Linux Kernel 2.2 Code\n");
	}
	if (strncmp("2.1", UTS_RELEASE, 3)==0) {
		KERNEL_VERSION=TWOPOINTZERO;
		fprintf (fstdout, "si:  Using Linux Kernel 2.1 Code\n");
	}
	if (strncmp("2.0", UTS_RELEASE, 3)==0) {
		KERNEL_VERSION=TWOPOINTZERO;
		fprintf (fstdout, "si:  Using Linux Kernel 2.0 Code\n");
	}
	return 0;
}

int command_line (int argc, char **argv, char **argp) {
	int i;

	fprintf (fstdout, "si:  Checking Command-Line\n");
	while (1) {
		i=getopt(argc, argv, "f:o:h?");
		if (i==EOF) break;
		switch (i) {
			case 'h' | '?':
				return 1;
				break;
			case 'f':
				OPT_OUTPUT_FILE=1;
				strcpy (OUTPUT_FILE, optarg);
				break;
			case 'o':
				if (instr(optarg, 's')==-1) OPT_SHOW_SYSTEM=0;
				if (instr(optarg, 'h')==-1) OPT_SHOW_HARDWARE=0;
				if (instr(optarg, 'm')==-1) OPT_SHOW_MEMORY=0;
				if (instr(optarg, 'k')==-1) OPT_SHOW_KERNEL=0;
				if (instr(optarg, 'f')==-1) OPT_SHOW_FILESYS=0;
				if (instr(optarg, 'n')==-1) OPT_SHOW_NETWORK=0;
				if (instr(optarg, 'p')==-1) OPT_SHOW_PROCESS=0;
				break;
			default:
				return 1;
				break;
		}
	}
	return 0;
}



