/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_checkbutton.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>

/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBCheckButton
// put here vdk class name string
#define VDK_CLASS "VDKCheckButton"
// put here vdk class name 
#define VDK_ANCESTOR  VDKCheckButton
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "checkbutton"
//=================================================================


static char buff[128];
// used to autogenerate default
int CLASS::Counter = 0;
/* 
 properties names
*/
char* vdkcheckbutton_props[] = {  "Caption","CaptionWrap",0 };
/*
button signals names && nicknames
Default response method name will be made by:
On<object name><nickname>.
For instance if object name = <Abutton> and nickname = <Click>
default response method name will be: <OnAbuttonClick>
*/
char* vdkcheckbutton_signals[] = { "toggled_signal",0 };
char* vdkcheckbutton_nicknames[] = { "Toggled",0 };
//////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDK_ANCESTOR);
DEFINE_EVENT_LIST(CLASS,VDK_ANCESTOR);
//////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, VDKForm* owner,
	     char* label, char* tip = NULL):
  VDKCheckButton(owner,label,tip),VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties list
  for(t=0; vdkcheckbutton_props[t]; t++)
    proplist.add(VDKBProperty(vdkcheckbutton_props[t]));
  // add to VDKBObject signal list 
  for(t=0; vdkcheckbutton_signals[t]; t++)
    siglist.add(VDKBSignal(vdkcheckbutton_signals[t],
			   this,
			   vdkcheckbutton_nicknames[t]));
  // connects common events.
  CONNECT_COMMON_EVENTS
  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
  //============================
  // USER STUFF BEGINS HERE
  if(label)
    SetPropValue("Caption",label);
}
/////////////////////////////////////
//
//          WRITE TO .FRM FILE
//
/////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  //============================
  // USER STUFF BEGINS HERE
  // writes specific properties
  fprintf(fp,"\n\tCaption:\"%s\";",(char*) GetProp("Caption"));
  fprintf(fp,"\n\tCaptionWrap:%s;",(char*) GetProp("CaptionWrap"));
}
////////////////////////////////////////
//
//        PREPARE GUI WIDGETS
//
////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
3 - unknown
 */
		       
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label button counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  // USER RESPONSIBILITY
  CLASS* vdkwidget = new CLASS(buff,owner,buff);
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  bool flag = false;
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(owner->Owner());
	  if(prjman && prjman->objInspector)
	    flag = prjman->objInspector->preview->Checked;
	  // we are on fixed geometry ?
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      vdkwidget->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      vdkwidget->SetPropValue( EXPAND_INTERNAL,buff);
	      container->AddWidget(vdkwidget,
				   int(event->x),
				   int(event->y),
				   true,true,true);
	    }
	  // or preferred gtk+ fashion ?
	  else
	    container->AddWidget(vdkwidget,l_justify,flag,flag,true,flag);
	}
      else if(! owner->Active->AddToParent(vdkwidget,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  char obj_caption[128];
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  if(!parser.GetParam(obj_caption,buffer,"Caption:"))
    strcpy(obj_caption,"none");
  sprintf(tmp,"\n%s = new %s(this,\"%s\");",
	  obj_name,
	  VDK_CLASS,
	  obj_caption);
  strcpy(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  // get label button  props
  if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->CaptionWrap = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
char obj_name[128];
char obj_parent[128];
char obj_caption[256];
char arg[64];
CLASS* button;
// get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  // get caption
  if(!parser.GetParam(obj_caption,buffer,"Caption:"))
    strcpy(obj_caption,"none");
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      button = new CLASS(obj_name,owner,obj_caption);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(button,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = 0;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  button->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) button->GetProp( EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) button->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) button->GetProp("_Padding");
      padding = atoi(p);
      if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
	{
	button->CaptionWrap = !strcmp(arg,CHECK_TRUE);
	button->SetPropValue("CaptionWrap",arg);
	}
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(button,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  // FIX ME: lang support
  VDKTable* table = new VDKTable(inspector,2,2,true);

  VDKLabelButton* button = 
    new VDKLabelButton(inspector,"Set Caption");
  table->AddToCell(button,0,0);
  button->Parent(this);
  SignalConnect(button,"clicked",&CLASS::OnSetCaption);
  caption = new VDKEntry(inspector,0,(char*) GetProp("Caption"));
  caption->SetSize(100,-1);
  table->AddToCell(caption,0,1);

  wrap = new VDKCheckButton(inspector,"Caption wrap");
  table->AddToCell(wrap,1,0);
  wrap->Parent(this);
  SignalConnect(wrap,"toggled",&CLASS::OnSetCaptionWrap);
  
  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}
//////////////////////////////////////////////////////
// These response methods actually change both 
// properties on widget and gui widget properties
//////////////////////////////////////////////////////
/*
 */
bool
CLASS::OnSetCaptionWrap(VDKObject*)
{
  SetPropValue("CaptionWrap", wrap->Checked ? CHECK_TRUE : CHECK_FALSE);
  CaptionWrap = wrap->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetCaption(VDKObject*)
{
  if(strlen(caption->Text)>0)
    {
      sprintf(buff,"%s",(char*) caption->Text);
      SetPropValue("Caption",buff);
      Caption = caption->Text;
      inspector->FormNeedToBeChanged();
    }
  return true;
}








