/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_combo.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBCombo
// put vdk class name to corresponding widget
#define VDK_ANCESTOR  VDKCombo
// put here vdk class name string
#define VDK_CLASS "VDKCombo"
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "combo"

static char buff[128];
// used to autogenerate default
int CLASS::Counter = 0;

//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDK_ANCESTOR);
DEFINE_EVENT_LIST(CLASS,VDK_ANCESTOR);
///////////////////////////////////////////////////
/* 
 properties names
 */
char* vdkcombo_props[] =
{ 
"Editable","Sorted","Hidden","CaseSensitive",0
};
/*
entry widget signal names && nicknames
Default response method name will be made by:
On<object name><nickname>.
For instance if object name = <Abutton> and nickname = <Click>
default response method name will be: <OnAbuttonClick>
*/
char* vdkcombo_signals[] =
{
"activate_signal","changed_signal","selection_changed_signal",
"focus_in_signal","focus_out_signal",0
};
char* vdkcombo_nicknames[] =
{
"Activate","Changed","SelectionChanged","FocusIn","FocusOut",0
};

/*
 */
int
EntryButtonPressed(GtkWidget * wid,
		   GdkEvent * ev,
		   gpointer p)
{
  gtk_signal_emit_stop_by_name(GTK_OBJECT(wid),
			       "button_press_event");
  // proceed to upper level
  return false;
}
//////////////////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, VDKForm* owner):
  VDK_ANCESTOR(owner),VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties 
  for(t=0; vdkcombo_props[t]; t++)
    proplist.add(VDKBProperty(vdkcombo_props[t]));
  // add to VDKBObject signal list
  for(t=0; vdkcombo_signals[t]; t++)
    siglist.add(VDKBSignal(vdkcombo_signals[t],
			   this,
			   vdkcombo_nicknames[t]));
  //
  CONNECT_COMMON_EVENTS;
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(widget)->entry),
		       "button_press_event",
		       GTK_SIGNAL_FUNC(::EntryButtonPressed),
		       (gpointer) this);
  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tEditable:\"%s\";",(char*) GetProp("Editable"));
  fprintf(fp,"\n\tHidden:%s;",(char*) GetProp("Hidden"));
  fprintf(fp,"\n\tSorted:%s;",(char*) GetProp("Sorted"));
  fprintf(fp,"\n\tCaseSensitive:%s;",(char*) GetProp("CaseSensitive"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label button counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  CLASS* combo = new CLASS(buff,owner);
  if(owner->Active)
    {
      bool flag = false;
      VDKBProjectManager* prjman = 
	dynamic_cast<VDKBProjectManager*>(owner->Owner());
      if(prjman && prjman->objInspector)
	flag = prjman->objInspector->preview->Checked;
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      combo->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      combo->SetPropValue( EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(combo,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(combo,l_justify,flag,flag,true,flag);
	}
      else if(! owner->Active->AddToParent(combo,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  sprintf(tmp,"\n%s = new %s(this);",obj_name,VDK_CLASS);
  strcpy(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  if(parser.GetParam(arg,buffer,"Editable:") && 
     strcmp(arg,NIHIL_PROP))
    {
      sprintf(tmp,"\n%s->Editable = %s;",obj_name,arg);
      strcat(source,tmp);
    }    

  if(parser.GetParam(arg,buffer,"Hidden:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->Hidden = %s;",obj_name,arg);
     strcat(source,tmp);
   }

  if(parser.GetParam(arg,buffer,"Sorted:") && 
     strcmp(arg,NIHIL_PROP))
    {
      sprintf(tmp,"\n%s->Sorted = %s;",obj_name,arg);
      strcat(source,tmp);
    }    

  // 
  if(parser.GetParam(arg,buffer,"CaseSensitive:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->CaseSensitive = %s;",obj_name,arg);
     strcat(source,tmp);
   }        
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }

  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
char obj_name[128];
char obj_parent[128];
char arg[64];
CLASS* combo;
// get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      combo = new CLASS(obj_name,owner);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(combo,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  combo->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) combo->GetProp(EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) combo->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) combo->GetProp("_Padding");
      padding = atoi(p);
      if(parser.GetParam(arg,buffer,"Editable:") && strcmp(arg,NIHIL_PROP))
	combo->SetPropValue("Editable",arg);
      if(parser.GetParam(arg,buffer,"Hidden:") && strcmp(arg,NIHIL_PROP))
	combo->SetPropValue("Hidden",arg);
      if(parser.GetParam(arg,buffer,"Sorted:") && strcmp(arg,NIHIL_PROP))
	combo->SetPropValue("Sorted",arg);
      if(parser.GetParam(arg,buffer,"CaseSensitive:") 
	 && strcmp(arg,NIHIL_PROP))
	combo->SetPropValue("CaseSensitive",arg);

      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(combo,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
 */
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  VDKString True = CHECK_TRUE;
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  VDKTable *table = new VDKTable(inspector,2,2);
  table->SetSize(219,-1);
  
  editable = new VDKCheckButton(inspector,"Editable");
  table->AddToCell(editable,0,0);
  editable->Checked = GetProp("Editable") == True;
  // this after (so no signal hang)
  editable->Parent(this);
  SignalConnect(editable,"toggled",&CLASS::OnSetEditable);

  sorted = new VDKCheckButton(inspector,"Sorted");
  table->AddToCell(sorted,0,1);
  sorted->Checked = GetProp("Sorted") == True;
  // this after (so no signal hang)
  sorted->Parent(this);
  SignalConnect(editable,"toggled",&CLASS::OnSetSorted);

  hidden = new VDKCheckButton(inspector,"Hidden");
  table->AddToCell(hidden,1,0);
  hidden->Checked = GetProp("Hidden") == True;
  // this after (so no signal hang)
  hidden->Parent(this);
  SignalConnect(hidden,"toggled",&CLASS::OnSetHidden);

  casesensitive = new VDKCheckButton(inspector,"CaseSensitive");
  table->AddToCell(casesensitive,1,1);
  casesensitive->Checked = GetProp("CaseSensitive") == True;
  // this after (so no signal hang)
  casesensitive->Parent(this);
  SignalConnect(casesensitive,"toggled",&CLASS::OnSetCaseSensitive);

  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}

/*
 */
bool
CLASS::OnSetEditable(VDKObject*)
{
  SetPropValue("Editable", editable->Checked ? CHECK_TRUE : CHECK_FALSE);
  Editable = editable->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetHidden(VDKObject*)
{
  SetPropValue("Hidden", hidden->Checked ? CHECK_TRUE : CHECK_FALSE);
  Hidden = hidden->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetSorted(VDKObject*)
{
  SetPropValue("Sorted", sorted->Checked ? CHECK_TRUE : CHECK_FALSE);
  Sorted = sorted->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetCaseSensitive(VDKObject*)
{
  SetPropValue("CaseSensitive", casesensitive->Checked ? CHECK_TRUE : CHECK_FALSE);
  CaseSensitive = casesensitive->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}


