/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 

#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <stdlib.h>
#include <vdkb_customlist.h>
#include <vdkb_clistdlg.h>
#include <vdkb_prjman.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBCustomList
// put here vdk class name string
#define VDK_CLASS "VDKCustomList"
// put here vdk class name 
#define VDK_ANCESTOR  VDKCustomList
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "customlist"

static char buff[128];

// used to autogenerate default
// labelbutton  names and captions

int VDKBCustomList::Counter = 0;
/* 
 label properties names
 */
char* vdkclist_props[] =
{ 
"VPolicy","HPolicy","BorderShadow","RowHeight","AutoResize",
"SelectionMode",0
};
static char *shadows[] = 
{ 
"shadow_none","shadow_in","shadow_out","shadow_etched_in",
"shadow_etched_out",0 
};

char* vdkclist_signals[] =
{
"select_row_signal","unselect_row_signal","click_column_signal",0
};
char* vdkclist_nicknames[] =
{
"SelectRow","UnselectRow","ClickColumn",0
};
//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDK_ANCESTOR);
DEFINE_EVENT_LIST(CLASS,VDK_ANCESTOR);
///////////////////////////////////////////////////
/*
 */
bool
CLASS::OnColumnClicked(VDKObject*)
{
  if(ColumnClicked() < 0)
    return true;
  else if(inspector)
    {
      inspector->SetActive(this);
      columnTitle->Text = column_titles[ColumnClicked()];
      columnTitle->Enabled = true;
    }
  return true;
}

//////////////////////////////////////////////////////////////
/*
  - constructor
 */
// default pixmap
// defined in vdkb_local.cc 
extern char** default_pixmap;

CLASS::CLASS(char* name, VDKForm* owner,
			       int cols, 
			       char** titles,
			       GtkSelectionMode mode):
  VDK_ANCESTOR(owner,cols,titles,mode),VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties 
  for(t=0; vdkclist_props[t]; t++)
    proplist.add(VDKBProperty(vdkclist_props[t]));
    // add to VDKBObject signal list  signals
  for(t=0; vdkclist_signals[t]; t++)
    siglist.add(VDKBSignal(vdkclist_signals[t],
			   this,
			   vdkclist_nicknames[t]));

  // make and load colums title array
  column_titles = TitlesArray(cols);
  for(t = 0 ; t < cols; t++)
    column_titles[t] = VDKString(titles[t]);

  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
  // a reasonable size
  SetSize(100,100);
  SetPropValue(USIZE,"100,100");
  CONNECT_COMMON_EVENTS;
  SignalConnect("click_column",&CLASS::OnColumnClicked,false);
  inspector = NULL;
}

////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tSelectionMode:%s;", (char*) GetProp("SelectionMode"));
  fprintf(fp,"\n\tAutoResize:%s;", (char*) GetProp("AutoResize"));
  fprintf(fp,"\n\tBorderShadow:%s;", (char*) GetProp("BorderShadow"));
  fprintf(fp,"\n\tColumns:%d;",column_titles.size());
  // writes columns titles:
  fprintf(fp,"\n\tTitles:\"");
  int t,last = column_titles.size();
  for(t = 0; t < last-1 ; t++)
    fprintf(fp,"%s,", (char*) column_titles[t].isNull() ? 
	    NIHIL_PROP : (char*) column_titles[t]);
  fprintf(fp,"%s\";", (char*) column_titles[last-1].isNull() ? 
	  NIHIL_PROP : (char*) column_titles[last-1]);
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
static void
AddSomeRows(CLASS* clist, int columns)
{
  int t,z;
  for(z = 0; z < 9; z++)
    {
      char** cells = new char*[columns];
      for(t=0;t<columns;t++)
	{
	  cells[t] = new char[32];
	  sprintf(cells[t],"Sample %d.%d",z,t);
	}
      clist->AddRow(cells);
      // free them
      for(t=0; t < columns; t++)
	delete cells[t];
      delete cells;
    }
}

int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  int columns = 0,t;
  // be sure target is a container (otherwise hangs -up )
  if(!dynamic_cast<VDKBEventContainer*>(owner->Active))
      return 2;
  // autogenerate first suitable pixmap counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  // columns should be asked to user
  Vdkb_clistdlgForm* dlg = new Vdkb_clistdlgForm(owner,NULL,&columns);
  dlg->Setup();
  dlg->ShowModal();
  columns = columns ? columns : 1;
  // autogenerates titles
  char **titles = new char*[columns];
  for(t=0; t < columns;t++)
    {
      titles[t] = new char[32];
      sprintf(titles[t],"Title#%d",t);
    }
  CLASS* clist = new CLASS(buff,
					     owner,
					     columns,
					     titles);
  for(t=0; t < columns; t++)
    delete titles[t];
  delete titles;
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  bool flag = false;
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(owner->Owner());
	  if(prjman && prjman->objInspector)
	    flag = prjman->objInspector->preview->Checked;
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      clist->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      clist->SetPropValue(EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(clist,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(clist,l_justify,flag,flag,true,flag);
	  if(! dynamic_cast<VDKBFixed*>(container))
	    AddSomeRows(clist, columns);
	}
      else if(! owner->Active->AddToParent(clist,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
static void
CreateTitles(char* source,char* obj_name,int obj_cols, char* obj_titles)
{
  char* p;
  char tmp[128];
  int t = 0;
  sprintf(tmp,"\nchar* %sTitles[] = { ",obj_name);
  strcpy(source,tmp);
  p = strtok(obj_titles,",");
  while(p)
    {
      sprintf(tmp,"\"%s\",",strcmp(p,NIHIL_PROP) ? p : " ");
      t++;
      p = strtok(NULL,",");
      if(!p)
	// cut comma
	tmp[strlen(tmp)-1] = '\0';
      strcat(source,tmp);
    }
  sprintf(tmp," };");
  strcat(source,tmp);
 if( t!= obj_cols)
   ; // FIX ME: error to be noticed to user
 return;
}
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  char obj_titles[1024];
  char obj_cols[32];
  char selmode[16];
  int  smode =  0;
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") ||
       !parser.GetParam(obj_cols,buffer,"Columns:") ||
       !parser.GetParam(obj_titles,buffer,"Titles:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  if(parser.GetParam(selmode,buffer,"SelectionMode:") &&
     strcmp(arg,NIHIL_PROP))
    {
      smode = atoi(selmode);
    }
     
  CreateTitles(source,obj_name,atoi(obj_cols),obj_titles);
  sprintf(tmp,
	  "\n%s = new %s(this,%s,%sTitles,(GtkSelectionMode) %d);",
	  obj_name,VDK_CLASS,obj_cols,obj_name,smode);
  strcat(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  if(parser.GetParam(arg,buffer,"AutoResize:") && 
     strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->AutoResize = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  // get shadow
  if(parser.GetParam(arg,buffer,"BorderShadow:") && strcmp(arg,NIHIL_PROP))
    {
      int ndx = atoi(arg);
      ndx = (ndx >= 0) && (ndx <= 4) ? ndx : 0;
      sprintf(tmp,"\n%s->BorderShadow = (GtkShadowType) %s;", 
	      obj_name,shadows[ndx]);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
static char **
GenerateTitles(int obj_cols, char* obj_titles)
{
  char* p, **titles;
  int t = 0;
  // generates titles
  if(obj_cols <= 0)
    return (char**) NULL;
  titles = new char*[obj_cols];
  p = strtok(obj_titles,",");
  while(p)
    {
      titles[t] = new char[strlen(p)+1];
      if(strcmp(p,NIHIL_PROP))
	strcpy(titles[t],p);
      else
	strcpy(titles[t]," ");
      t++;
      p = strtok(NULL,",");
    }
  return t == obj_cols ? titles : (char**) NULL;
}
/*
 */
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
  VDKString True = CHECK_TRUE;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char obj_titles[1024];
  char obj_cols[32];
  char selmode[16];
  int  smode =  0;
  //char arg[64];
  CLASS* clist;
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") ||
       !parser.GetParam(obj_cols,buffer,"Columns:") ||
       !parser.GetParam(obj_titles,buffer,"Titles:") 
       )
    return false;
  
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  if(parser.GetParam(selmode,buffer,"SelectionMode:") &&
     strcmp(arg,NIHIL_PROP))
    smode = atoi(selmode);

  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      char** tit = GenerateTitles(atoi(obj_cols),obj_titles);
      clist = new CLASS(obj_name,owner,
				 atoi(obj_cols),
				 tit, (GtkSelectionMode) smode);
      if(tit)
	{
	  int t = 0;
	  int z = atoi(obj_cols);
	  for( ; t < z; t++)
	    delete tit[t];
	  delete tit;
	}

      if(parser.GetParam(selmode,buffer,"SelectionMode:") &&
	 strcmp(arg,NIHIL_PROP))
	clist->SetPropValue("SelectionMode",selmode);
      // shadow
      if(parser.GetParam(arg,buffer,"AutoResize:") &&
	 strcmp(arg,NIHIL_PROP))
	{
	  clist->SetPropValue("AutoResize",arg);
	  clist->AutoResize = !strcmp(arg,CHECK_TRUE);
	}
      if(parser.GetParam(arg,buffer,"BorderShadow:") &&
	 strcmp(arg,NIHIL_PROP))
	{
	  clist->SetPropValue("BorderShadow",arg);
	  int sh = atoi(arg);
	  clist->BorderShadow = (GtkShadowType) sh;
	}
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(clist,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  clist->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) clist->GetProp(EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) clist->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) clist->GetProp("_Padding");
      padding = atoi(p);
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(clist,justification,expand,fill,padding,true);
      if(! dynamic_cast<VDKBFixed*>(container))
	AddSomeRows(clist, atoi(obj_cols));
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
GTK_SELECTION_SINGLE,  GTK_SELECTION_BROWSE,  GTK_SELECTION_MULTIPLE,
  GTK_SELECTION_EXTENDED
*/
static
char *selectionmodes[] = { "single","browse","multiple","extended",0 };
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  VDKString True = CHECK_TRUE;
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  VDKTable *table = new VDKTable(inspector,4,2);
  table->SetSize(219,-1);

  VDKLabelButton *setmode = new VDKLabelButton(inspector,"Set sel mode");
  table->AddToCell(setmode,0,0);
  setmode->Parent(this);
  SignalConnect(setmode,"clicked",&CLASS::OnSetSelectionMode);

  selmode = new VDKCombo(inspector,NULL);
  selmode->SetSize(100,-1);
  StringList sm;
  int t = 0;
  for(;selectionmodes[t];t++)
    sm.add(VDKString(selectionmodes[t]));
  selmode->PopdownStrings = sm;
  table->AddToCell(selmode,0,1);
  int r = atoi(GetProp("SelectionMode"));
  selmode->SelectItem(r);
 

  table->AddToCell(new VDKLabel(inspector,"Column title: "),1,0);
  columnTitle = new VDKEntry(inspector,64);
  columnTitle->SetSize(100,-1);
  table->AddToCell(columnTitle,1,1);
  columnTitle->Parent(this);
  SignalConnect(columnTitle,"activate",&CLASS::OnSetColumnTitle);
  columnTitle->Enabled = false;


  autoresize = new VDKCheckButton(inspector,"AutoResize");
  table->AddToCell(autoresize,2,1);
  autoresize->Checked = GetProp("AutoResize") == True;
  autoresize->Parent(this);
  SignalConnect(autoresize,"toggled",&CLASS::OnSetAutoResize);


  VDKLabelButton *set = new VDKLabelButton(inspector,"Set border shadow");
  table->AddToCell(set,3,0);
  set->Parent(this);
  SignalConnect(set,"clicked",&CLASS::OnSetShadow);
  shadow = new VDKCombo(inspector,NULL);
  shadow->SetSize(100,-1);
  StringList sl;
  t = 0;
  for(;shadows[t];t++)
    sl.add(VDKString(shadows[t]));
  shadow->PopdownStrings = sl;
  table->AddToCell(shadow,3,1);
  r = atoi(GetProp("BorderShadow"));
  shadow->SelectItem(r);

  bframe->Add(table,l_justify,false,false,false);
  return bframe;
}
//////////////////////////////////////////////////////
// These response methods actually change both 
// properties on widget and gui widget properties
//////////////////////////////////////////////////////
/*
 */
bool
CLASS::OnSetColumnTitle(VDKObject*)
{
  if(ColumnClicked() < 0)
    return true;
  else if(strlen(columnTitle->Text) <= 0)
    sprintf(buff,NIHIL_PROP);
  else
    sprintf(buff,"%s",(char*) columnTitle->Text);
  gtk_clist_set_column_title (GTK_CLIST(CustomWidget()),
			      ColumnClicked(),
			      strcmp(buff,NIHIL_PROP) ? buff : " ");
  column_titles[ColumnClicked()] = buff;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetAutoResize(VDKObject*)
{
  SetPropValue("AutoResize", autoresize->Checked ? CHECK_TRUE : CHECK_FALSE);
  AutoResize = autoresize->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetShadow(VDKObject*)
{
  int sel = shadow->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  SetPropValue("BorderShadow",buff);
  int shd = atoi(buff);
  BorderShadow = (GtkShadowType) shd;
  inspector->FormNeedToBeChanged();
  return true;
}
bool
CLASS::OnSetSelectionMode(VDKObject*)
{
  int sel = selmode->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  SetPropValue("SelectionMode",buff);
  gtk_clist_set_selection_mode ( GTK_CLIST(CustomWidget()),
				(GtkSelectionMode)  sel);
  inspector->FormNeedToBeChanged();
  return true;
}







