/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * =========================== 
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_entry.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBEntry
// put here vdk class name string
#define VDK_CLASS "VDKEntry"
// put here vdk class name 
#define VDK_ANCESTOR  VDKEntry
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "entry"

static char buff[128];

// used to autogenerate default
// labelbutton  names and captions

int CLASS::Counter = 0;
/* 
 label properties names
 */
char* vdkentry_props[] =
{ 
"Text","Editable","Hidden","MaxChars",0
};
/*
entry widget signal names && nicknames
Default response method name will be made by:
On<object name><nickname>.
For instance if object name = <Abutton> and nickname = <Click>
default response method name will be: <OnAbuttonClick>
*/
char* vdkentry_signals[] =
{
"activate_signal","changed_signal",
"focus_in_signal","focus_out_signal",0
};
char* vdkentry_nicknames[] =
{
"Activate","Changed","FocusIn","FocusOut",0
};
//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDK_ANCESTOR);
DEFINE_EVENT_LIST(CLASS,VDK_ANCESTOR);
//////////////////////////////////////////////////
/*
overridden ButtonPressed
used in substitution of
VDKBObject::ButtonPress
*/
bool
VDKBEntry::OverriddenButtonPressed(VDKObject* sender, GdkEvent* ev)
{
  VDKBEventContainer* container;
  gtk_signal_emit_stop_by_name(GTK_OBJECT(sender->WrappedWidget()),
  		       "button_press_event");
  container = dynamic_cast<VDKBEventContainer*>(sender->Parent());
  if(container)
    // we pass <this> instead <sender>
    // otherwise entry widget selection
    // is lost (don'y ask me why !)
    container->OnButtonPressed(this,ev);
  return true;
}

//////////////////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, VDKForm* owner):
  VDK_ANCESTOR(owner,0,name),VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties list label button properties
  // names. (Others props are prepended  by VDKBObject constructor)
  for(t=0; vdkentry_props[t]; t++)
    proplist.add(VDKBProperty(vdkentry_props[t]));
  // add to VDKBObject signal list  signals
  for(t=0; vdkentry_signals[t]; t++)
    siglist.add(VDKBSignal(vdkentry_signals[t],
			   this,
			   vdkentry_nicknames[t]));
  SetPropValue("Text",name);
  SetPropValue("MaxChars","0");
  // connects events.
  // an exception here
  EventConnect("button_press_event",&VDKBEntry::OverriddenButtonPressed);
  // others kept from ancestor
  EventConnect("button_release_event",&VDKBObject::ButtonReleased);
  EventConnect("enter_notify_event",&VDKBObject::OnEnter);
  EventConnect("leave_notify_event",&VDKBObject::OnLeave);  
  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tEditable:\"%s\";",(char*) GetProp("Editable"));
  fprintf(fp,"\n\tHidden:%s;",(char*) GetProp("Hidden"));
  fprintf(fp,"\n\tText:%s;",(char*) GetProp("Text"));
  fprintf(fp,"\n\tMaxChars:%s;",(char*) GetProp("MaxChars"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable text counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  CLASS* text = new CLASS(buff,owner);
  if(owner->Active)
    {
      bool flag = false;
      VDKBProjectManager* prjman = 
	dynamic_cast<VDKBProjectManager*>(owner->Owner());
      if(prjman && prjman->objInspector)
	flag = prjman->objInspector->preview->Checked;
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      text->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      text->SetPropValue(EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(text,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(text,l_justify,flag,flag,true,flag);
	}
      else if(! owner->Active->AddToParent(text,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char obj_editable[16];
  char obj_text[256];
  char obj_maxchars[16];
  char arg[64];
  char tmp[256];
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  //
  if(parser.GetParam(obj_maxchars,buffer,"MaxChars:") && 
     strcmp(obj_maxchars,NIHIL_PROP))
    ;
  else
    strcpy(obj_maxchars,"0");
    //
  if(parser.GetParam(obj_text,buffer,"Text:") && 
     strcmp(obj_text,NIHIL_PROP))
    ;
  else
    *obj_text = '\0';
  if(*obj_text)
    sprintf(tmp,"\n%s = new %s(this,%s,\"%s\");",
	  obj_name,VDK_CLASS,obj_maxchars,obj_text);
  else
    sprintf(tmp,"\n%s = new %s(this,%s);", VDK_CLASS,obj_name,obj_maxchars);
  strcpy(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  //
  if(parser.GetParam(obj_editable,buffer,"Editable:") && 
     strcmp(obj_editable,NIHIL_PROP))
    {
      sprintf(tmp,"\n%s->Editable = %s;",obj_name,obj_editable);
      strcat(source,tmp);
    }    

  // get label button  props
  if(parser.GetParam(arg,buffer,"Hidden:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->Hidden = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
char obj_name[128];
char obj_parent[128];
char arg[256];
CLASS* text;
// get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      text = new CLASS(obj_name,owner);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(text,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  text->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) text->GetProp(EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) text->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) text->GetProp("_Padding");
      padding = atoi(p);
      if(parser.GetParam(arg,buffer,"Editable:") && strcmp(arg,NIHIL_PROP))
	text->SetPropValue("Editable",arg);
      if(parser.GetParam(arg,buffer,"Hidden:") && strcmp(arg,NIHIL_PROP))
	text->SetPropValue("Hidden",arg);
      if(parser.GetParam(arg,buffer,"MaxChars:") && strcmp(arg,NIHIL_PROP))
	text->SetPropValue("MaxChars",arg);
      if(parser.GetParam(arg,buffer,"Text:") && strcmp(arg,NIHIL_PROP))
	text->SetPropValue("Text",arg);
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(text,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
 */
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  VDKString True = CHECK_TRUE;
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  VDKTable *table = new VDKTable(inspector,4,2);
  table->SetSize(219,-1);
  
  hidden = new VDKCheckButton(inspector,"Hidden");
  table->AddToCell(hidden,0,1);
  hidden->Checked = GetProp("Hidden") == True;
  // this after (so no signal hang)
  hidden->Parent(this);
  SignalConnect(hidden,"toggled",&CLASS::OnSetHidden);

  editable = new VDKCheckButton(inspector,"Editable");
  table->AddToCell(editable,1,1);
  editable->Checked = GetProp("Editable") == True;
  // this after (so no signal hang)
  editable->Parent(this);
  SignalConnect(editable,"toggled",&CLASS::OnSetEditable);

  table->AddToCell(new VDKLabel(inspector,"Text"),2,0);
  text = new VDK_ANCESTOR(inspector,0,GetProp("Text"));
  text->SetSize(100,-1);
  table->AddToCell(text,2,1);
  text->Parent(this);
  SignalConnect(text,"activate",&CLASS::OnSetEntryText);

  table->AddToCell(new VDKLabel(inspector,"Max entry chars "),3,0);
  int mc = atoi(GetProp("MaxChars"));
  maxchars = new VDKSpinButton(inspector,mc , 0, 255 , 1 ,0 );
  maxchars->SetSize(100,-1);
  table->AddToCell(maxchars,3,1);
  maxchars->Parent(this);
  // note last arg as <false>
  // is necessary since gtk+ "value_changed"
  // isn't treated by gtk+
  SignalConnect(maxchars,"value_changed",&CLASS::OnSetMaxChars,false);
  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}
//////////////////////////////////////////////////////
// These response methods actually change both 
// properties on widget and gui widget properties
//////////////////////////////////////////////////////
/*
 */
bool
CLASS::OnSetHidden(VDKObject*)
{
  SetPropValue("Hidden", hidden->Checked ? CHECK_TRUE : CHECK_FALSE);
  Hidden = hidden->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetEditable(VDKObject*)
{
  SetPropValue("Editable", editable->Checked ? CHECK_TRUE : CHECK_FALSE);
  Editable = editable->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetEntryText(VDKObject*)
{
  if(strlen(text->Text)>0)
    {
      Text = (char*) text->Text;
      sprintf(buff,"\"%s\"",(char*) text->Text);
      SetPropValue("Text",buff);
      inspector->FormNeedToBeChanged();
    }
  return true;
}
/*
 */
bool
CLASS::OnSetMaxChars(VDKObject*)
{
  sprintf(buff,"%d",(int) maxchars->ValueAsInt);
  SetPropValue("MaxChars",buff);
  inspector->FormNeedToBeChanged();
  return true;
}









