/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * =========================== 
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#include <vdkb_evcontain.h>
#include <vdkb_form.h>
#include <vdkb_types.h>
#include <vdkb_prjman.h>
#include <vdkb_objinspect.h>
#include <vdkb_paned.h>
#include <vdkb_notebook.h>
#include <vdkb_table.h>
#include <vdkb_frame.h>
#include <vdkb_fixed.h>
extern OpState  OperationalState;
static char buff[256];
int VDKBCanvas::Counter = 0;
int VDKBEventContainer::Counter = 0;
/*
 */
VDKBEventContainer::VDKBEventContainer(char* name,VDKForm* owner):
  VDKObjectContainer(owner),VDKBObject(name)
{
  VDKBObject::object = this;
  widget = sigwid = gtk_event_box_new(); 
  container = NULL;
  canvas = NULL;
  slot = 0;
  mode = -1;
  outerbox = NULL;
  popmenu = NULL;
  Counter++;
}

/*
 */
void 
VDKBEventContainer::Add(VDKObject* obj, int justify,
		   int expand, int fill, int padding) 
{
  VDKBGuiForm* ownerform = dynamic_cast<VDKBGuiForm*>(Owner());
  if(ownerform)
    ownerform->Changed = true;
  boxlist.add(obj);
  VDKObjectContainer::Add(obj,justify,expand,fill,padding);
}


/*
 */
VDKBEventContainer* 
VDKBEventContainer::Outerbox(VDKBEventContainer* b)
{
  if(b) outerbox = b;
  return outerbox;
}

/*
Removes active container and recursively
remove parent boxes if empty. (disabled)
With some exceptions:
- boxes embedded in table,paned or notebook
should not be removed
- gui form inner box should not be removed
 */
bool 
VDKBEventContainer::DelBox(VDKObject* sender)
{
  VDKBEventContainer* self = (VDKBEventContainer*) this;
  VDKBEventContainer* outer = self->outerbox;
  VDKBGuiForm* ownerform = dynamic_cast<VDKBGuiForm*>(Owner());
  if(! ownerform)
    return true;
  // paned,table,frame and notebook embedded boxes
  // cant be deleted
  if(outer && dynamic_cast<VDKEventBox*>(self))
    {
      if( 
	 dynamic_cast<VDKPaned*>(outer) ||
	 dynamic_cast<VDKTable*>(outer) ||
	 dynamic_cast<VDKBGuiNotebook*>(outer) ||
	 dynamic_cast<VDKBFrame*>(outer)
	 )
	return true;
    }
  // has a parent container
  if( outer )
    {
      // removing the active one
      if ( ownerform->Active == self)
	ownerform->Active = NULL;
      outer->boxlist.remove(self);

      self->Visible = false;
      // this produce a strange bug
      // FIX ME:
      //outer->RemoveObject(self);

      // removes ancestor empty box if any
      // error prone :-(
      // recursive destroyng disabled
      // together with nested layouts
      // (see vdkb_evbox.cc)
      /*
	if( (outer != ownerform->InnerBox()) &&
	(dynamic_cast<VDKBEventBox*>(outer)) &&
	(outer->boxlist.size() == 0)
	)
	outer->DelBox(NULL); 
      */
     }
  // removing active one
  else if (ownerform->Active == self)
    ownerform->Active = NULL;
  ownerform->Changed = true;
  return true;
}

/*
 */
bool
VDKBEventContainer::OnButtonPressed(VDKObject* sender, GdkEvent* event)
{
  VDKBObject* prevActive = NULL;
  GdkEventButton* evbtn = (GdkEventButton*) event;
  VDKBObject* active = dynamic_cast<VDKBObject*>(sender);

  VDKBGuiForm* ownerform = dynamic_cast<VDKBGuiForm*>(Owner());
  if( ownerform->Active)
    {
      ownerform->Active->ClearMark();
      prevActive = ownerform->Active;
    }
  active->Mark();
  ownerform->Active = active;
  switch(evbtn->button)
    {
      // STUB HERE
    case 1:
      if(OperationalState.state == op_stand_by &&
	 OperationalState.action == act_add_widget )
	{
	  // propagate to form
	  OperationalState.source = NULL;
	  ownerform->OnButtonPress(this,event);
	}
      // set active to object inspector
      else
	  // propagate to form
	  ownerform->OnButtonPress(sender,event);
      break;
    case 3:
      if(prevActive != active)
      	ownerform->OnButtonPress(sender,event);
      active->PopMenu();
      break;
    }
  return true;
}
/*
 */
void 
VDKBEventContainer::ClearMark() 
{
  if(canvas) 
    {
      canvas->Clear(); 
      gtk_widget_draw(canvas->Widget(),NULL);
    }
  else
    VDKBObject::ClearMark();

}
/*
 */
void 
VDKBEventContainer::Mark() 
{ 
  if(canvas) 
     canvas->Mark(); 
  else
    VDKBObject::Mark();
}

/*
 */
bool
VDKBEventContainer::OnExpose(VDKObject* sender, GdkEvent* event)
{
  //ClearMark();
  /*
  GtkWidget* wid = sender->WrappedWidget();
  GdkDrawable* drawable = (GdkDrawable*) wid->window;
  VDKPoint p = VDKPoint(wid->allocation.width,wid->allocation.height);
  GdkColormap * colormap = gdk_window_get_colormap (drawable);
  GdkColor oldcolor = wid->style->fg[GTK_WIDGET_STATE(wid)];
  GdkColor black;
  gdk_color_black(colormap,&black);
  wid->style->fg[GTK_WIDGET_STATE(wid)] = black;
  gdk_draw_rectangle(drawable,
	 	     wid->style->fg_gc[GTK_WIDGET_STATE(wid)],
		     0, 0, 0, 0, 0);
  wid->style->fg[GTK_WIDGET_STATE(wid)] = oldcolor;
  */
  return true;
}
/*
 */
VDKObjectContainer* 
VDKBEventContainer::ExtraWidget(VDKBObjectInspector* isp) 
{ 
return (VDKObjectContainer*) NULL; 
}
/*
 */
bool
VDKBEventContainer::SelectParentContainer(VDKObject*)
{
  VDKBEventContainer* container = 
    dynamic_cast<VDKBEventContainer*>(Parent());
  if(container)
    {
      VDKBGuiForm* ownerform = 
	dynamic_cast<VDKBGuiForm*>(Owner());
      if(ownerform)
	{
	  ownerform->Active->ClearMark();
	  ownerform->Active = container;
	  ownerform->Active->Mark();
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(ownerform->Owner());
	  if(prjman && prjman->objInspector)
	    //	    prjman->objInspector->SetActive(container);
	    prjman->objInspector->SelectWidgetByTree(container);
	}
    }
  return true;
}
/////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////
/*
 */

VDKBCanvas::VDKBCanvas(char* name,VDKForm* owner, int w, int h):
  VDKCanvas(owner,w,h),VDKBObject(name)
{
VDKBObject::object = this;
Counter++;
}
/*
 */
void
VDKBCanvas::Mark()
{
  VDKPoint p = VDKPoint(Widget()->allocation.width,
  		Widget()->allocation.height);
  GdkRectangle brush;
  brush.width = 4;
  brush.height = 4;

  brush.x = 0;
  brush.y = 0;
  DrawRect(true,brush.x,brush.y,brush.width,brush.height);

  brush.x = 0;
  brush.y = p.Y()-brush.height;
  DrawRect(true,brush.x,brush.y,brush.width,brush.height);

  brush.x = p.X()-brush.width;
  brush.y = 0;
  DrawRect(true,brush.x,brush.y,brush.width,brush.height);

  brush.x = p.X()-brush.width;
  brush.y = p.Y()-brush.height;
  DrawRect(true,brush.x,brush.y,brush.width,brush.height);

  if(Parent() && Parent()->isA() == vdkbevbox_class)
    {
      VDKBEventBox* box = (VDKBEventBox*) Parent();
      sprintf(buff,"%s",(char*) box->Name());
      DrawString(10,20,buff);
    }
  Redraw();
}
/*
 */
void 
VDKBCanvas::DrawUnfilledRect(int x, int y, int w, int h)
{
if(pixmap)
  gdk_draw_rectangle(pixmap,
		     widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		     FALSE,
		     x,y,w,h);
}





