/*
 * ===========================
 * VDK Builder
 * Version 0.1
 * Revision 0.5
 * August 1999
 * ===========================
 *
 * Copyright (C) 1998,1999 Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * Based on VDK Library 
 * Copyright (C) 1998, Mario Motta
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */ 

#include <vdkb_evcontain.h>
#include <vdkb_labelbutton.h>
#include <vdkb_form.h>
#include <vdk/vdk.h>
#include <vdkb_utils.h>
#include <vdkb_parser.h>
#include <vdkb_prjman.h>
#include <vdkb_objinspect.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
#include <vdkb_evbox.h>
static char buff[128];

int VDKBFixed::Counter = 0;
#define VERBOSE 0
/* 
 properties 
 */
char* vdkfixed_props[] =
{ 
"BorderWidth","have_grid","h_grid_spacing","v_grid_spacing", 0
};

DEFINE_EVENT_LIST(VDKBFixed,VDKBEventContainer);
DEFINE_SIGNAL_LIST(VDKBFixed,VDKBEventContainer);
// into vdkb_widsel.c
extern "C" void
fixed_draw_grid (GtkWidget * widget,
		 int grid_horz_spacing,
		 int grid_vert_spacing,
		 int grid_style);
/*
 */
bool
VDKBFixed:: DelBox(VDKObject* sender)
{
  // destroy inner gtk+ placeholder widget
  gtk_widget_destroy(container);
  // call ancestor delete box
  VDKBEventContainer::DelBox(sender);
  // notify to inspector that object was deleted
  VDKBGuiForm* ownerform = dynamic_cast<VDKBGuiForm*>(Owner());
  if(ownerform)
    {
      VDKBProjectManager* prjman = 
	dynamic_cast<VDKBProjectManager*>(ownerform->Owner());
      if(prjman && prjman->objInspector)
	prjman->objInspector->SetActive(NULL);
    }
return true;
}

/*
 */
bool 
VDKBFixed::SetBoxSize(VDKObject* sender)
{
  VDKBGuiForm* ownerform = dynamic_cast<VDKBGuiForm*>(Owner());
  if(ownerform)
    {
      ownerform->SetBoxSize(NULL);
    }
  return true;
}
/*
FIXED MOVING STUFF
 */
GtkWidget *grabbed_widget = NULL;

bool
VDKBFixed::OnButtonPressed(VDKObject* sender, GdkEvent* event)
{
  // pass to ancestor so widget will be marked
  gtk_signal_emit_stop_by_name(GTK_OBJECT(sender->WrappedWidget()),
			       "button_press_event");
  VDKBEventContainer::OnButtonPressed(this,event);
  OnExpose(NULL,NULL);
  if(grabbed_widget)
    {
      gtk_grab_remove(grabbed_widget);
      grabbed_widget = NULL;
    }
  return true;
}
/*
 */
bool
VDKBFixed::OnButtonReleased(VDKObject* sender, GdkEvent* event)
{
  // pass to ancestor so widget will be marked
  gtk_signal_emit_stop_by_name(GTK_OBJECT(sender->WrappedWidget()),
			       "button_release_event");
  if(grabbed_widget)
    {
      gtk_grab_remove(grabbed_widget);
      grabbed_widget = NULL;
    }
  return true;
}

/*
 */
void
VDKBFixed::ClearMark()
{
  VDKBObject::ClearMark();
#if VERBOSE
  printf("\nVDKBFixed::ClearMark()");
  fflush(stdout);
  OnExpose(NULL,NULL);
#endif
}
/*
 */
void
VDKBFixed::Mark()
{
  VDKBObject::Mark();
#if VERBOSE
  printf("\nVDKBFixed::Mark()");
  fflush(stdout);
  OnExpose(NULL,NULL);
#endif
}
/*
 */
bool
VDKBFixed::OnExpose(VDKObject* sender, GdkEvent* ev)
{
  // draw grid
  // checks if widget is still on life
  VDKString True = CHECK_TRUE;
  if(GetProp("have_grid") == True)
    {
      int grid_horz_spacing = 
	atoi((char*) GetProp("h_grid_spacing"));
      int grid_vert_spacing = 
	atoi((char*) GetProp("v_grid_spacing"));
      grid_horz_spacing = grid_horz_spacing <= 0 ? 8: grid_horz_spacing;
      grid_vert_spacing = grid_vert_spacing <= 0 ? 8: grid_vert_spacing;
      fixed_draw_grid (container, grid_horz_spacing,
		       grid_vert_spacing, 1); // dots
    }
   return true;
}

//===========================================
/*
 */
VDKBFixed::VDKBFixed(char* name, VDKForm* owner): 
  VDKBEventContainer(name,owner)
{
  Counter++;
  VDKBObject::object = this;
  VDKBEventContainer::mode = mode;
  Init();
}

/*
 */
VDKBFixed::VDKBFixed(char* name,VDKBEventContainer* outer): 
    VDKBEventContainer(name,outer->Owner())
{
  VDKBObject::object = this;
  Counter++;
  outerbox = outer;
  Init();
 }
/*
 */
void
VDKBFixed::Init()
{
  int t;
  for(t=0; vdkfixed_props[t]; t++)
    proplist.add(VDKBProperty(vdkfixed_props[t]));
  SetPropValue("have_grid",CHECK_TRUE);
  SetPropValue("h_grid_spacing","8");
  SetPropValue("v_grid_spacing","8");
  AddBox();
  // makes a pop menu
  popmenu = new VDKMenu(Owner());
  setsize = new VDKMenuItem(popmenu,"Set min size");
  selectparent =  new VDKMenuItem(popmenu,"Select parent container");
  SignalConnect(selectparent,"activate",
		&VDKBFixed::SelectParentContainer);
  delBox = new VDKMenuItem(popmenu,"Remove fixed");
  SignalConnect(delBox,"activate",&VDKBFixed::DelBox);
  SignalConnect(setsize,"activate",&VDKBFixed::SetBoxSize);
  EventConnect("button_press_event",
	       &VDKBFixed::OnButtonPressed);
  EventConnect("button_release_event",
	       &VDKBFixed::OnButtonReleased);
  EventConnect("expose_event",
	       &VDKBFixed::OnExpose);
  /* assign this as parent so this can receive signals  */
  popmenu->Parent(this);
  /*
    better add it to owner, so will be surely
    destroyed even if never popped
  */
  Owner()->AddItem(popmenu);
}
/*
 */
void
VDKBFixed::AddBox()
{
  container = gtk_fixed_new(); 
  gtk_container_add(GTK_CONTAINER(widget),container);
  gtk_widget_show(container);
}

/*
extra args unused
 */
void 
VDKBFixed::AddWidget(VDKObject* wid, int justify,
			int expand, int fill , int padding,
			bool forceArgs)
{
  int x = justify;
  int y = expand;
  /* Calculate real position according to grid & snapping */
  VDKString isTrue = CHECK_TRUE;
  if(GetProp("have_grid") == isTrue)
    {
      int horz_spacing = atoi((char*) GetProp("h_grid_spacing"));
      int vert_spacing = atoi((char*) GetProp("v_grid_spacing"));
      // snaps to grid
      x -= x % horz_spacing;
      y -= y % vert_spacing;
    }
  gtk_fixed_put(GTK_FIXED(container),wid->Widget(),x,y);
  // adding a nested box set border width to 0 
  VDKBObject* vdkbobj = dynamic_cast<VDKBObject*>(wid);
  if(vdkbobj)
    {
      VDKBEventBox* box = dynamic_cast<VDKBEventBox*>(vdkbobj);
      if(box)
	{
	  box->BorderWidth(0);
	  box->SetPropValue("BorderWidth","0");
	  sprintf(buff,"%d,%d",80,80);
	  box->SetSize(80,80);
	  box->SetPropValue(USIZE,buff);
	}
    }
  VDKBEventContainer::Add(wid,l_justify,expand,fill,padding);
}
/*
extra args used
 */
void 
VDKBFixed::Add(VDKObject* wid, int justify,
			int expand, int fill , int padding,
			bool forceArgs)
{
AddWidget(wid,justify,expand,fill,padding,forceArgs);
}
/*
  This method is called by global MakeWidget() in vdkb_design.cc
  MakeWidget() scans a table that maps class id's with each
  static MakeWidget() for each class. Class id's are generated
  during clicks on widget palette.
  On return:
  0 - successfull
  1 - unsupported widget
  2 - target is not a container
  3 - no active widget
*/
int 
VDKBFixed::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable frame counter
  // to ensure unicity
  int result = 0;
  VDKBFixed* fixed = NULL;
  if(owner->Active)
    {
      for(sprintf(buff,"fixed%d",VDKBFixed::Counter);
	  owner->ChildWithName(buff)!= (VDKObject*) NULL;
	  VDKBFixed::Counter++)
	sprintf(buff,"fixed%d",VDKBFixed::Counter);
      fixed = new VDKBFixed(buff,owner);
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      fixed->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      fixed->SetPropValue(EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(fixed,int(event->x),
				   int(event->y),
				   true,true,true);
	    }
	  else
	    container->AddWidget(fixed);
	  fixed->outerbox = container;
	}
      else if(! owner->Active->AddToParent(fixed,ev))
	// target isn't a container
	  result =  2;
    }
  else
    // no active widget
    result = 3;
  // 0 on success
  if(result && fixed)
     fixed->Destroy();
 return result;
}
////////////////////////////////////////////////////////////////////
/*
 */
void
VDKBFixed::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tBorderWidth:%s;", (char*) GetProp("BorderWidth"));
  fprintf(fp,"\n\thave_grid:%s;", (char*) GetProp("have_grid"));
  fprintf(fp,"\n\th_grid_spacing:%s;", (char*) GetProp("h_grid_spacing"));
  fprintf(fp,"\n\tv_grid_spacing:%s;", (char*) GetProp("v_grid_spacing"));
}

char*
VDKBFixed::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char temp[256];
  char bw[16];
  //char arg[128];
  // get name, mode and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  else
    source = new char[1024];
  sprintf(temp,"\n%s = new VDKFixed(this);",obj_name);
  strcpy(source,temp);
  // get size
  VDKPoint size = parser.Size(buffer);
  if(size.X() > 0 || size.Y() > 0)
    {
      sprintf(temp,"\n%s->SetSize(%d,%d);",obj_name,size.X(),size.Y());
      strcat(source,temp);
    }
  if(strcmp(obj_parent,NIHIL_PROP))
    sprintf(temp,"\n%s->Add(%s",obj_parent,obj_name);
  else
    sprintf(temp,"\nAdd(%s",obj_name);
  strcat(source,temp);
  // prepares arguments for add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(temp,",%s,%s,%s,%s);",
	      justify,expand,fill,padding);
      strcat(source,temp);
    }
  else
    {
      sprintf(temp,");");
      strcat(source,temp);
    }
  if(parser.GetParam(bw,buffer,"BorderWidth:") &&
     strcmp(bw,NIHIL_PROP))
    {
      sprintf(temp,"\n%s->BorderWidth(%s);",obj_name,bw);
      strcat(source,temp);
    }
  return source;
}
/*
 */
bool
VDKBFixed::CreateWidget(VDKBGuiForm* owner, char* buffer,VDKBParser& parser)
{
  char obj_name[128];
  char obj_parent[128];
  VDKBFixed* box;
  // get name, mode and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // get mode and size
  VDKPoint size = parser.Size(buffer);
  // get packing args
  int justification = l_justify;
  int expand=0,fill=0,padding=0;
  int bw; // border width
  char arg[32];
  if(parser.GetParam(arg,buffer,PROP_JUSTIFY_INTERNAL))
     justification = atoi(arg);
  if(parser.GetParam(arg,buffer,PROP_EXPAND_INTERNAL))
    expand = atoi(arg);
  if(parser.GetParam(arg,buffer,PROP_FILL_INTERNAL))
    fill = atoi(arg);
  if(parser.GetParam(arg,buffer,PROP_PADDING_INTERNAL))
    padding = atoi(arg);
  if(parser.GetParam(arg,buffer,"BorderWidth:") &&
     strcmp(arg,NIHIL_PROP))
    bw = atoi(arg);
  else
    bw = -1;
  // no parent, widget will be added to owner form innerbox
  if(!strcmp(obj_parent,NIHIL_PROP))
    {
      box = new VDKBFixed(obj_name,owner->InnerBox());
      owner->AddWidget(box,justification,expand,fill,padding);
    }
  // get parent container address
  else
    {
      VDKObject* p = owner->ChildWithName(obj_parent);
      VDKBEventContainer* container = p ? 
	dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
      if(container)
	{
	  box = new VDKBFixed(obj_name,container);
	  container->AddWidget(box,justification,expand,fill,padding,true);
	  box->outerbox = container;
	}
      else
	// FIX ME: user warning
	return false;
    }
  // call ancestor to set common properties
  VDKBObject::CreateWidget(box,buffer,parser);
  if(size.X() > 0 || size.Y() > 0)
    box->Object()->SetSize(size.X(),size.Y());
  if(bw >= 0)
    {
      box->BorderWidth(bw);
      box->SetPropValue("BorderWidth",arg);
    }
  if(parser.GetParam(arg,buffer,"have_grid:") && strcmp(arg,NIHIL_PROP))
      box->SetPropValue("have_grid",arg);
  if(parser.GetParam(arg,buffer,"h_grid_spacing:") && strcmp(arg,NIHIL_PROP))
      box->SetPropValue("h_grid_spacing",arg);
  if(parser.GetParam(arg,buffer,"_grid_spacing:") && strcmp(arg,NIHIL_PROP))
      box->SetPropValue("v_grid_spacing",arg);
  return true;
}

/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
 */
VDKObjectContainer* 
VDKBFixed::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  bool have_grid = false;
  int horz_spacing = 1,vert_spacing = 1;
  VDKString isTrue = CHECK_TRUE;
  inspector = isp;
  have_grid = GetProp("have_grid") == isTrue;
  if(have_grid)
    {
      horz_spacing = atoi((char*) GetProp("h_grid_spacing"));
      vert_spacing = atoi((char*) GetProp("v_grid_spacing"));
    }
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  // FIX ME: lang support
  VDKTable* table = new VDKTable(inspector,3,2,true);
  setgrid = 
    new VDKCustomButton(inspector,"Set grid options");
  table->AddToCell(setgrid,0,0);
  setgrid->Parent(this);
  SignalConnect(setgrid,"clicked",&VDKBFixed::OnSetGrid);
  gridcb = new VDKCheckButton(inspector,"have grid");
  gridcb->Checked = have_grid;
  table->AddToCell(gridcb,0,1);
  table->AddToCell(new VDKLabel(inspector,"V spacing: "),1,0);
  vspace = new VDKSpinButton(inspector, vert_spacing, 1, 50 , 1 ,0 );
  vspace->SetSize(60,-1);
  table->AddToCell(vspace,1,1);
  table->AddToCell(new VDKLabel(inspector,"H spacing: "),2,0);
  hspace = new VDKSpinButton(inspector, horz_spacing, 1, 50 , 1 ,0 );
  hspace->SetSize(60,-1);
  table->AddToCell(hspace,2,1);
  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}
//////////////////////////////////////////////////////
// These response methods actually change both 
// properties on widget and gui widget properties
//////////////////////////////////////////////////////
/*
 */
bool
VDKBFixed::OnSetGrid(VDKObject*)
{
  SetPropValue("have_grid", gridcb->Checked ? CHECK_TRUE : CHECK_FALSE);
  sprintf(buff,"%d",(int) vspace->ValueAsInt);
  SetPropValue("v_grid_spacing",buff);
  sprintf(buff,"%d",(int) hspace->ValueAsInt);
  SetPropValue("h_grid_spacing",buff);
  inspector->FormNeedToBeChanged();
  if(gridcb->Checked)
    fixed_draw_grid (Container(), (int) hspace->ValueAsInt,
		   (int) vspace->ValueAsInt, 1); // dots
  return true;
}

