/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998,1999 Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * Based on VDK Library 
 * Copyright (C) 1998, Mario Motta
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */ 

#include <vdkb_evcontain.h>
#include <vdkb_labelbutton.h>
#include <vdkb_form.h>
#include <vdk/vdk.h>
#include <vdkb_utils.h>
#include <vdkb_parser.h>
#include <vdkb_prjman.h>
#include <vdkb_objinspect.h>
#include <stdlib.h>
#include <vdkb_frame.h>
#include <vdkb_evbox.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBFrame
// put here vdk class name string
#define VDK_CLASS "VDKFrame"
// put here vdk class name 
#define VDK_ANCESTOR  VDKFrame
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "frame"

static char buff[128];
static char *shadows[] = 
{ 
"shadow_none","shadow_in","shadow_out","shadow_etched_in",
"shadow_etched_out",0 
};
static char *aligns[] = 
{ 
"l_justify","c_justify","r_justify",0
};

int CLASS::Counter = 0;

/* 
 properties 
 */
char* vdkframe_props[] =
{ 
"BorderWidth",
"Label",
"Shadow",
"Align",
0
};

DEFINE_EVENT_LIST(CLASS,VDKBEventContainer);
DEFINE_SIGNAL_LIST(CLASS,VDKBEventContainer);


/*
 */
bool
CLASS:: DelBox(VDKObject* sender)
{
  // destroy inner gtk+ placeholder widget
  gtk_widget_destroy(container);
  // call ancestor delete box
  VDKBEventContainer::DelBox(sender);
  // notify to inspector that object was deleted
  VDKBGuiForm* ownerform = dynamic_cast<VDKBGuiForm*>(Owner());
  if(ownerform)
    {
      VDKBProjectManager* prjman = 
	dynamic_cast<VDKBProjectManager*>(ownerform->Owner());
      if(prjman && prjman->objInspector)
	prjman->objInspector->SetActive(NULL);
    }
return true;
}

/*
 */
bool 
CLASS::SetBoxSize(VDKObject* sender)
{
  VDKBGuiForm* ownerform = dynamic_cast<VDKBGuiForm*>(Owner());
  if(ownerform)
    {
      ownerform->SetBoxSize(NULL);
    }
  return true;
}
//===========================================
/*
 */
CLASS::CLASS(char* name, VDKForm* owner): 
  VDKBEventContainer(name,owner)
{
  Counter++;
  VDKBObject::object = this;
  VDKBEventContainer::mode = mode;
  Init();
}

/*
 */
CLASS::CLASS(char* name,VDKBEventContainer* outer): 
    VDKBEventContainer(name,outer->Owner())
{
  VDKBObject::object = this;
  Counter++;
  outerbox = outer;
  Init();
 }
/*
 */
void
CLASS::Init()
{
  int t;
  // add  frame properties
  // names. (Others props are prepended  by VDKBObject constructor)
  for(t=0; vdkframe_props[t]; t++)
    proplist.add(VDKBProperty(vdkframe_props[t]));
  if(name)
    SetPropValue("Label",name);
  AddBox(name);
  //AddCanvas(w,h);
  // makes a pop menu
  popmenu = new VDKMenu(Owner());
  setsize = new VDKMenuItem(popmenu,"Set min size");
  selectparent =  new VDKMenuItem(popmenu,"Select parent container");
  SignalConnect(selectparent,"activate",
		&CLASS::SelectParentContainer);
  delBox = new VDKMenuItem(popmenu,"Remove frame");
  SignalConnect(delBox,"activate",&CLASS::DelBox);
  SignalConnect(setsize,"activate",&CLASS::SetBoxSize);

  CONNECT_COMMON_EVENTS;
  /* assign this as parent so this can receive signals  */
  popmenu->Parent(this);
  /*
    better add it to owner, so will be surely
    destroyed even if never popped
  */
  Owner()->AddItem(popmenu);
}
/*
 */
void
CLASS::AddBox(char* name)
{
  container = gtk_frame_new(name); 
  gtk_container_add(GTK_CONTAINER(widget),container);
  gtk_widget_show(container);
}

/*
extra args unused
 */
void 
CLASS::AddWidget(VDKObject* wid, int justify,
			int expand, int fill , int padding,
			bool forceArgs)
{
  gtk_container_add(GTK_CONTAINER(container),wid->Widget());
  // adding a nested box set border width to 10 
  VDKBObject* vdkbobj = dynamic_cast<VDKBObject*>(wid);
  if(vdkbobj)
    {
      VDKBEventBox* box = dynamic_cast<VDKBEventBox*>(vdkbobj);
      if(box)
	{
	  box->BorderWidth(0);
	  box->SetPropValue("BorderWidth","0");
	}
    }
  VDKBEventContainer::Add(wid,l_justify,expand,fill,padding);
}
/*
extra args used
 */
void 
CLASS::Add(VDKObject* wid, int justify,
			int expand, int fill , int padding,
			bool forceArgs)
{
AddWidget(wid);
}
/*
  This method is called by global MakeWidget() in vdkb_design.cc
  MakeWidget() scans a table that maps class id's with each
  static MakeWidget() for each class. Class id's are generated
  during clicks on widget palette.
  On return:
  0 - successfull
  1 - unsupported widget
  2 - target is not a container
  3 - no active widget
*/
int 
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable frame counter
  // to ensure unicity
  int result = 0;
  CLASS* frame = NULL;
  VDKBEventBox *box1 = NULL;
  if(owner->Active)
    {
      for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
	  owner->ChildWithName(buff)!= (VDKObject*) NULL;
	  CLASS::Counter++)
	sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      frame = new CLASS(buff,owner);
      // add a box to frame
      for(sprintf(buff,"VBox%d",VDKBEventBox::Counter);
	  owner->ChildWithName(buff) != (VDKObject*) NULL;
	  VDKBEventBox::Counter++)
	sprintf(buff,"VBox%d",VDKBEventBox::Counter);
      box1 = new VDKBEventBox(buff,frame);
      box1->SetPropValue(JUSTIFY_INTERNAL,"l_justify");
      frame->AddWidget(box1,l_justify,true,true,true,true);
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      frame->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      frame->SetPropValue(EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(frame,int(event->x),
				   int(event->y),
				   true,true,true);
	    }
	  else
	    container->AddWidget(frame);
	  frame->outerbox = container;
	}
      else if(! owner->Active->AddToParent(frame,ev))
	// target isn't a container
	  result =  2;
    }
  else
    // no active widget
    result = 3;
  // 0 on success
  if(result && frame)
     frame->Destroy();
 return result;
}
////////////////////////////////////////////////////////////////////
/*
 */
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tBorderWidth:%s;", (char*) GetProp("BorderWidth"));
  fprintf(fp,"\n\tLabel:\"%s\";", (char*) GetProp("Label"));
  fprintf(fp,"\n\tShadow:%s;", (char*) GetProp("Shadow"));
  fprintf(fp,"\n\tAlign:%s;", (char*) GetProp("Align"));
}

char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char shadow[32];
  char temp[256];
  char bw[16];
  char label[128];
  char arg[128];
  // get name, mode and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  else
    source = new char[1024];
  strcpy(shadow,"shadow_etched_in");
  // get label
  if(parser.GetParam(label,buffer, PROP_LABEL) && strcmp(arg,NIHIL_PROP))
    {
      sprintf(temp,"\n%s = new %s(this,\"%s\",v_box,%s);"
	  ,obj_name,VDK_CLASS,label,shadow);
      strcpy(source,temp);
    }
  else
    {
      sprintf(temp,"\n%s = new %s(this,NULL,v_box,%s);"
	  ,obj_name,VDK_CLASS,shadow);
      strcpy(source,temp);
    }
  // get shadow
  if(parser.GetParam(arg,buffer,"Shadow:") && strcmp(arg,NIHIL_PROP))
    {
      int ndx = atoi(arg);
      ndx = (ndx >= 0) && (ndx <= 4) ? ndx : 0;
      sprintf(temp,"\n%s->Shadow = %s;", obj_name,shadows[ndx]);
      strcat(source,temp);
    }
  // get align
  if(parser.GetParam(arg,buffer,"Align:") && strcmp(arg,NIHIL_PROP))
    {
      int ndx = atoi(arg);
      ndx = (ndx >= 0) && (ndx <= 2) ? ndx : 0;
      sprintf(temp,"\n%s->Align = %s;",  
	      obj_name,aligns[ndx]);
      strcat(source,temp);
    }
  // get size
  VDKPoint size = parser.Size(buffer);
  if(size.X() > 0 || size.Y() > 0)
    {
      sprintf(temp,"\n%s->SetSize(%d,%d);",obj_name,size.X(),size.Y());
      strcat(source,temp);
    }
  if(strcmp(obj_parent,NIHIL_PROP))
    sprintf(temp,"\n%s->Add(%s",obj_parent,obj_name);
  else
    sprintf(temp,"\nAdd(%s",obj_name);
  strcat(source,temp);
  // prepares arguments for add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(temp,",%s,%s,%s,%s);",
	      justify,expand,fill,padding);
      strcat(source,temp);
    }
  else
    {
      sprintf(temp,");");
      strcat(source,temp);
    }
  
  if(parser.GetParam(bw,buffer,"BorderWidth:") &&
     strcmp(bw,NIHIL_PROP))
    {
      sprintf(temp,"\n%s->BorderWidth(%s);",obj_name,bw);
      strcat(source,temp);
    }
  return source;
}
/*
 */
bool
CLASS::CreateWidget(VDKBGuiForm* owner, char* buffer,VDKBParser& parser)
{
  char obj_name[128];
  char obj_parent[128];
  CLASS* box;
  // get name, mode and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // get mode and size
  VDKPoint size = parser.Size(buffer);
  // get packing args
  int justification = l_justify;
  int expand=0,fill=0,padding=0;
  int bw; // border width
  char arg[32];
  if(parser.GetParam(arg,buffer,PROP_JUSTIFY_INTERNAL))
     justification = atoi(arg);
  if(parser.GetParam(arg,buffer,PROP_EXPAND_INTERNAL))
    expand = atoi(arg);
  if(parser.GetParam(arg,buffer,PROP_FILL_INTERNAL))
    fill = atoi(arg);
  if(parser.GetParam(arg,buffer,PROP_PADDING_INTERNAL))
    padding = atoi(arg);
  if(parser.GetParam(arg,buffer,"BorderWidth:") &&
     strcmp(arg,NIHIL_PROP))
    bw = atoi(arg);
  else
    bw = -1;
  // no parent, widget will be added to owner form innerbox
  if(!strcmp(obj_parent,NIHIL_PROP))
    {
      box = new CLASS(obj_name,owner->InnerBox());
      owner->AddWidget(box,justification,expand,fill,padding);
    }
  // get parent container address
  else
    {
      VDKObject* p = owner->ChildWithName(obj_parent);
      VDKBEventContainer* container = p ? 
	dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
      if(container)
	{
	  box = new CLASS(obj_name,container);
	  container->AddWidget(box,justification,expand,fill,padding,true);
	  box->outerbox = container;
	}
      else
	// FIX ME: user warning
	return false;
    }
  // call ancestor to set common properties
  VDKBObject::CreateWidget(box,buffer,parser);
  if(size.X() > 0 || size.Y() > 0)
    box->Object()->SetSize(size.X(),size.Y());
  // label
  if(parser.GetParam(arg,buffer, PROP_LABEL) &&
     strcmp(arg,NIHIL_PROP))
    {
      box->SetPropValue("Label",arg);
      gtk_frame_set_label(GTK_FRAME(box->container),arg);
    }
  // shadow
  if(parser.GetParam(arg,buffer,"Shadow:") &&
     strcmp(arg,NIHIL_PROP))
    {
      box->SetPropValue("Shadow",arg);
      int sh = atoi(arg);
      gtk_frame_set_shadow_type(GTK_FRAME(box->container),
				(GtkShadowType) sh);
    }
  // align
  if(parser.GetParam(arg,buffer,"Align:") &&
     strcmp(arg,NIHIL_PROP))
    {
      box->SetPropValue("Align",arg);
      int sh = atoi(arg);
      gtk_frame_set_label_align(GTK_FRAME(box->container),
				0.0+0.5*sh,0.0);
    }
  if(bw >= 0)
    {
      box->BorderWidth(bw);
      box->SetPropValue("BorderWidth",arg);
    }
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
This is called by object inspector when a widget
is selected by user, thus to allow widget to set
his own property controls.
 */

VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp)
{
  inspector = isp;
  VDK_ANCESTOR* tframe = 
    new VDK_ANCESTOR(inspector,NULL,v_box,shadow_etched_in);
  VDKTable *table = new VDKTable(inspector,4,2);
  table->SetSize(219,-1);
  VDKLabelButton* button = 
    new VDKLabelButton(inspector,"Set border width");
  table->AddToCell(button,0,0);
  int bw = atoi( (char*) GetProp("BorderWidth"));
  border = new VDKSpinButton(inspector, bw, 0, 20 , 1 ,0 );
  border->SetSize(60,-1);
  table->AddToCell(border,0,1);
  button->Parent(this);
  SignalConnect(button,"clicked",&CLASS::OnBorderWidth);

  table->AddToCell(new VDKLabel(inspector,"Label: "),1,0);
  label = new VDKEntry(inspector,0,(char*) GetProp("Label"));
  label->SetSize(100,-1);
  table->AddToCell(label,1,1);
  label->Parent(this);
  SignalConnect(label,"activate",&CLASS::OnSetLabel);
  
  // shadow
  set = new VDKLabelButton(inspector,"Set shadow/align");
  table->AddToCell(set,2,0);
  shadow = new VDKCombo(inspector,NULL);
  shadow->SetSize(100,-1);
  StringList sl;
  int t = 0;
  for(;shadows[t];t++)
    sl.add(VDKString(shadows[t]));
  shadow->PopdownStrings = sl;
  table->AddToCell(shadow,2,1);
  int r = atoi(GetProp("Shadow"));
  shadow->SelectItem(r);
  set->Parent(this);
  SignalConnect(set,"clicked",&CLASS::OnSetShadowAndAlign);
  // align
  align = new VDKCombo(inspector,NULL);
  align->SetSize(100,-1);
  StringList sa;
  for(t=0;aligns[t];t++)
    sa.add(VDKString(aligns[t]));
  align->PopdownStrings = sa;
  r = atoi(GetProp("Align"));
  align->SelectItem(r);
  align->SetSize(100,-1);
  table->AddToCell(align,3,1);
  tframe->Add(table,l_justify,false,false,false);
  return tframe;
}
/*
 */
bool
CLASS::OnBorderWidth(VDKObject*)
{
  sprintf(buff,"%d",(int) border->ValueAsInt);
  SetPropValue("BorderWidth",buff);
  BorderWidth((int) border->ValueAsInt);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetLabel(VDKObject*)
{
  if(strlen(label->Text)>0)
    {
      sprintf(buff,"%s",(char*) label->Text);
      SetPropValue("Label",buff);
      gtk_frame_set_label(GTK_FRAME(container),buff);
      inspector->FormNeedToBeChanged();
    }
  return true;
}
/*
 */
bool
CLASS:: OnSetShadowAndAlign(VDKObject*)
{
  int sel = shadow->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  SetPropValue("Shadow",buff);
  int shd = atoi(buff);
  gtk_frame_set_shadow_type(GTK_FRAME(container),(GtkShadowType) shd);
  
  sel = align->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  SetPropValue("Align",buff);
  int alg = atoi(buff);
  gtk_frame_set_label_align (GTK_FRAME(container),0.0+alg*0.5,0.0);
  inspector->FormNeedToBeChanged();
  return true;
}





