/*
 * ===========================
 * VDK Builder
 * Version 0.1
 * Revision 0.3
 * July 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_grid.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_tabledlg.h>
#include <vdkb_scrolled.h>
#include <vdkb_fixed.h>

/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBGrid
// put here vdk class name string
#define VDK_CLASS "VDKGrid"
// put here vdk class name 
#define VDK_ANCESTOR  VDKGrid
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "grid"
// for common events

static char buff[128];

int CLASS::Counter = 0;
/* 
 properties 
 */
char* vdkgrid_props[] =
{ 
"ColWidth",
"RowHeight",
"Rows",
"Columns",
"ColVisible",
"RowVisible",
"Editable",
"Viewport",
0
};

//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDK_ANCESTOR);
DEFINE_EVENT_LIST(CLASS,VDK_ANCESTOR);
///////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, VDKForm* owner, int rows, int cols,
		   bool editable):
  VDK_ANCESTOR(owner,rows,cols,editable,name),VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
   for(t=0; vdkgrid_props[t]; t++)
    proplist.add(VDKBProperty(vdkgrid_props[t]));

  SetPropValue(JUSTIFY_INTERNAL,"l_justify");
  SetPropValue(EXPAND_INTERNAL,CHECK_TRUE);
  SetPropValue("_Fill",CHECK_TRUE);
  sprintf(buff,"%d",rows);
  SetPropValue("Rows",buff);
  sprintf(buff,"%d",cols);
  SetPropValue("Columns",buff);
  SetPropValue("Editable",editable ? CHECK_TRUE : CHECK_FALSE);
  SetPropValue("Viewport",CHECK_FALSE);
  SetPropValue("ColVisible",CHECK_TRUE);
  SetPropValue("RowVisible",CHECK_TRUE);
  // assign this to VDKBObject <object> member.
  object = this;
  // connects events.
  CONNECT_COMMON_EVENTS;
  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
  // connect global selection button in order to stop clicked signal
  // emission too.
  GtkSheet* sheet = GTK_SHEET(Object()->Widget());
  gtk_widget_set_sensitive(sheet->button,0);
  gtk_signal_connect(GTK_OBJECT(sheet->button),"clicked",
		     GTK_SIGNAL_FUNC(CLASS::OnGlobalButtonClicked),
		     (gpointer) NULL);
}

void 
CLASS::OnGlobalButtonClicked(GtkWidget* wid, gpointer gp)
{
  gtk_signal_emit_stop_by_name(GTK_OBJECT(wid), "clicked");  
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tRows:%s;", (char*) GetProp("Rows"));
  fprintf(fp,"\n\tColumns:%s;", (char*) GetProp("Columns"));
  fprintf(fp,"\n\tEditable:%s;", (char*) GetProp("Editable"));
  bool viewport = dynamic_cast<VDKBScrolled*>(parentobj) != NULL;
  fprintf(fp,"\n\tViewport:%s;", viewport ? CHECK_TRUE :CHECK_FALSE);
  fprintf(fp,"\n\tRowVisible:%s;", (char*) GetProp("RowVisible"));
  fprintf(fp,"\n\tColVisible:%s;", (char*) GetProp("ColVisible"));    
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  int r = 8,c = 4;
  // autogenerate first suitable label button counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  Vdkb_tabledlgForm* dlg = new Vdkb_tabledlgForm(owner,NULL,&r,&c);
  dlg->Setup();
  dlg->ShowModal();
  // user cancelled
  if (!r || !c)
    return 3;
  CLASS* Grid = new CLASS(buff,owner,r,c);
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      Grid->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      Grid->SetPropValue(EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(Grid,int(event->x),
				   int(event->y),
				   true,true,true);
	    }
	  else
	    container->AddWidget(Grid,l_justify,true,true,true,true);
	}
      else if(! owner->Active->AddToParent(Grid,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  // rows, columns
  int rows = 8,cols = 4;
  if(parser.GetParam(arg,buffer,"Rows:"))
    rows = atoi(arg);
  if(parser.GetParam(arg,buffer,"Columns:"))
    cols = atoi(arg);
  bool  editable = false;
  if(parser.GetParam(arg,buffer,"Editable:"))
    editable = !strcmp(arg,CHECK_TRUE);
  sprintf(tmp,"\n%s = new %s(this,%d,%d,%s);", 
	  obj_name,
	  VDK_CLASS,
	  rows,cols,
	  editable ? CHECK_TRUE : CHECK_FALSE);
  strcpy(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  bool viewport = false;
  if(parser.GetParam(arg,buffer,"Viewport:"))
    viewport = !strcmp(arg,CHECK_TRUE);
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->%s(%s,%s,%s,%s,%s);",
	      obj_parent,
	      viewport ? "AddWithoutViewport" : "Add",
	      obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->%s(%s);",
	      obj_parent,
	      viewport ? "AddWithoutViewport" : "Add",
	      obj_name);
      strcat(source,tmp);
    }

  if(parser.GetParam(arg,buffer,"RowVisible:") && 
     strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->RowTitleVisible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  if(parser.GetParam(arg,buffer,"ColVisible:") && 
     strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->ColumnTitleVisible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
char obj_name[128];
char obj_parent[128];
char arg[32];
CLASS* Grid;
// get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:")
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  int rows = 8,cols = 4;
  if(parser.GetParam(arg,buffer,"Rows:"))
    rows = atoi(arg);
  if(parser.GetParam(arg,buffer,"Columns:"))
    cols = atoi(arg);
  bool  editable = false;
  if(parser.GetParam(arg,buffer,"Editable:"))
    editable = !strcmp(arg,"true");
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      Grid = new CLASS(obj_name,owner,rows,cols,editable);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(Grid,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  Grid->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) Grid->GetProp(EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) Grid->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) Grid->GetProp("_Padding");
      padding = atoi(p);
      if(parser.GetParam(arg,buffer,"RowVisible:") &&
	 strcmp(arg,NIHIL_PROP))
	{
	  Grid->SetPropValue("RowVisible",arg);
	  Grid->RowTitleVisible = !strcmp(arg,CHECK_TRUE);
	}
      if(parser.GetParam(arg,buffer,"ColVisible:") &&
	 strcmp(arg,NIHIL_PROP))
	{
	  Grid->SetPropValue("ColVisible",arg);
	  Grid->ColumnTitleVisible = !strcmp(arg,CHECK_TRUE);
	}
 
      Grid->ColumnTitleVisible = !strcmp(arg,CHECK_TRUE);
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(Grid,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}

/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  VDKString True = CHECK_TRUE;
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  VDKTable *table = new VDKTable(inspector,2,2);
  table->SetSize(219,-1);

  rowvisible = new VDKCheckButton(inspector,"Rows visibles");
  table->AddToCell(rowvisible,0,0);
  rowvisible->Checked = GetProp("RowVisible") == True;
  rowvisible->Parent(this);
  SignalConnect(rowvisible,"toggled",&CLASS::OnSetRowVisible);

  colvisible = new VDKCheckButton(inspector,"Cols visibles");
  table->AddToCell(colvisible,0,1);
  colvisible->Checked = GetProp("ColVisible") == True;
  colvisible->Parent(this);
  SignalConnect(colvisible,"toggled",&CLASS::OnSetColVisible);

  editable_check = new VDKCheckButton(inspector,"Editable");
  table->AddToCell(editable_check,1,0);
  editable_check->Checked = GetProp("Editable") == True;
  editable_check->Parent(this);
  SignalConnect(editable_check,"toggled",&CLASS::OnSetEditable);

  bframe->Add(table,l_justify,false,false,false);
  return bframe;
}

/*
 */
bool
CLASS::OnSetRowVisible(VDKObject*)
{
  SetPropValue("RowVisible", rowvisible->Checked ? CHECK_TRUE : CHECK_FALSE);
  RowTitleVisible = rowvisible->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}

/*
 */
bool
CLASS::OnSetColVisible(VDKObject*)
{
  SetPropValue("ColVisible", colvisible->Checked ? CHECK_TRUE : CHECK_FALSE);
  ColumnTitleVisible = colvisible->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetEditable(VDKObject*)
{
  SetPropValue("Editable", editable_check->Checked ? CHECK_TRUE : CHECK_FALSE);
  inspector->FormNeedToBeChanged();
  return true;
}




