/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0 
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_pbar.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBProgressBar
// put here vdk class name string
#define VDK_CLASS "VDKProgressBar"
// put here vdk class name 
#define VDK_ANCESTOR  VDKProgressBar
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "progressbar"
//=================================================================
static char buff[128];

// used to autogenerate default
// button  names and captions

int CLASS::Counter = 0;
/* 
 label button properties names
 */
char* vdkpbar_props[] =
{ 
"BlockCount","BarStyle","BarOrientation","ActivityMode",
"Min","Max",0
};

//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDKBEvProgressBar);
DEFINE_EVENT_LIST(CLASS,VDKBEvProgressBar);
//////////////////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, 
			       VDKForm* owner,
			       float min,
			       float max):
  VDKBEvProgressBar(owner,min,max),
  VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties list 
  // (Others props are prepended  by VDKBObject constructor)
  for(t=0; vdkpbar_props[t]; t++)
    proplist.add(VDKBProperty(vdkpbar_props[t]));
  sprintf(buff,"%f",min);
  SetPropValue("Min",buff);
  sprintf(buff,"%f",max);
  SetPropValue("Max",buff);
  // progress bar not expand
  SetPropValue(JUSTIFY_INTERNAL,"l_justify");
  SetPropValue( EXPAND_INTERNAL,CHECK_FALSE);
  SetPropValue("_Fill",CHECK_FALSE);
  SetPropValue("_Padding",CHECK_FALSE);
  // connects events.
 CONNECT_COMMON_EVENTS;
  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
  // set bar to max value
  Update(max);
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tMin:%s;",(char*) GetProp("Min"));
  fprintf(fp,"\n\tMax:%s;",(char*) GetProp("Max"));
  fprintf(fp,"\n\tBlockCount:%s;",(char*) GetProp("BlockCount"));
  fprintf(fp,"\n\tBarStyle:%s;",(char*) GetProp("BarStyle"));
  fprintf(fp,"\n\tBarOrientation:%s;",(char*) GetProp("BarOrientation"));
  fprintf(fp,"\n\tActivityMode:%s;",(char*) GetProp("ActivityMode"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label bar counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  CLASS* bar = new CLASS(buff,owner);
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  bool flag = false;
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(owner->Owner());
	  if(prjman && prjman->objInspector)
	    flag = prjman->objInspector->preview->Checked;
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      bar->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      bar->SetPropValue( EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(bar,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(bar,l_justify,false,false,false,true);
	}
      else if(! owner->Active->AddToParent(bar,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  double min = 0.0,
    max = 1.0;

  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  // get default values
  if(parser.GetParam(arg,buffer,"Min:"))
    min = atof(arg);
  if(parser.GetParam(arg,buffer,"Max:"))
    max = atof(arg);
  source = new char[4096];
  sprintf(tmp,"\n%s = new %s(this,%f,%f);",
	  obj_name,
	  VDK_CLASS,
	  min,
	  max);
  strcpy(source,tmp);
  
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  if(parser.GetParam(arg,buffer,"BlockCount:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->BlockCount =  %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  if(parser.GetParam(arg,buffer,"BarStyle:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->BarStyle = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  if(parser.GetParam(arg,buffer,"BarOrientation:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->BarOrientation = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  if(parser.GetParam(arg,buffer,"ActivityMode:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->ActivityMode = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
  char obj_name[128];
  char obj_parent[128];
  double min = 0.0,
    max = 1.0;
  char arg[64];
  CLASS* bar;
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  // get default values
  if(parser.GetParam(arg,buffer,"Min:"))
    min = atoi(arg);
  if(parser.GetParam(arg,buffer,"Max:"))
    max = atof(arg);
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      bar = new CLASS(obj_name,owner,min,max);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(bar,buffer,parser);
      // prepares packing args based on those loaded by 
      if(parser.GetParam(arg,buffer,"BlockCount:") && strcmp(arg,NIHIL_PROP))
	{
	  bar->SetBlockCount(atoi(arg));
	  bar->SetPropValue("BlockCount",arg);
	}
      if(parser.GetParam(arg,buffer,"BarStyle:") && strcmp(arg,NIHIL_PROP))
	{
	  bar->SetBarStyle( atoi(arg));
	  bar->SetPropValue("BarStyle",arg);
	}
      if(parser.GetParam(arg,buffer,"BarOrientation:") && 
	 strcmp(arg,NIHIL_PROP))
	{
	  bar->SetBarOrientation( atoi(arg));
	  bar->SetPropValue("BarOrientation",arg);
	}
      if(parser.GetParam(arg,buffer,"ActivityMode:") && 
	 strcmp(arg,NIHIL_PROP))
	{
	  bar->SetActivityMode(atoi(arg));
	  bar->SetPropValue("ActivityMode",arg);
	}
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  bar->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) bar->GetProp( EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) bar->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) bar->GetProp("_Padding");
      padding = atoi(p);
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(bar,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
char* pstyles[] = {"continuous","discrete",0};
char* porient[] = { "left to right","right to left",
		    "bottom to top","top to bottom", 0};
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  inspector = isp;
  VDKLabelButton* button;
  int bc=0,t=0;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,
				  v_box,shadow_etched_in);

  VDKTable *table = new VDKTable(inspector,6,2);
  table->SetSize(219,-1);

  table->AddToCell(new VDKLabel(inspector,"Minimum value"),0,0);
  min = new VDKEntry(inspector,0,(char*) GetProp("Min"));
  min->SetSize(100,-1);
  table->AddToCell(min,0,1);
  min->Parent(this);
  SignalConnect(min,"activate",&CLASS::OnSetMin);

  table->AddToCell(new VDKLabel(inspector,"Maximum value"),1,0);
  max = new VDKEntry(inspector,0,(char*) GetProp("Max"));
  max->SetSize(100,-1);
  table->AddToCell(max,1,1);
  max->Parent(this);
  SignalConnect(max,"activate",&CLASS::OnSetMax);

  button = new VDKLabelButton(inspector,"Set BlockCount");
  table->AddToCell(button,2,0);
  SignalConnect(button,"clicked",&CLASS::OnSetBlockCount);
  bc = atoi( (char*) GetProp("BlockCount"));
  blockcount = new VDKSpinButton(inspector, bc, 0, 100 , 1 ,0 );
  blockcount->SetSize(100,-1);
  table->AddToCell(blockcount,2,1);

  button = new VDKLabelButton(inspector,"Set bar style");
  table->AddToCell(button,3,0);
  SignalConnect(button,"clicked",&CLASS::OnSetBarStyle);
  button->Parent(this);
  bc = atoi( (char*) GetProp("BarStyle"));
  barstyle = new VDKCombo(inspector,NULL);
  barstyle->SetSize(100,-1);
  StringList sl;
  for(t = 0;pstyles[t];t++)
    sl.add(VDKString(pstyles[t]));
  barstyle->PopdownStrings = sl;
  barstyle->SelectItem(bc);
  table->AddToCell(barstyle,3,1);

  sl.flush();
  button = new VDKLabelButton(inspector,"Set orientation");
  table->AddToCell(button,4,0);
  SignalConnect(button,"clicked",&CLASS::OnSetBarOrient);
  button->Parent(this);
  bc = atoi( (char*) GetProp("BarOrientation"));
  barorient = new VDKCombo(inspector,NULL);
  barorient->SetSize(100,-1);
  for(t=0;porient[t];t++)
    sl.add(VDKString(porient[t]));
  barorient->PopdownStrings = sl;
  barorient->SelectItem(bc);
  table->AddToCell(barorient,4,1);

  activitymode = new VDKCheckButton(inspector,"Activity mode");
  table->AddToCell(activitymode,5,1);
  activitymode->Parent(this);
  SignalConnect(activitymode,"toggled",
		&CLASS::OnSetActivityMode);
  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}

/*
 */
bool
CLASS::OnSetMin(VDKObject*)
{
  if(strlen(min->Text) > 0)
    SetPropValue("Min",(char*) min->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetMax(VDKObject*)
{
  if(strlen(max->Text) > 0)
    SetPropValue("Max",(char*) max->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetBlockCount(VDKObject*)
{
  sprintf(buff,"%d",(int) blockcount->ValueAsInt);
  SetPropValue("BlockCount",buff);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetBarStyle(VDKObject*)
{
  int sel = barstyle->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  int style = atoi(buff);
  SetPropValue("BarStyle",buff);
  SetBarStyle(style);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetBarOrient(VDKObject*)
{
  int sel = barorient->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  int orient = atoi(buff);
  SetPropValue("BarOrientation",buff);
  SetBarOrientation( orient);
  inspector->FormNeedToBeChanged();
  return true;
}

/*
 */
bool
CLASS::OnSetActivityMode(VDKObject*)
{
  SetPropValue("ActivityMode", activitymode->Checked ? CHECK_TRUE : CHECK_FALSE);
  SetActivityMode( activitymode->Checked ? true : false);
  inspector->FormNeedToBeChanged();
  return true;
}
