/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_pixbutton.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <stdlib.h>
#include <vdk/FileDialog.h>
#include <vdkb_prjman.h>
#include <vdkxpmbrowser.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBPixmapButton
// put here vdk class name string
#define VDK_CLASS "VDKPixmapButton"
// put here vdk class name 
#define VDK_ANCESTOR  VDKPixmapButton
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "button"

static char buff[128];

// used to autogenerate default
// labelbutton  names and captions

int CLASS::Counter = 0;
/* 
 label button properties names
 */
char* vdkpixmapbutton_props[] =
{ 
"Caption","CaptionWrap","Relief","Glyph",0
};
/*
label button signals names && nicknames
Default response method name will be made by:
On<object name><nickname>.
For instance if object name = <Abutton> and nickname = <Click>
default response method name will be: <OnAbuttonClick>
*/
char* vdkpixmapbutton_signals[] =
{
"clicked_signal","enter_signal","leave_signal",0
};
char* vdkpixmapbutton_nicknames[] =
{
"Click","Enter","Leave",0
};
//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDK_ANCESTOR);
DEFINE_EVENT_LIST(CLASS,VDK_ANCESTOR);
///////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, VDKForm* owner,
				 char* pixfile, char* label, char* tip = NULL):

  VDK_ANCESTOR(owner, pixfile , label,tip),VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties list label button properties
  // names. (Others props are prepended  by VDKBObject constructor)
  for(t=0; vdkpixmapbutton_props[t]; t++)
    proplist.add(VDKBProperty(vdkpixmapbutton_props[t]));
  if(label)
    SetPropValue("Caption",label);
  // add to VDKBObject signal list label button signals
  for(t=0; vdkpixmapbutton_signals[t]; t++)
    siglist.add(VDKBSignal(vdkpixmapbutton_signals[t],
			   this,
			   vdkpixmapbutton_nicknames[t]));
  // connects events.
  CONNECT_COMMON_EVENTS;

  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tCaption:\"%s\";",(char*) GetProp("Caption"));
  fprintf(fp,"\n\tCaptionWrap:%s;",(char*) GetProp("CaptionWrap"));
  fprintf(fp,"\n\tRelief:%s;", (char*) GetProp("Relief"));
  fprintf(fp,"\n\tGlyph:%s;",(char*) GetProp("Glyph"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label button counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  CLASS* button = new CLASS(buff,owner,
			    (char*) NULL,buff);
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  bool flag = false;
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(owner->Owner());
	  if(prjman && prjman->objInspector)
	    flag = prjman->objInspector->preview->Checked;
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      button->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      button->SetPropValue( EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(button,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(button,l_justify,flag,flag,true,flag);
	}
      else if(! owner->Active->AddToParent(button,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  char obj_caption[128];
  char obj_glyph[256];
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  if(!parser.GetParam(obj_caption,buffer,"Caption:"))
    strcpy(obj_caption,"none");
  if(parser.GetParam(obj_glyph,buffer,"Glyph:") && 
     strcmp(obj_glyph,NIHIL_PROP))
    ;
  else
    *obj_glyph = '\0';

  sprintf(tmp,"\n%s = new %s(this,%s%s%s,\"%s\");",
	  obj_name,
	  VDK_CLASS,
	  *obj_glyph ? "\"" : "",
	  *obj_glyph ? obj_glyph : "(char*) NULL",
	  *obj_glyph ? "\"" : "",
	  obj_caption);
  strcpy(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  // get label button  props
  if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->CaptionWrap = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  if(parser.GetParam(arg,buffer,"Relief:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->Relief = (GtkReliefStyle) %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
char obj_name[128];
char obj_parent[128];
char obj_caption[256];
char arg[64];
char obj_glyph[256];
CLASS* button;
// get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  // get caption
  if(!parser.GetParam(obj_caption,buffer,"Caption:"))
    strcpy(obj_caption,"none");
  if(parser.GetParam(obj_glyph,buffer,"Glyph:") && 
     strcmp(obj_glyph,NIHIL_PROP))
    ;
  else
      strcpy(obj_glyph,"");
  
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      if(*obj_glyph)
	button = new CLASS(obj_name,owner,obj_glyph,obj_caption);
      else
	button = new CLASS(obj_name,owner,(char*) NULL,obj_caption);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(button,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  button->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) button->GetProp( EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) button->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) button->GetProp("_Padding");
      padding = atoi(p);
      if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
	{
	button->CaptionWrap = !strcmp(arg,CHECK_TRUE);
	button->SetPropValue("CaptionWrap",arg);
	}
      if(parser.GetParam(arg,buffer,"Relief:") && strcmp(arg,NIHIL_PROP))
	{
	  button->Relief = (GtkReliefStyle) atoi(arg);
	  button->SetPropValue("Relief",arg);
	}
      // glyph
      if(*obj_glyph)
	button->SetPropValue("Glyph",obj_glyph);
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(button,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
 */
static char *reliefs[] = 
{ "Normal","Half","None",0 };
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  // FIX ME: lang support
  VDKTable* table = new VDKTable(inspector,5,2,true);

  VDKLabelButton* button = 
    new VDKLabelButton(inspector,"Set Caption");
  table->AddToCell(button,0,0);
  button->Parent(this);
  SignalConnect(button,"clicked",&CLASS::OnSetCaption);
  caption = new VDKEntry(inspector,0,(char*) GetProp("Caption"));
  caption->SetSize(100,-1);
  table->AddToCell(caption,0,1);

  wrap = new VDKCheckButton(inspector,"Caption wrap");
  table->AddToCell(wrap,1,0);
  wrap->Parent(this);
  SignalConnect(wrap,"toggled",&CLASS::OnSetCaptionWrap);
  
  button = new VDKLabelButton(inspector,"Set Relief");
  table->AddToCell(button,2,0);
  button->Parent(this);
  SignalConnect(button,"clicked",&CLASS::OnSetRelief);
  int r = atoi(GetProp("Relief"));
  relief = new VDKCombo(inspector,NULL);
  relief->SetSize(100,-1);
  StringList sl;
  int t = 0;
  for(;reliefs[t];t++)
    sl.add(VDKString(reliefs[t]));
  relief->PopdownStrings = sl;
  relief->SelectItem(r);
  table->AddToCell(relief,2,1);

  button = new VDKLabelButton(inspector,"Set glyph");
  table->AddToCell(button,3,0);
  button->Parent(this);
  SignalConnect(button,"clicked",&CLASS::OnSetGlyph);
  VDKString s = GetProp("Glyph");
  pixfile = new VDKEntry(inspector,0,
			 !strcmp((char*) s,NIHIL_PROP) ? NULL : (char*) s);
  table->AddToCell(pixfile,3,1);
  pixfile->SetSize(100,-1);
  pixfile->Editable = false;
  
  shortname = new VDKLabelButton(inspector,"Set xpm to project dir");
  table->AddToCell(shortname,4,0);
  shortname->Parent(this);
  SignalConnect(shortname,"clicked",&CLASS::OnSetShortName);
  shortname->Enabled = false;

  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}
//////////////////////////////////////////////////////
// These response methods actually change both 
// properties on widget and gui widget properties
//////////////////////////////////////////////////////
/*
 */
bool
CLASS::OnSetCaptionWrap(VDKObject*)
{
  SetPropValue("CaptionWrap", wrap->Checked ? CHECK_TRUE : CHECK_FALSE);
  CaptionWrap = wrap->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetRelief(VDKObject*)
{
  int sel = relief->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  int relief = atoi(buff);
  SetPropValue("Relief",buff);
  Relief = (GtkReliefStyle) relief;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetCaption(VDKObject*)
{
  if(strlen(caption->Text)>0)
    {
      sprintf(buff,"%s",(char*) caption->Text);
      SetPropValue("Caption",buff);
      Caption = caption->Text;
      inspector->FormNeedToBeChanged();
    }
  return true;
}

/*
 */
bool
CLASS::OnSetGlyph(VDKObject*)
{
  FileStringArray selections;
  VDKXpmBrowser *child = new VDKXpmBrowser(Owner(),&selections,
					   file_dialog_prompts[0]);
  child->ShowModal();
  if(selections.size() > 0)
    {
      SetPixmap((char*) selections[0]);
      pixfile->Text = (char*) selections[0];
      SetPropValue("Glyph",(char*) selections[0]);
      inspector->FormNeedToBeChanged();
      shortname->Enabled = true;
    }
  return true;
}
/*
 */
bool
CLASS::OnSetShortName(VDKObject*)
{
  VDKString s = GetProp("Glyph");
  VDKString path;
  VDKBProject* project;
  VDKBProjectManager* prjman;
  if(!inspector)
    return true;
  prjman =  dynamic_cast<VDKBProjectManager*>(inspector->Owner());
  if(!prjman)
    return true;
  project = prjman->Project();
  if(!project)
    return true;
  path = project->Path;
  sprintf(buff,"cp %s %s/",(char*) s, (char*) path);
  system(buff);
  char* p = get_shortfilename((char*) s);
  if(p)
    {
      VDKString short_name = p;
      pixfile->Text = (char*) short_name;
      SetPropValue("Glyph",(char*) short_name);
      shortname->Enabled = false;
      inspector->FormNeedToBeChanged();
    }
  return true;
}





