/*
 * ===========================
 * VDK Builder
 * Version 0.1
 * Revision 0.0
 * November 1998
 * ===========================
 * 
 * Copyright (C) 1998,1999 Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * Based on VDK Library 
 * Copyright (C) 1998, Mario Motta
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */

#include <vdkb_prjman.h>
#include <vdkb_locale.h>
#include <vdkb_types.h>
#include <vdk/FileDialog.h>
#include <vdk/FileSaveAsDialog.h>
#include <vdkb.h>
#include <vdk/evlisthandle.h>
#include <vdkb_unit.h>
#include <vdkb_utils.h>

extern char *c_xpm[];
extern char *h_xpm[];
extern char *prj_node_xpm[];
extern char *unit_node_xpm[];
extern char *exec_xpm[];
static char buff[512];
extern char *ext_types[];
extern char *newform_xpm[];
////////////// SIGNAL STUFF ////////////////
DEFINE_SIGNAL_MAP(VDKBProjectManager,VDKForm)
  ON_SIGNAL(tree,select_node_signal,OnSelectRow),
  ON_SIGNAL(toolbar,clicked_signal,HandleToolbar)
END_SIGNAL_MAP

DEFINE_EVENT_LIST(VDKBProjectManager,VDKForm);
DEFINE_SIGNAL_LIST(VDKBProjectManager,VDKForm);

bool
VDKBProjectManager::HandleToolbar(VDKObject* )
{
int button = toolbar->ButtonPressed;
switch(button)
  {
  case NEW_PROJECT_BUTTON:
    ((VDKBMainForm*) Owner())->NewProject(NULL);
    break;

  case ADD_UNIT_BUTTON:
    AddUnit();
    break;

  case REMOVE_UNIT_BUTTON:
    RemoveUnit();
    break;

  case ACTIVATE_EDITOR_BUTTON:
    OnSelectRow(tree);
    break;

  case FORM_PROPS_BUTTON:
    // select root (thus form)
    if(objInspector && ActiveChild.child)
	  objInspector->SelectWidgetByTree(NULL);
    break;
  }
return true;
}
/*
activate editor
on a selected tree
node
 */
void
VDKBProjectManager::ActivateEditor(VDKTreeNode node, 
				   bool editable,
				   bool hilite)
{
  if(! node)
    return;
  VDKBMainForm* mainform = (VDKBMainForm*) Owner();
  VDKBEditor* editor =   mainform->MakeEditor();
  //look for the file pointed by node
  int t = 0;
  bool found = false;
  TextListIterator li(editor->textlist);
  for(;li;li++,t++)
    {
      if(! strcmp(li.current()->Filename(),tree->Key(node)))
	{
	  found = true;
	  break;
	}
    }
  // does not exist in editor, make it
  if(! found)
    editor->AddText(tree->Key(node),editable,hilite);
  // activate page containing file
  editor->nbook->ActivePage = t;
  // set toggle form/unit
  char* cc_ext = (char*) VDKBuilder::ideDefaults.unit.cc_ext;
  char* p = get_extension(tree->Key(node));
  if(p)
    {
      p++;
      if(!strcmp(p,cc_ext))
	{
	  char local[256];
	  strcpy(local,tree->Key(node));
	  char* q = get_extension(local);
	  if(q)
	    {
	      *q = '\0';
	      sprintf(buff,"%s.frm",local);
	      mainform->EnableToggleFormUnit(true,buff);
	    }
	  else
	    mainform->EnableToggleFormUnit(false,buff);
	}
      else
	mainform->EnableToggleFormUnit(false,buff);
    }
}

/*
activate editor
on a generic file
pointed by <text>
 */
void
VDKBProjectManager::ActivateEditor(char* text, 
				   bool editable,
				   bool hilite)
{
  if(! text)
    return;
  VDKBMainForm* mainform = (VDKBMainForm*) Owner();
  VDKBEditor* editor =  mainform->MakeEditor();
  //look for the file int text list
  int t = 0;
  bool found = false;
  TextListIterator li(editor->textlist);
  for(;li;li++,t++)
    {
      if(! strcmp(li.current()->Filename(),text))
	{
	  found = true;
	  break;
	}
    }
  // does not exist in editor, make it
  if(! found)
    editor->AddText(text,editable,hilite);
  // activate page containing file
  editor->nbook->ActivePage = t;
  // set toggle form/unit
  char* cc_ext = (char*) VDKBuilder::ideDefaults.unit.cc_ext;
  char* p = get_extension(text);
    if(p)
    {
      p++;
      if(!strcmp(p,cc_ext))
	{
	  char local[256];
	  strcpy(local,text);
	  char* q = get_extension(local);
	  if(q)
	    {
	      *q = '\0';
	      sprintf(buff,"%s.frm",local);
	      mainform->EnableToggleFormUnit(true,buff);
	    }
	  else
	    mainform->EnableToggleFormUnit(false,buff);
	}
      else
	mainform->EnableToggleFormUnit(false,buff);
    }
}

/*
 */
bool
VDKBProjectManager::OnSelectRow(VDKObject* obj)
{
  VDKCustomTree* tree = dynamic_cast<VDKCustomTree*>(obj);
  if(!tree ) return true;
  VDKTreeNode node = tree->SelectedNode;
  if(! node) return true;
  // get stored info
  gpointer p = gtk_ctree_node_get_row_data(GTK_CTREE(tree->CustomWidget()),
					   node);
  VDKBUnit*  unit = (VDKBUnit*) p;

  // is a leaf without other info, edit it
  if(! p && tree->IsLeaf(node))
    ActivateEditor(node);
  // is a form unit
  // create or show form
  else if(p && (p == (gpointer) form_unit))
    {
      char* t = new char[strlen(tree->Key(node))+1];
      strcpy(t,tree->Key(node));
      char* p = get_extension(t);
      if(p) *p = '\0';
      VDKString text = t;
      text += ".";
      text += FORM_EXT;
      // create a form parsing .frm file (or raise)
      VDKBGuiForm* form = CreateForm(tree->Key(node));
      // make inspector
      if(form)
	{
	  if( !objInspector)
	    {
	      objInspector = new VDKBObjectInspector(this,NULL);
	      objInspector->Setup();
	      objInspector->Show(); 
	    }
	  else if(objInspector->Iconized)
	    objInspector->Iconized = false;
	  else
	    objInspector->Raise();
	  objInspector->LoadTree(form);
	  form->Raise();
	}
      // set automa
      VDKBMainForm* mainform = (VDKBMainForm*) Owner();
      mainform->Automa(automa_edit_form_on);
      delete t;
    }
  // isn't a leaf and is editable, edit it
  else if(p && (unit->Type() == c_source_unit))
	ActivateEditor(node);

  return true;
}
/*

 */
bool VDKBProjectManager::OnReleaseButton(VDKObject* obj, GdkEvent*)
{
VDKCustomTree* tree = dynamic_cast<VDKCustomTree*>(obj);
// run time type checking
if(!tree)
  return true;
VDKTreeNode node = tree->SelectedNode;
// no node selected so get away
if(! node)
  return true;
// refuses multiple selections
// deselecting all nodes
if(tree->Selections().size() > 1)
  {
    gtk_ctree_unselect_recursive (GTK_CTREE(tree->CustomWidget()), NULL);
    (*toolbar)[1]->Enabled = true;
    (*toolbar)[2]->Enabled = false;
    (*toolbar)[3]->Enabled = false;
    (*toolbar)[4]->Enabled = false;
    return true;
  } 
// now can proceed
// is root ?
if(node == root)
  {
    (*toolbar)[1]->Enabled = true;
    (*toolbar)[2]->Enabled = false;
    (*toolbar)[3]->Enabled = false;
    (*toolbar)[4]->Enabled = false;
    // prompts name & status
    VDKString name = project->Name;
    sprintf(buff,"%s - %s",
	    (char*) name,
	    prj_status[project->Status]);
    bar->Panels()[0]->Caption = buff;
  }
// is a node ? 
else if(! tree->IsLeaf(node))
    {
      // prompts name & status
      VDKBUnit*  unit = (VDKBUnit*)
      gtk_ctree_node_get_row_data(GTK_CTREE(tree->CustomWidget()),node);
      // not a .frm
      if( ((gpointer) unit) != ((gpointer) form_unit))
	{
	  (*toolbar)[1]->Enabled = false;
	  (*toolbar)[2]->Enabled = true;
	  (*toolbar)[3]->Enabled = 
	    unit->Type() == c_source_unit ? true : false;
	  (*toolbar)[4]->Enabled = false;
	  sprintf(buff,"%s.%s - %s",
		  (char*) VDKString(unit->Name()),
		  ext_types[unit->Type()],
		  prj_status[unit->Status]);
	  bar->Panels()[0]->Caption = buff;
	}
      else
	{
	  (*toolbar)[1]->Enabled = false;
	  (*toolbar)[2]->Enabled = false;
	  (*toolbar)[3]->Enabled = true;
	  (*toolbar)[4]->Enabled = (ActiveChild.child) ? true : false;
	  bar->Panels()[0]->Caption = "";
	}
    }
// is a leaf
else
    {
      gpointer p = 
	gtk_ctree_node_get_row_data(GTK_CTREE(tree->CustomWidget()),node);
      (*toolbar)[1]->Enabled = false;
      (*toolbar)[2]->Enabled = false;
      (*toolbar)[3]->Enabled = true;
      (*toolbar)[4]->Enabled = 
	((p == (gpointer) form_unit) && (ActiveChild.child)) ? 
	true : false;
      bar->Panels()[0]->Caption = "";
    }
 return true;
}

/////////////////////////////////////////////
/*
 */ 
VDKBProjectManager::VDKBProjectManager(VDKForm* owner):
  VDKForm(owner)
{
project = NULL;
root = NULL;
ForceToClose = false;
Changed = false;
ActiveChild.child = NULL;
ActiveChild.name = "";
objInspector = NULL;
}
/*
 */
VDKBProjectManager::~VDKBProjectManager()
{
if(project)
  delete project;
}
/*
 */
void
VDKBProjectManager::Setup()
{
  Title = prjman_prompts[0];
  // a separator
  VDKSeparator *sep = new VDKSeparator(this,h_separator);
  Add(sep,l_justify,false,false,false);
  // the toolbar
  toolbar = new VDKToolbar(this);
  toolbar->Style = GTK_TOOLBAR_BOTH;
  toolbar->Spacing = 5;
  toolbar->AddButton(prjman_pixmaps[1],NULL,prjman_prompts[1]);
  toolbar->AddButton(prjman_pixmaps[2],NULL,prjman_prompts[2]);
  toolbar->AddButton(prjman_pixmaps[3],NULL,prjman_prompts[3]);
  toolbar->AddButton(prjman_pixmaps[4],NULL,prjman_prompts[4]);
  toolbar->AddButton(prjman_pixmaps[5],NULL,prjman_prompts[5]);
  (*toolbar)[1]->Enabled = false;
  (*toolbar)[2]->Enabled = false;
  (*toolbar)[3]->Enabled = false;
  (*toolbar)[4]->Enabled = false;
  Add(toolbar,l_justify,false,false,false);
  // another sep
  sep = new VDKSeparator(this,h_separator);
  Add(sep,l_justify,false,false,false);
  // the manager tree
  Add(tree = new VDKBCustomTree(this,1,NULL,GTK_SELECTION_EXTENDED));
  bar = new VDKPanelbar(this);
  Add(bar,l_justify,false,false,false); 
  EventConnect(tree,"button_release_event",
	       &VDKBProjectManager::OnReleaseButton);
  SetSize(300,300);
}

/* 
 */
void 
VDKBProjectManager::OnShow(VDKForm*)
{
  int w,h;
  VDKPoint ownerPos = Owner()->Position;
  gdk_window_get_size(GTK_WIDGET(Owner()->Window())->window, &w, &h);
  VDKPoint ownerSize = VDKPoint(w,h);
  VDKPoint p(ownerPos.X(),ownerPos.Y()+ownerSize.Y()+5);
  Position = p;
  gtk_window_set_policy(GTK_WINDOW(Window()),false,false,true);
}
/*
 */
void
VDKBProjectManager::AddNewProject(char* name, int type)
{
  if(type < 0)
    return;
 
  if(project)
  {
    // this should manage project closing
    // (saving all prj stuff)
    CanClose();
    // 
    // force to close all child form (if any)
    formlist.Destroy();
    if(objInspector)
      {
	objInspector->ForceToClose = true;
	objInspector->Close();
	objInspector = NULL;
      }
    ActiveChild.child = NULL;
    // removes all editor page
    VDKBMainForm* mainform = (VDKBMainForm*) Owner();
    VDKBEditor* editor = mainform->Editor();
    if( editor)
      {
	int t = 0;
	while( (t = editor->nbook->Pages.size()) > 0)
	    editor->nbook->RemovePage(t-1);
	editor->textlist.flush();
      }
    delete project;
    project = NULL;
  }

  if(type >= 0)
    project = new VDKBProject(this,name,type,true);

 // makes template files
 bool result = false;

 switch(type)
   {
   case vdk_project:
     result = project->CreateTemplatesFilesForVDK();
     break;
   case console_project:
     result = project->CreateTemplatesFilesForConsole();
     break;
   default:
     break;
   }

 if(result)
   {
     LoadTree();
     ((VDKBMainForm*) Owner())->Automa(automa_prjman_on);
     // change to project dir
     VDKString dir = project->Path;
     chdir((char*) dir);
     // activate editor on project_name.cc
     char* cc_ext = (char*) VDKBuilder::ideDefaults.unit.cc_ext;
     VDKString mainsource = project->PathName;
     mainsource += ".";
     mainsource += cc_ext;
     ActivateEditor((char*) mainsource,true,true);
   }

}
/*
 */
void
VDKBProjectManager::OpenProject()
{
  FileStringArray selections;
  char* prj_ext   = (char*) VDKBuilder::ideDefaults.project.prj_ext;
  VDKFileDialog *child = new VDKFileDialog(this,&selections,
					   file_dialog_prompts[0]);
  sprintf(buff,"*.%s",prj_ext);
  child->Filter = buff;  
  child->ShowModal();
  if(selections.size() <= 0)
    return;
  
  // load project
  VDKBProject *temp =  new VDKBProject(this,(char*) selections[0]);
  if(temp->Valid)
    {
      if(project)
	{
	  // this should manage project closing
	  CanClose();
	  // force to close all child form (if any)
	  formlist.Destroy();
	  ActiveChild.child = NULL;
	  if(objInspector)
	    {
	      objInspector->ForceToClose = true;
	      objInspector->Close();
	      objInspector = NULL;
	    }
	  // removes all editor page
	  VDKBMainForm* mainform = (VDKBMainForm*) Owner();
	  VDKBEditor* editor = mainform->Editor();
	  if( editor)
	    {
	      int t = 0;
	      while( (t = editor->nbook->Pages.size()) > 0)
		  editor->nbook->RemovePage(t-1);
	      editor->textlist.flush();
	    }
	}
      delete project;
      project = temp;
      LoadTree();
      ((VDKBMainForm*) Owner())->Automa(automa_prjman_on);
      Changed = false;
      // change to project dir
      VDKString dir = project->Path;
      chdir((char*) dir);
      // make it if does not exists
      VDKBMainForm* mainform = (VDKBMainForm*) Owner();
      mainform->MakeEditor();
      // activate editor on project_name.cc
      char* cc_ext = (char*) VDKBuilder::ideDefaults.unit.cc_ext;
      VDKString mainsource = project->PathName;
      //char* p = get_extension((char*) mainsource);
      mainsource += ".";
      mainsource += cc_ext;
      ActivateEditor((char*) mainsource,true,true);
    }
  else
    delete temp;
}
/*
 */
void
VDKBProjectManager::LoadTree()
{
  if(! project)
    return;
  // clears tree
  tree->Clear();
  tree->Freeze();
  // add project to tree
  VDKString name = project->Name; 
  char* prj = (char*) name;
  // add root
  root = tree->AddNode(&prj,NULL,true,false,
		       prj_node_xpm,prj_node_xpm);
  // sets project as row data
  gtk_ctree_node_set_row_data(GTK_CTREE(tree->CustomWidget()), root,
			      (gpointer) project);

  // add units
  UnitListIterator li(project->Units());
  for(;li;li++)
	AddUnitNode(li.current());
  tree->Thaw();
  // select root node
  tree->SelectedNode = NULL;
  // enables 
  (*toolbar)[1]->Enabled = true;
  (*toolbar)[2]->Enabled = false;

  (*toolbar)[3]->Enabled = false;
}
/*
 */
void
VDKBProjectManager::AddUnitNode(VDKBUnit* unit)
{
  char **xpm,*p; 
  int type = unit->Type();
  switch(type)
    {
    case source_unit:
    case project_unit:
      xpm = unit_node_xpm;
      p = unit->ShortName();
      break;
    case c_source_unit:
      xpm = unit_node_xpm;
      sprintf(buff,"%s.c",unit->Name());
      p = buff;
      break;
    case object_unit:
    case staticlib_unit:
      xpm = exec_xpm;
      p = unit->ShortName();
      break;
    default:
      xpm = unit_node_xpm;
      p = unit->ShortName();
    }

  VDKTreeNode node = tree->AddNode(&p,root,
				   false,false,xpm,xpm);
  // sets unit as row data
  gtk_ctree_node_set_row_data(GTK_CTREE(tree->CustomWidget()), node,
			      (gpointer) unit);
  // if unit type == source_unit
  // add child source and header
  if(unit->Type() == source_unit 
     ||  unit->Type() == project_unit
     )
    {
      char* source = unit->Source();
      // check for exist
      if(! access(source,F_OK))
	{
	VDKTreeNode n = tree->AddNode(&source,node,false,true,c_xpm,c_xpm);
	gtk_ctree_node_set_row_data(GTK_CTREE(tree->CustomWidget()), n,
			      (gpointer) NULL);
	}
      char* header = unit->Header();
      if(! access(header,F_OK))
	{
	VDKTreeNode n = tree->AddNode(&header,node,false,true,h_xpm,h_xpm);
	gtk_ctree_node_set_row_data(GTK_CTREE(tree->CustomWidget()), n,
			      (gpointer) NULL);
	}
      char* gui = unit->Gui();
      VDKTreeNode n = NULL;
      if(! access(gui,F_OK))
	{
	n = tree->AddNode(&gui,node,false,false,
				      newform_xpm,newform_xpm);
	if(n)
	  gtk_ctree_node_set_row_data(GTK_CTREE(tree->CustomWidget()), n,
			      (gpointer) form_unit);
	//
	char* p = unit->GuiSource();
	if(n && p && ! access(p,F_OK))
	   {
	     VDKTreeNode n1 = tree->AddNode(&p,n,false,true,
				      c_xpm,c_xpm);
	     if(n1)
	       gtk_ctree_node_set_row_data(GTK_CTREE(tree->CustomWidget()), n1,
					 (gpointer) NULL);
	   }
	//
	 p = unit->DfmHeader();
	 if( n && p && ! access(p,F_OK))
	   {
	     VDKTreeNode n2 = tree->AddNode(&p,n,false,true,
				      h_xpm,h_xpm);
	     if(n2)
	       gtk_ctree_node_set_row_data(GTK_CTREE(tree->CustomWidget()), n2,
					 (gpointer) NULL);
	   }
	}
    }
}
/*
 */

void  
VDKBProjectManager::RemoveUnit()
{
  VDKBUnit* unit;
  VDKTreeNode node = tree->SelectedNode;
  if(!node)
    return;
  else
    unit = (VDKBUnit*)  gtk_ctree_node_get_row_data(
	 GTK_CTREE(tree->CustomWidget()), node);
  
  if(unit->Type() == project_unit)
    Application()->MessageBox(APPNAME,
			      user_messages[user_cant_remove_unit],
			      MB_OK| MB_ICONINFORMATION,
			      user_messages[user_cancel]);
  
  else if(project->Units().remove(unit))
    {
      tree->RemoveNode(node);
      delete unit;
      Changed = true;
    }
  return;
}

/*
 */
void VDKBProjectManager::AddUnit()
{
  FileStringArray selections;
  char* cc_ext   = (char*) VDKBuilder::ideDefaults.unit.cc_ext;
  VDKFileDialog *child = new VDKFileDialog(this,&selections,
					   file_dialog_prompts[2]);
  sprintf(buff,"*.%s",cc_ext);
  child->Filter = buff;  
  child->ShowModal(); 
  int t; 
  if(selections.size() <= 0)
    return ;
  for(t=0; t < selections.size();t++)
    {
      char* selected = (char*) selections[t];
      VDKBUnit* unit = new VDKBUnit(selected,prj_unsaved);
      if(! Unique(unit))
	{
	  // duplicate not allowed
	  sprintf(buff,"%s\n%s",
		  unit->Name(),
		  user_messages[user_dup_unit]);
	  Application()->MessageBox(APPNAME,
				    buff,
				    MB_OK| MB_ICONINFORMATION,
				    user_messages[user_cancel]);

	  delete unit; 
	}
      else if(unit->Type() != unknow_type)
	{
	  project->Units().add(unit);
	  Changed = true;
	  AddUnitNode(unit);
	}
      else
	{
	  // unknown type can't be added
	  sprintf(buff,"%s\n%s\n%s",
		  unit->Name(),
		  user_messages[user_unknown_unit],
		  user_messages[user_unknown_unit1]);
	  Application()->MessageBox(APPNAME,
				    buff,
				    MB_OK| MB_ICONINFORMATION,
				    user_messages[user_cancel]);
	  delete unit;
	}
    }
}
    

/*
  checks unique
 */
bool
VDKBProjectManager::Unique(VDKBUnit* unit)
{
UnitListIterator li(project->Units());
for(;li;li++)
   if (*unit == *(li.current()))
     return false;
return true;
}
/*
Save project
 */
bool 
VDKBProjectManager::Save()
{
if(project)
  {
    if(project->Status == prj_never_saved)
      {
	char* prj_ext = (char*) VDKBuilder::ideDefaults.project.prj_ext;
	FileStringArray selections;
	VDKFileSaveAsDialog *child = 
	  new VDKFileSaveAsDialog(this,&selections,
				  file_dialog_prompts[3]);
	sprintf(buff,"*.%s",prj_ext);
	child->Filter = buff;
	child->ShowModal(); 
	if(selections.size() <= 0)
	  return false;
	project->Save((char*) selections[0]);
	Changed = false;
	// reload tree 
	LoadTree(); 
      }
    else
      {
	// save editors
	VDKBMainForm* mainform = (VDKBMainForm*) Owner();
	VDKBEditor* editor =   mainform ? mainform->Editor() : NULL;
	if(editor && editor->Visible)
	  editor->UpdateFiles();
	// save project
	project->Save();
	// writes frm file (in vdkb_design.cc)
	WriteGuiFiles();
	Changed = false;
	// reload tree 
	LoadTree(); 
      }
  }
 return true;
}

/*
Project manager can'te be closed
 */
bool
VDKBProjectManager::CanClose()
{
  if(project)
    {
      // save editors
      VDKBMainForm* mainform = (VDKBMainForm*) Owner();
      VDKBEditor* editor =   mainform ? mainform->Editor() : NULL;
      if(editor && editor->Visible)
	editor->UpdateFiles();
      // save project
      project->Save();
      // writes frm file (in vdkb_design.cc)
      WriteGuiFiles();
      Changed = false;
      // can be closed via mainform only.
      if(ForceToClose)
	{
	  // allows child list to be close
	  // setting child ForceToClose = true
	  formlist.Close();
	  if(objInspector)
	    objInspector->ForceToClose = true;
	  return ForceToClose;
	}
    }
  return ForceToClose;
}

/////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////
typedef struct { VDKTreeNodeList* list; char* key; } SelectInfo;
/*
called by Select(char* key)->gtk_ctree_post_recursive(),
if node key matches with key
add that node to list
 */
static void
SelectOnTree(GtkCTree     *ctree,
	      GtkCTreeNode *node,
	      gpointer      i)
{
SelectInfo* info = (SelectInfo*) i;
char *text;
int res = gtk_ctree_get_node_info (ctree, node, &text, 
			 NULL, NULL, NULL, NULL,NULL,NULL,NULL);
if(res && strstr(text,info->key))
   info->list->add(node);
}

/*
return a list of nodes that contain
<key>, user should delete list after use
 */
VDKTreeNodeList* 
VDKBCustomTree::Select(char* key)
{

  VDKTreeNodeList* nodelist = new VDKTreeNodeList;
  SelectInfo info = { nodelist, key };
  gtk_ctree_post_recursive(GTK_CTREE(custom_widget), 
			   NULL, SelectOnTree, (gpointer) &info);
  return nodelist;
}
