/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_radiobutton.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
static char buff[128];
extern bool onDraggingAnObjectIntoAFixed;
extern GtkWidget *grabbed_widget;
int 
VDKBRadioButton::OnButtonEvent(GtkWidget *w, GdkEvent* event, void* o)
{ 
  g_return_val_if_fail(o != NULL, FALSE);
  g_return_val_if_fail(w != NULL, FALSE);
  VDKBRadioButton* obj = reinterpret_cast<VDKBRadioButton*>(o);
  VDKBEventContainer* container;
  gtk_signal_emit_stop_by_name(GTK_OBJECT(w),
			       "button_press_event");
  container = dynamic_cast<VDKBEventContainer*>(obj->Object()->Parent());
  if(container)
    container->OnButtonPressed(obj,event); 
  return TRUE;
}

int
VDKBRadioButton::OnEnterEvent(GtkWidget *w, 
				  GdkEvent* event, 
				  void* o)
{ 
  g_return_val_if_fail(o != NULL, FALSE);
  g_return_val_if_fail(w != NULL, FALSE);
  //VDKBRadioButton* obj = reinterpret_cast<VDKBRadioButton*>(o);
  gtk_signal_emit_stop_by_name(GTK_OBJECT(w),"enter_notify_event"); 
  return true;
}

int
VDKBRadioButton::OnLeaveEvent(GtkWidget *w, 
				  GdkEvent* event, 
				  void* o)
{ 
  g_return_val_if_fail(o != NULL, FALSE);
  g_return_val_if_fail(w != NULL, FALSE);
  //  VDKBRadioButton* obj = reinterpret_cast<VDKBRadioButton*>(o);
  gtk_signal_emit_stop_by_name(GTK_OBJECT(w),"leave_notify_event"); 
  return true;
}
//////////////////////////////////////////////////////////////
/*
  - constructor
 */
VDKBRadioButton::VDKBRadioButton(char* name, VDKForm* owner,
				 char* label, char* tip = NULL):
  VDKBCheckButton(name,owner,label,tip)
{
  gtk_widget_destroy(widget);
  widget = gtk_radio_button_new_with_label(NULL,label);
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  gtk_signal_connect(GTK_OBJECT(widget),"button_press_event",
		   GTK_SIGNAL_FUNC(VDKBRadioButton::OnButtonEvent),
			       reinterpret_cast<gpointer>(this));
  gtk_signal_connect(GTK_OBJECT(widget),"enter_notify_event",
		   GTK_SIGNAL_FUNC(VDKBRadioButton::OnEnterEvent),
			       reinterpret_cast<gpointer>(this));
  gtk_signal_connect(GTK_OBJECT(widget),"leave_notify_event",
		   GTK_SIGNAL_FUNC(VDKBRadioButton::OnLeaveEvent),
			       reinterpret_cast<gpointer>(this));

}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
VDKBRadioButton::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tCaption:\"%s\";",(char*) GetProp("Caption"));
  fprintf(fp,"\n\tCaptionWrap:%s;",(char*) GetProp("CaptionWrap"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
VDKBRadioButton::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label button counter
  // to ensure unicity
  for(sprintf(buff,"radioButton%d",VDKBCheckButton::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      VDKBCheckButton::Counter++)
    sprintf(buff,"radioButton%d",VDKBCheckButton::Counter);
  VDKBRadioButton* button = new VDKBRadioButton(buff,owner,buff);
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  bool flag = false;
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(owner->Owner());
	  if(prjman && prjman->objInspector)
	    flag = prjman->objInspector->preview->Checked;
	   if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      button->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      button->SetPropValue( EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(button,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(button,l_justify,flag,flag,true,flag);
	}
      else if(! owner->Active->AddToParent(button,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
VDKBRadioButton::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  char obj_caption[128];
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  if(!parser.GetParam(obj_caption,buffer,"Caption:"))
    strcpy(obj_caption,"none");
  sprintf(tmp,"\n%s = new VDKRadioButton(this,\"%s\");",
	  obj_name,obj_caption);
  strcpy(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  // get label button  props
  if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->CaptionWrap = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
VDKBRadioButton::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
char obj_name[128];
char obj_parent[128];
char obj_caption[256];
char arg[64];
VDKBRadioButton* button;
// get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  // get caption
  if(!parser.GetParam(obj_caption,buffer,"Caption:"))
    strcpy(obj_caption,"none");
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      button = new VDKBRadioButton(obj_name,owner,obj_caption);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(button,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  button->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) button->GetProp( EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) button->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) button->GetProp("_Padding");
      padding = atoi(p);
      if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
	{
	button->CaptionWrap = !strcmp(arg,CHECK_TRUE);
	button->SetPropValue("CaptionWrap",arg);
	}
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(button,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}








