/*
 * ===========================
 * VDK Builder
 * Version 0.1
 * Revision 0.3 
 * July 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_slider.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBSlider
// put here vdk class name string
#define VDK_CLASS "VDKSlider"
// put here vdk class name 
#define VDK_ANCESTOR  VDKSlider
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "slider"

static char buff[128];

// used to autogenerate default
// button  names and captions

int CLASS::Counter = 0;
/* 
 label button properties names
 */
static char* vdkslider_props[] =
{ 
"Digits","Step","Lower","Upper","UpdatePolicy","DefValue",0
};

//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDKBEvSlider);
DEFINE_EVENT_LIST(CLASS,VDKBEvSlider);
///////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, 
			       VDKForm* owner):
  VDKBEvSlider(owner), VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties list 
  // (Others props are prepended  by VDKBObject constructor)
  for(t=0; vdkslider_props[t]; t++)
    proplist.add(VDKBProperty(vdkslider_props[t]));
  SetPropValue("Lower","0");
  SetPropValue("Upper","20");
  SetPropValue("Step","0.1");
  SetPropValue("DefValue","0");
  SetPropValue("UpdatePolicy","0");
  SetPropValue("Digits","1");
  // progress bar not expand
  SetPropValue(JUSTIFY_INTERNAL,"l_justify");
  SetPropValue( EXPAND_INTERNAL,CHECK_FALSE);
  SetPropValue("_Fill",CHECK_FALSE);
  SetPropValue("_Padding",CHECK_FALSE);
  // connects events.
  CONNECT_COMMON_EVENTS;
  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tLower:%s;",(char*) GetProp("Lower"));
  fprintf(fp,"\n\tUpper:%s;",(char*) GetProp("Upper"));
  fprintf(fp,"\n\tStep:%s;",(char*) GetProp("Step"));
  fprintf(fp,"\n\tDigits:%s;",(char*) GetProp("Digits"));
  fprintf(fp,"\n\tUpdatePolicy:%s;",(char*) GetProp("UpdatePloicy"));
  fprintf(fp,"\n\tDefValue:%s;",(char*) GetProp("DefValue"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label bar counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  CLASS* bar = new CLASS(buff,owner);
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  bool flag = false;
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(owner->Owner());
	  if(prjman && prjman->objInspector)
	    flag = prjman->objInspector->preview->Checked;
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      bar->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      bar->SetPropValue( EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(bar,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(bar,l_justify,false,false,false,true);
	}
      else if(! owner->Active->AddToParent(bar,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  float lower = 0.0, upper = 20, step_inc = 0.1, def = 0.0;

  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  // get default values
  if(parser.GetParam(arg,buffer,"Lower:"))
    lower = atof(arg);
  if(parser.GetParam(arg,buffer,"Upper:"))
    upper = atof(arg);
  if(parser.GetParam(arg,buffer,"Step:"))
    step_inc = atof(arg);
  if(parser.GetParam(arg,buffer,"DefValue:"))
    def = atof(arg);

  source = new char[4096];
  sprintf(tmp,"\n%s = new %s(this,%f,%f,%f,%f);",
	  obj_name,VDK_CLASS,def,lower,upper,step_inc);
  strcpy(source,tmp);
  
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  if(parser.GetParam(arg,buffer,"Digits:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->Digits =  %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  if(parser.GetParam(arg,buffer,"UpdatePolicy:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->UpdatePolicy = (VDKUpdatePolicy) %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
  char obj_name[128];
  char obj_parent[128];
  float lower = 0.0, upper = 20, step_inc = 0.1, def = 0.0;
  char arg[64];
  CLASS* bar;
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  // get default values
  if(parser.GetParam(arg,buffer,"Lower:"))
    lower = atof(arg);
  if(parser.GetParam(arg,buffer,"Upper:"))
    upper = atof(arg);
  if(parser.GetParam(arg,buffer,"Step:"))
    step_inc = atof(arg);
  if(parser.GetParam(arg,buffer,"DefValue:"))
    def = atof(arg);
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      bar = new CLASS(obj_name,owner);//,def,lower,upper,step_inc);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(bar,buffer,parser);
      // prepares packing args based on those loaded by 
      if(parser.GetParam(arg,buffer,"Digits:") && strcmp(arg,NIHIL_PROP))
	{
	  bar->SetDigits( atoi(arg));
	  bar->SetPropValue("Digits",arg);
	}
      if(parser.GetParam(arg,buffer,"UpdatePolicy:") && 
	 strcmp(arg,NIHIL_PROP))
	{
	  bar->SetUpdatePolicy((VDKUpdateType) atoi(arg));
	  bar->SetPropValue("UpdatePolicy",arg);
	}
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  bar->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) bar->GetProp( EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) bar->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) bar->GetProp("_Padding");
      padding = atoi(p);
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(bar,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
static char* update_policy[] = {"continuous","discontinuos","delayed",0};
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  inspector = isp;
  VDKLabelButton* button;
  int t=0;
  float bc = 0.1;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,
				  v_box,shadow_etched_in);

  VDKTable *table = new VDKTable(inspector,6,2);
  table->SetSize(219,-1);

  table->AddToCell(new VDKLabel(inspector,"Lower"),0,0);
  lower = new VDKEntry(inspector,0,(char*) GetProp("Lower"));
  lower->SetSize(100,-1);
  table->AddToCell(lower,0,1);
  lower->Parent(this);
  SignalConnect(lower,"activate",&CLASS::OnSetLower);

  table->AddToCell(new VDKLabel(inspector,"Upper"),1,0);
  upper = new VDKEntry(inspector,0,(char*) GetProp("Upper"));
  upper->SetSize(100,-1);
  table->AddToCell(upper,1,1);
  upper->Parent(this);
  SignalConnect(upper,"activate",&CLASS::OnSetUpper);

  button = new VDKLabelButton(inspector,"Step");
  table->AddToCell(button,2,0);
  SignalConnect(button,"clicked",&CLASS::OnSetStep);
  bc = atof( (char*) GetProp("Step"));
  step = new VDKSpinButton(inspector, bc,0,10,0.01,0 );
  step->SetSize(100,-1);
  table->AddToCell(step,2,1);

  int abc = atoi((char*) GetProp("UpdatePolicy"));
  button = new VDKLabelButton(inspector,"UpdatePolicy");
  table->AddToCell(button,3,0);
  SignalConnect(button,"clicked",&CLASS::OnSetUpdatePolicy);
  button->Parent(this);
  updatepolicy = new VDKCombo(inspector,NULL);
  updatepolicy->SetSize(100,-1);
  StringList sl;
  for(t = 0;update_policy[t];t++)
    sl.add(VDKString(update_policy[t]));
  updatepolicy->PopdownStrings = sl;
  updatepolicy->SelectItem(abc);
  table->AddToCell(updatepolicy,3,1);

  button = new VDKLabelButton(inspector,"Digits");
  table->AddToCell(button,4,0);
  SignalConnect(button,"clicked",&CLASS::OnSetDigits);
  digits = new VDKEntry(inspector, 0, (char*) GetProp("Digits") );
  digits->SetSize(100,-1);
  table->AddToCell(digits,4,1);

  button = new VDKLabelButton(inspector,"Default Value");
  table->AddToCell(button,5,0);
  SignalConnect(button,"clicked",&CLASS::OnSetDefValue);
  defvalue = new VDKEntry(inspector, 0, (char*) GetProp("DefValue") );
  defvalue->SetSize(100,-1);
  table->AddToCell(defvalue,5,1);


  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}

/*
 */
bool
CLASS::OnSetLower(VDKObject*)
{
  if(strlen(lower->Text) > 0)
    SetPropValue("Lower",(char*) lower->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetUpper(VDKObject*)
{
  if(strlen(upper->Text) > 0)
    SetPropValue("Upper",(char*) lower->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetStep(VDKObject*)
{
  sprintf(buff,"%f",(float) step->ValueAsFloat);
  SetPropValue("Step",buff);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetUpdatePolicy(VDKObject*)
{
  int sel = updatepolicy->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  SetPropValue("UpdatePolicy",buff);
  inspector->FormNeedToBeChanged();
  return true;
}

/*
 */
bool
CLASS::OnSetDigits(VDKObject*)
{
  if(strlen(digits->Text) > 0)
    SetPropValue("Digits",(char*) digits->Text);
  inspector->FormNeedToBeChanged();
  return true;
}

/*
 */
bool
CLASS::OnSetDefValue(VDKObject*)
{
  if(strlen(defvalue->Text) > 0)
    SetPropValue("DefValue",(char*) defvalue->Text);
  inspector->FormNeedToBeChanged();
  return true;
}



