/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_spinbutton.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBSpinButton
// put here vdk class name string
#define VDK_CLASS "VDKSpinButton"
// put here vdk class name 
#define VDK_ANCESTOR  VDKSpinButton
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "spinbutton"

static char buff[128];

// used to autogenerate default
// button  names and captions

int CLASS::Counter = 0;
/* 
 label button properties names
 */
char* vdkspinbutton_props[] =
{ 
"Digits","DefValue","Lower","Upper","Step","Climb",0
};
/*
button signals names && nicknames
Default response method name will be made by:
On<object name><nickname>.
For instance if object name = <Abutton> and nickname = <Click>
default response method name will be: <OnAbuttonClick>
*/
char* vdkspinbutton_signals[] =
{
"value_changed_signal","focus_out_signal", 0
};
char* vdkspinbutton_nicknames[] =
{
"ValueChanged","FocusOut",0
};
//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDK_ANCESTOR);
DEFINE_EVENT_LIST(CLASS,VDK_ANCESTOR);
///////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, 
			       VDKForm* owner,
			       float defValue = 0.0,
			       float lower,
			       float upper,
			       float step,
			       float climb):
  VDK_ANCESTOR(owner,defValue,lower,upper,step, climb),
  VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties list 
  // (Others props are prepended  by VDKBObject constructor)
  for(t=0; vdkspinbutton_props[t]; t++)
    proplist.add(VDKBProperty(vdkspinbutton_props[t]));
  SetPropValue("Digits","2");
  SetPropValue("DefValue","0.0");
  SetPropValue("Lower","0.0");
  SetPropValue("Upper","1.0");
  SetPropValue("Step","0.05");
  SetPropValue("Climb","0.0");
  // add to VDKBObject signal list label button signals
  for(t=0; vdkspinbutton_signals[t]; t++)
    siglist.add(VDKBSignal(vdkspinbutton_signals[t],
			   this,
			   vdkspinbutton_nicknames[t]));
  // connects events.
 CONNECT_COMMON_EVENTS;

  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tDigits:%s;",(char*) GetProp("Digits"));
  fprintf(fp,"\n\tDefValue:%s;",(char*) GetProp("DefValue"));
  fprintf(fp,"\n\tLower:%s;",(char*) GetProp("Lower"));
  fprintf(fp,"\n\tUpper:%s;",(char*) GetProp("Upper"));
  fprintf(fp,"\n\tStep:%s;",(char*) GetProp("Step"));
  fprintf(fp,"\n\tClimb:%s;",(char*) GetProp("Climb"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label button counter
  // to ensure unicity
  for(sprintf(buff,"SpinButton%d",CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"SpinButton%d",CLASS::Counter);
  CLASS* button = new CLASS(buff,owner);
  if(owner->Active)
    {
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  bool flag = false;
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(owner->Owner());
	  if(prjman && prjman->objInspector)
	    flag = prjman->objInspector->preview->Checked;
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      button->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      button->SetPropValue( EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(button,int(event->x),
				   int(event->y),
				   flag,true,true);
	    }
	  else
	    container->AddWidget(button,l_justify,flag,flag,true,flag);
	}
      else if(! owner->Active->AddToParent(button,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  double defvalue = 0.0,
    lower = 0.0,
    upper = 1.0,
    step = 0.05,
    climb = 0.0;
  int digits = 2;

  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  // get default values
  if(parser.GetParam(arg,buffer,"Digits:"))
    digits = atoi(arg);
  if(parser.GetParam(arg,buffer,"DefValue:"))
    defvalue = atof(arg);
  if(parser.GetParam(arg,buffer,"Lower:"))
    lower = atof(arg);
  if(parser.GetParam(arg,buffer,"Upper:"))
    upper = atof(arg);
  if(parser.GetParam(arg,buffer,"Step:"))
    step = atof(arg);
  if(parser.GetParam(arg,buffer,"Climb:"))
    climb = atof(arg);
  source = new char[4096];
  
  sprintf(tmp,"\n%s = new %s(this,%f,%f,%f,%f,%f);",
	  obj_name,VDK_CLASS,defvalue,lower,upper,step,climb);
  strcpy(source,tmp);
  sprintf(tmp,"\n%s->Digits = %d;",obj_name,digits);
  strcat(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
  char obj_name[128];
  char obj_parent[128];
  double defvalue = 0.0,
    lower = 0.0,
    upper = 1.0,
    step = 0.05,
    climb = 0.0;
  int digits = 2;
  char arg[64];
  CLASS* button;
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  // get default values
  if(parser.GetParam(arg,buffer,"Digits:"))
    digits = atoi(arg);
  if(parser.GetParam(arg,buffer,"DefValue:"))
    defvalue = atof(arg);
  if(parser.GetParam(arg,buffer,"Lower:"))
    lower = atof(arg);
  if(parser.GetParam(arg,buffer,"Upper:"))
    upper = atof(arg);
  if(parser.GetParam(arg,buffer,"Step:"))
    step = atof(arg);
  if(parser.GetParam(arg,buffer,"Climb:"))
    climb = atof(arg);
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      button = new CLASS(obj_name,owner,defvalue,
				  lower,upper,step,climb);
      button->Digits = digits;
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(button,buffer,parser);
      // prepares packing args based on those loaded by 
      sprintf(buff,"%d",digits);
      button->SetPropValue("Digits",buff);

      sprintf(buff,"%f",defvalue);
      button->SetPropValue("DefValue",buff);

      sprintf(buff,"%f",lower);
      button->SetPropValue("Lower",buff);

      sprintf(buff,"%f",upper);
      button->SetPropValue("Upper",buff);

      sprintf(buff,"%f",step);
      button->SetPropValue("Step",buff);

      sprintf(buff,"%f",climb);
      button->SetPropValue("Climb",buff);

      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  button->GetProp(JUSTIFY_INTERNAL);
      justification =  atoi(p);
      p = (char*) button->GetProp( EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) button->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) button->GetProp("_Padding");
      padding = atoi(p);
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(button,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,
				  v_box,shadow_etched_in);

  VDKTable *table = new VDKTable(inspector,6,2);
  table->SetSize(219,-1);

  table->AddToCell(new VDKLabel(inspector,"Default value"),0,0);
  defvalue = new VDKEntry(inspector,0,(char*) GetProp("DefValue"));
  defvalue->SetSize(100,-1);
  table->AddToCell(defvalue,0,1);
  defvalue->Parent(this);
  SignalConnect(defvalue,"activate",&CLASS::OnSetDefValue);

  table->AddToCell(new VDKLabel(inspector,"Lower range value"),1,0);
  lower = new VDKEntry(inspector,0,(char*) GetProp("Lower"));
  lower->SetSize(100,-1);
  table->AddToCell(lower,1,1);
  lower->Parent(this);
  SignalConnect(lower,"activate",&CLASS::OnSetLower);

  table->AddToCell(new VDKLabel(inspector,"Upper range value"),2,0);
  upper = new VDKEntry(inspector,0,(char*) GetProp("Upper"));
  upper->SetSize(100,-1);
  table->AddToCell(upper,2,1);
  upper->Parent(this);
  SignalConnect(upper,"activate",&CLASS::OnSetUpper);

  table->AddToCell(new VDKLabel(inspector,"Step increment"),3,0);
  step = new VDKEntry(inspector,0,(char*) GetProp("Step"));
  step->SetSize(100,-1);
  table->AddToCell(step,3,1);
  step->Parent(this);
  SignalConnect(step,"activate",&CLASS::OnSetStep);

  table->AddToCell(new VDKLabel(inspector,"Climb rate"),4,0);
  climb = new VDKEntry(inspector,0,(char*) GetProp("Climb"));
  climb->SetSize(100,-1);
  table->AddToCell(climb,4,1);
  climb->Parent(this);
  SignalConnect(climb,"activate",&CLASS::OnSetClimb);

  table->AddToCell(new VDKLabel(inspector,"Precision"),5,0);
  digits = new VDKEntry(inspector,0,(char*) GetProp("Digits"));
  digits->SetSize(100,-1);
  table->AddToCell(digits,5,1);
  digits->Parent(this);
  SignalConnect(digits,"activate",&CLASS::OnSetDigits);

  bframe->Add(table,l_justify,false,false,false);
  return bframe; 
}
//////////////////////////////////////////////////////
// These response methods actually change both 
// properties on widget and gui widget properties
//////////////////////////////////////////////////////
/*
 */
bool
CLASS::OnSetDefValue(VDKObject*)
{
  if(strlen(defvalue->Text) > 0)
    SetPropValue("DefValue",(char*) defvalue->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetLower(VDKObject*)
{
  if(strlen(lower->Text) > 0)
    SetPropValue("Lower",(char*) lower->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetUpper(VDKObject*)
{
  if(strlen(upper->Text) > 0)
    SetPropValue("Upper",(char*) upper->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetStep(VDKObject*)
{
  if(strlen(step->Text) > 0)
    SetPropValue("Step",(char*) step->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetClimb(VDKObject*)
{
  if(strlen(climb->Text) > 0)
    SetPropValue("Climb",(char*) climb->Text);
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetDigits(VDKObject*)
{
  if(strlen(digits->Text) > 0)
    SetPropValue("Digits",(char*) digits->Text);
  inspector->FormNeedToBeChanged();
  return true;
}





