/*
 * ===========================
 * VDK Builder
 * Version 0.1
 * Revision 0.0
 * December 1998
 * ===========================
 *
 * Copyright (C) 1998,1999 Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * Based on VDK Library 
 * Copyright (C) 1998, Mario Motta
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <vdkb_text.h>
#include <gdk/gdkkeysyms.h>
#include <vdkb_utils.h>
#include <stdio.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <vdkb.h>
#include <vdkb_utils.h>
#define vdkeCourier "-*-courier-medium-r-*-*-12-*-*-*-*-*-*-*"
#define vdkeCourierBold "-*-courier-bold-r-*-*-12-*-*-*-*-*-*-*"
#define vdkeCourierSlant "-*-courier-medium-o-*-*-12-*-*-*-*-*-*-*"

#ifdef HAVE_SYNTAX
DEFINE_SIGNAL_LIST(VDKBText,VDKEditor);
DEFINE_EVENT_LIST(VDKBText, VDKEditor);
#else
DEFINE_SIGNAL_LIST(VDKBText,VDKSCText);
DEFINE_EVENT_LIST(VDKBText, VDKSCText);
#endif

//===================================
const unsigned int guessOffset = 1000;
static char buff[256];
/*
 */
void
VDKBText::DeleteText(GtkWidget* wid, int start, int end, void* gp)
{
  VDKBText* text = reinterpret_cast<VDKBText*>(gp);
  ActionDelete* action = new ActionDelete(text,start,end);
  text->action_stack.push(action);
}
/*
 */
void
VDKBText::InsertText(GtkWidget* wid, 
		     const char* buf,
		     int len, int pos, void* gp)

{
  VDKBText* text = reinterpret_cast<VDKBText*>(gp);
  int position = text->Pointer;
  ActionInsert* action = new ActionInsert(text,position,len);
  text->action_stack.push(action);
}
/*
*/
bool
VDKBText::KeyEventAfter(VDKObject *sender,GdkEvent *event)
{
  GdkEventKey *ev = (GdkEventKey*) event;
  // dispatch directly to text parent (VDKBnotebook)
  // from wich will reach VDKBEditor or VDKSCText
  if(Changed)
      Parent()->SignalEmit("text_changed");
  // others keys:
  GList* g;
  int t = 0;
  switch(ev->keyval)
    {
    case GDK_Up:
    case GDK_Down:
    case GDK_Page_Up:
    case GDK_Page_Down:
      g  = GTK_SCTEXT(WrappedWidget())->current_line;
      for(t = 0; g; g = g->prev,t++);
      CurrentLine(t);
      Parent()->SignalEmit("line_changed");
      break;
    case GDK_F3:
      Parent()->SignalEmit("repeat_search_text");
      break;
    }
  return true;
}
/*
 */
bool
VDKBText::KeyEventBefore(VDKObject *sender,GdkEvent *event)
{
  GdkEventKey *ev = (GdkEventKey*) event;
  if(! Editable)
    return true;
  Action* action;
  switch(ev->keyval)
    {

    case GDK_F4:
      action = action_stack.pop();
      if(action)
	{
	  action->undo();
	  delete action;
	}
      else
	Parent()->SignalEmit("no_more_undo");
      break;

    case GDK_F5:

#ifdef VDKBDEBUG
      printf("\nsorry, redo not yet implemented");
      fflush(stdout);
#endif
      break;

#ifdef HAVE_SYNTAX 
    case GDK_F6:
      gtk_editor_hilite_buffer (GTK_EDITOR(WrappedWidget()));
      break;
#endif 

    default:
      break;
    }
  gtk_signal_emit_stop_by_name(GTK_OBJECT(WrappedWidget()),
				   "key_press_event");
  return true;
}
/*
*/
bool
VDKBText::ButtonPressEvent(VDKObject* , GdkEvent* )
{
  GList* g;
  int t = 0;
  g  = GTK_SCTEXT(WrappedWidget())->current_line;
    // check if file is changed from disk
  char* p = Filename();
  if(p)
    {
     struct stat finfo;
     if(!stat(p,&finfo))
       {
       if( (Mtime() != 0) &&
	  (finfo.st_mtime !=  Mtime()) )
	   {
	     // FIX ME: adjust dialog text with lang support
	     sprintf(buff,"%s\n%s",p,"Changed from disk, reload ?");
	     if(Owner()-> Application()->MessageBox(APPNAME,
					      buff,
					      MB_ICONQUESTION|MB_YESNO,
					      user_messages[user_ok],
					      user_messages[user_no]) == IDYES)
	       {
		 Clear();
		 LoadFromFile(p);
	       }
	   }
       else
	 {
	   for(t = 0; g; g = g->prev,t++);
	   // FIX ME: line counter does not work well
	   CurrentLine(t);
	   Parent()->SignalEmit("line_changed");
	 }
       }
    }
  return true;
}
/*
pop menu
*/
bool
VDKBText::ButtonReleaseEvent(VDKObject* sender, GdkEvent *event)
{
  // casts to button event
  GdkEventButton *ev = (GdkEventButton*) event;
  // pops menu on right button pressed
  if(ev->button == 3)
    {
      Parent()->SignalEmit("pop_menu");
      return TRUE;
    }
  return true;
}

/*
actually unused
*/
bool
VDKBText::MappedEvent(VDKObject* sender, GdkEvent *event)
{

  if(!mapped)
    {
      mapped = true;
      Pointer = 0;
    }
  return true;
}
/*
*/
bool
VDKBText::Realized(VDKObject* sender)
{
  char* cc_ext   = (char*) VDKBuilder::ideDefaults.unit.cc_ext;
  char* h_ext    = (char*) VDKBuilder::ideDefaults.unit.h_ext;
  // this flag is set to false by constructor
  // so only one realized signal is catched
  if(!realized)
    {
      SetStyle();
      // load from file if found
      if(!access(Filename(),F_OK)) 
	LoadFromFile(Filename());
      // create a new unit file if not empty filename
      else if(*Filename())
	{ 
	  // is a new unit file ?
	  Changed = true;
	  char *local = new char[strlen(Filename())+1];
	  strcpy(local,Filename());
	  char* ext = get_extension(local);
	  // advance to jump '.' 
	  if(ext)
	    {
	      *ext = '\0';
	      ext++;
	      // is a .cc file ?
	      if(!strcmp(ext,cc_ext))
		{
		  sprintf(buff,"#include <%s.h>\n",local);
		  TextInsert(buff);
		} 
	      // is .h file ?
	      else if(!strcmp(ext,h_ext))
		{
		  TextInsert("//=======================================\n");
		  sprintf(buff,"#ifndef _%s_h\n",local);
		  TextInsert(buff);
		  sprintf(buff,"#define _%s_h\n",local);
  		  TextInsert("//====== put your code below here =======\n");
		  TextInsert(buff);
		  TextInsert("\n\n"); 
  		  TextInsert("//=======================================\n");
		  TextInsert("#endif\n");
		}
	    }
	  delete local;
	}
      realized = true;
      Pointer = 0;
    } 
 return true;
} 

/*
  set syntax colouring style
  (not yet user customizable)
*/
void 
VDKBText::SetStyle()
{
  bool hilite = Hilite;
  char* font = (char*) VDKBuilder::ideDefaults.editor.font;
  if(font)
    {
      VDKFont* edit_font = new VDKFont(this,font);
      if( *edit_font != (GdkFont*) NULL)
	Font = edit_font;
      else
	edit_font->Destroy();
    }
  // force to false if do not have syntax installed
#ifndef HAVE_SYNTAX 
  hilite = false;
#endif
  char* bg_color = (char*) VDKBuilder::ideDefaults.editor.bg;
  if(bg_color) 
    NormalBackground = VDKRgb(bg_color);

  if(!hilite)
    {
      char* fg_color = (char*) VDKBuilder::ideDefaults.editor.fg;
      if(fg_color)  
	{
	  TextFont.foreground = *(new VDKColor(this,(fg_color)));
	  Foreground = VDKRgb(fg_color);
	}
    }
  
#ifdef HAVE_SYNTAX 
  else
    {
      //VDKColor *comment_color = new VDKColor(this,clSiena);
      VDKColor *comment_color = new VDKColor(this,
	   (char*) VDKBuilder::ideDefaults.editor.comment_color);
      //VDKFont  *comment_font  = new VDKFont(this,vdkeCourierSlant);
      VDKFont  *comment_font  = new VDKFont(this,
           (char*) VDKBuilder::ideDefaults.editor.comment_font);

      // VDKColor *string_const_color = new VDKColor(this,clBlue);
      VDKColor *string_const_color = new VDKColor(this,
	   (char*) VDKBuilder::ideDefaults.editor.string_const_color);
      // VDKFont  *string_const_font = new VDKFont(this,vdkeCourier);
      VDKFont  *string_const_font = new VDKFont(this,
           (char*) VDKBuilder::ideDefaults.editor.string_const_font);

      // VDKColor *chars_const_color = new VDKColor(this,clRed);
      VDKColor *chars_const_color = new VDKColor(this,
	   (char*) VDKBuilder::ideDefaults.editor.chars_const_color);
      // VDKFont  *chars_const_font = new VDKFont(this,vdkeCourier);
      VDKFont  *chars_const_font = new VDKFont(this,
           (char*) VDKBuilder::ideDefaults.editor.chars_const_font);
      InstallSyntaxTable (comment_color,
			  comment_font,
			  string_const_color,
			  string_const_font,
			  chars_const_color,
			  chars_const_font);
      
      // VDKColor* siena = new VDKColor(this,clSiena);
      VDKColor* siena = new VDKColor(this,
	   (char*) VDKBuilder::ideDefaults.editor.key_color);
      //VDKFont*  bold = new VDKFont(this,vdkeCourierBold);
      VDKFont*  bold = new VDKFont(this,
	   (char*) VDKBuilder::ideDefaults.editor.key_font);
      // VDKColor* forestg = new VDKColor(this,0,154,0);
      VDKColor* forestg = new VDKColor(this,
	   (char*) VDKBuilder::ideDefaults.editor.preprocess_color);
      InstallPatternTable(siena, bold,forestg);
    }
#endif
}

//////////////////////////////////////
/*
 */
VDKBText::VDKBText(VDKForm* owner, bool editable, char* filename ):
#ifndef HAVE_SYNTAX
  VDKSCText(owner,editable),filename(filename),
#else
  VDKEditor(owner,editable),filename(filename),
#endif
    CurrentLine("CurrentLine",this,1)
{
  Editable = editable;
  mapped = false;
  lastGuessed = 0;

  realized = false;
  Hilite = false;

#ifdef HAVE_SYNTAX
  GTK_EDITOR(text)->screen_size =  400;
#endif

  // setup event handler to intercept key press
  // intercepts undo/redo, stops signal in this case
  // and process undo/redo 
  // "true" as last arg means connect after (default false)
  EventConnect("key_press_event",&VDKBText::KeyEventAfter,true);
  EventConnect("button_press_event",&VDKBText::ButtonPressEvent,true);
  EventConnect("button_release_event", &VDKBText::ButtonReleaseEvent,true);
  EventConnect("map_event",&VDKBText::MappedEvent,true);
  SignalConnect("realize",&VDKBText::Realized,true,true);
  EventConnect("key_press_event",&VDKBText::KeyEventBefore);
  gtk_signal_connect(GTK_OBJECT(text),"delete_text",
		     GTK_SIGNAL_FUNC(VDKBText::DeleteText),this);
  gtk_signal_connect(GTK_OBJECT(text),"insert_text",
		     GTK_SIGNAL_FUNC(VDKBText::InsertText),this);
  // load file last modification time
  if(filename)
    {
     struct stat finfo;
     if(!stat(filename,&finfo))
       mtime = finfo.st_mtime;
     else
       mtime = 0;
    }
  else
    mtime = 0;
}
/*
 */
int 
VDKBText::SaveToFile(char* filename) 
{

  int result = VDKSCText::SaveToFile(filename);
  if(filename)
    {
      struct stat finfo;
      if(!stat(filename,&finfo))
	mtime = finfo.st_mtime;
      else
	mtime = 0;
    }
  else
    mtime = 0;
  return result;
}
/*
 */
int 
VDKBText::LoadFromFile(char* filename)
{
 int result = VDKSCText::LoadFromFile(filename);
  if(filename)
    {
      struct stat finfo;
      if(!stat(filename,&finfo))
	mtime = finfo.st_mtime;
      else
	mtime = 0;
    }
  else
    mtime = 0;
  return result;
}
///////////////////////////////////////
/*
 */
VDKBText::~VDKBText()
{
action_stack.flush();
}

/* 
 */
char* VDKBText::ShortName()
{
  char* p = (char*) filename;
  char* s = get_shortfilename(p);
  return s ? s : p;
}
/*
 */
char* VDKBText::Extension()
{
char* p = (char*) filename;
return get_extension(p);
}
/*
 */
char* VDKBText::Filename(char* name)
{
if(name)
  filename = name;
return (char*) filename;
}

bool
VDKBText::GoToLine(int line)
{
  char* buf = GetChars(0,-1);
  char* p = buf; 
  if(!p)
    return false;
  // count lines and chars
  int l,count;
  for(l = count = 0; (l < line) && *p ;p++,count++)
    if(*p == '\n') l++;
  for(;*p;p++)
    if(*p == '\n') l++;
  GtkAdjustment* adj = GTK_SCTEXT(text)->vadj;
  float range = adj->upper-adj->lower;
  float factor = float(line) / float(l);
  // this should be made smarter without use such a const value
  // 
  float linetoscroll = adj->lower + range*factor-50;
  if(linetoscroll > adj->lower && linetoscroll < adj->upper) 
    gtk_adjustment_set_value(adj,linetoscroll);
  Pointer = count;
  gtk_editable_select_region(GTK_EDITABLE(text), count-2, count);
  // what you get from GetChars must be freed.
  g_free(buf);
  return true;
}
/*
 */
void
VDKBText::ScrollTo(int pos)
{
  int count = 0,line = 0,totLines = 0;
  char* buf = GetChars(0,-1);
  char* p = buf; 
  if(!p)
    return ;
  for(; (count < pos) && *p ; p++,count++)
    if(*p == '\n') line++;
  for(totLines = line;*p;p++)
    if(*p == '\n') totLines++;
  GtkAdjustment* adj = GTK_SCTEXT(text)->vadj;
  float range = adj->upper-adj->lower;
  float factor = float(line) / float(totLines);
  float linetoscroll = adj->lower + range*factor-50;
  // this should be made smarter without use such a const value
  if(linetoscroll > adj->lower && linetoscroll < adj->upper) 
    gtk_adjustment_set_value(adj,linetoscroll);
  g_free(buf);
}

int
VDKBText::Search(char* st,int from, bool select, bool bell)
{
char* buf = GetChars(from,-1);
int pos = -1;
if(! buf)
  return 0;
char* p = strstr(buf,st);
if(p)
  {
    pos = from + (p-buf);
    ScrollTo(pos);
    Pointer = pos;
    if(select)
	gtk_editable_select_region (GTK_EDITABLE (WrappedWidget()), pos,
				pos + strlen (st));
  }
else if(bell)
  gdk_beep();
g_free(buf);
return pos;
}
/*
 */
int 
VDKBText::GuessPosition(int cl)
{
  char* buf = GetChars(0,-1);
  int l = 0; 
  char* p = buf;
  int t = 0;
  for(; l < cl && *p; p++,t++)
    if(*p == '\n') l++;
  g_free(buf);
  return t;
}



