/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_textlabel.h>
#include <vdkb_evbox.h>
#include <vdkb_utils.h>
#include <vdkb_form.h>
#include <vdkb_parser.h>
#include <vdkb_objinspect.h>
#include <vdkb_prjman.h>
#include <stdlib.h>
#include <vdkb_fixed.h>
/*
================================
symbolic constants to templatize
a bit 
================================
*/
// for methods and other stuff
#define CLASS VDKBTextLabel
// put here vdk class name string
#define VDK_CLASS "VDKLabel"
// put here vdk class name 
#define VDK_ANCESTOR  VDKLabel
// put here here the widget will be named
// (name+counter)
#define VDK_WIDGET "label"
static char buff[128];

// used to autogenerate default
// labelbutton  names and captions

int CLASS::Counter = 0;
/* 
 label properties names
 */
char* vdklabel_props[] =
{ 
"Caption","CaptionWrap","Justify",0
};
//////////////////////////////////////////////////
// dynamic tables
DEFINE_SIGNAL_LIST(CLASS,VDKBEvLabel);
DEFINE_EVENT_LIST(CLASS,VDKBEvLabel);
///////////////////////////////////////////////////
/*
  - constructor
 */
CLASS::CLASS(char* name, VDKForm* owner,
				 char* label):
  VDKBEvLabel(owner,label),VDKBObject(name)
{
  int t;
  // newly constructed widget counter is incremented
  // each time 
  Counter++;
  // assign this to VDKBObject <object> member.
  object = this;
  // add to VDKBObject properties list label button properties
  // names. (Others props are prepended  by VDKBObject constructor)
  for(t=0; vdklabel_props[t]; t++)
    proplist.add(VDKBProperty(vdklabel_props[t]));
  if(label)
    SetPropValue("Caption",label);
  // connects events.
  CONNECT_COMMON_EVENTS;
  // makes a pop menu common to all widgets (in vdkb_widpopmenu.cc/h)
  // this pop menu will be popped at righ button press event.
  popmenu = new VDKBWidgetPopMenu(this);
}
////////////////////////////////////////////////////////////////
//
//               WRITER TO .FRM FILE
//
///////////////////////////////////////////////////////////////
/*
Writes a .frm format representation of label button widget
This virtual function is called by VDKBForm::WriteBoxesOnFrm()
a recursive algorithm that scans VDKBForm widget tree.
*/
void
CLASS::WriteOnFrm(FILE* fp, VDKBObject* parentobj)
{
  // first of all call ancestor to write common properties
  VDKBObject::WriteOnFrm(fp,parentobj);
  fprintf(fp,"\n\tCaption:\"%s\";",(char*) GetProp("Caption"));
  fprintf(fp,"\n\tCaptionWrap:%s;",(char*) GetProp("CaptionWrap"));
  fprintf(fp,"\n\tJustify:%s;",(char*) GetProp("Justify"));
}
//////////////////////////////////////////////////////////////////
//
//               PREPARE GUI WIDGETS
//
//////////////////////////////////////////////////////////////////
/*
This method is called by global MakeWidget() in vdkb_design.cc
MakeWidget() scans a table that maps class id's with each
static MakeWidget() for each class. Class id's are generated
during clicks on widget palette.
On return:
0 - successfull
1 - unsupported widget
2 - target is not a container
 */
int
CLASS::MakeWidget(VDKBGuiForm* owner, GdkEvent* ev)
{
  // autogenerate first suitable label button counter
  // to ensure unicity
  for(sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
      owner->ChildWithName(buff)!= (VDKObject*) NULL;
      CLASS::Counter++)
    sprintf(buff,"%s%d",VDK_WIDGET,CLASS::Counter);
  CLASS* label = new CLASS(buff,owner,buff);
  if(owner->Active)
    {
      bool flag = false;
      VDKBProjectManager* prjman = 
	dynamic_cast<VDKBProjectManager*>(owner->Owner());
      if(prjman && prjman->objInspector)
	flag = prjman->objInspector->preview->Checked;
      VDKBEventContainer* container = 
	dynamic_cast<VDKBEventContainer*>(owner->Active);
      if(container)
	{
	  if(ev && dynamic_cast<VDKBFixed*>(container))
	    {
	      GdkEventButton* event = (GdkEventButton*) ev;
	      sprintf(buff,"%d",int(event->x));
	      label->SetPropValue(JUSTIFY_INTERNAL,buff);
	      sprintf(buff,"%d",int(event->y));
	      label->SetPropValue( EXPAND_INTERNAL,buff);
	      // others than justify and flag unuseful
	      container->AddWidget(label,int(event->x),
				   int(event->y),
				   true,true,true);
	    }
	  else
	    container->AddWidget(label,l_justify,flag,flag,true,flag);
	}
       else if(! owner->Active->AddToParent(label,ev))
	return 2;
    }
  else
    return 3;
 return 0;
}
/*
This is called by a global CreateSource() in vdkb_parser.cc.
CreateSource() scans a table that maps class names with
each static CreateSource() in widget class.
 */
char*
CLASS::CreateSource(char* buffer,VDKBParser& parser)
{
  char* source;
  char obj_name[128];
  char obj_parent[128];
  char arg[64];
  char tmp[256];
  char obj_caption[128];
  // get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return NULL;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return NULL;
  source = new char[4096];
  if(parser.GetParam(obj_caption,buffer,"Caption:") && 
     strcmp(obj_caption,NIHIL_PROP))
    ;
  else
    strcpy(obj_caption," ");
  sprintf(tmp,"\n%s = new %s(this,\"%s\");",
	  obj_name,VDK_CLASS,obj_caption);
  strcpy(source,tmp);
  ///////////////////////////////////////
  // call ancestor to set common properties
  char* props = VDKBObject::CreateSource(buffer,parser,obj_name);
  if(props)
    {
      strcat(source,props);
      delete props;
    }
  // get label button  props
  if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->CaptionWrap = %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  if(parser.GetParam(arg,buffer,"Justify:") && strcmp(arg,NIHIL_PROP))
   {
     sprintf(tmp,"\n%s->Justify =  (GtkJustification) %s;",obj_name,arg);
     strcat(source,tmp);
   }    
  // prepares arguments to add widget to container
  char justify[16],expand[16],fill[16],padding[16];
  if(parser.GetParam(justify,buffer,PROP_JUSTIFY_INTERNAL) &&
     parser.GetParam(expand,buffer,PROP_EXPAND_INTERNAL) &&
     parser.GetParam(fill,buffer,PROP_FILL_INTERNAL) &&
     parser.GetParam(padding,buffer,PROP_PADDING_INTERNAL))
    {
      sprintf(tmp,"\n%s->Add(%s,%s,%s,%s,%s);",
	      obj_parent,obj_name,justify,expand,fill,padding);
      strcat(source,tmp);
    }
  else
    {
      sprintf(tmp,"\n%s->Add(%s);",
	      obj_parent,obj_name);
      strcat(source,tmp);
    }
  // visible property must be wrote after adding it to a parent
  // container. That's the reason why is written here and not
  // in vdkb_object class as should be.
  // written only if == false
  if(parser.GetParam(arg,buffer,PROP_VISIBLE) && !strcmp(arg,CHECK_FALSE))
   {
     sprintf(tmp,"\n%s->Visible = %s;",obj_name,arg);
     strcat(source,tmp);
   }
  return source;
}
/*
Invoked by VDKBGuiForm::MakeGuiObjects() during gui creation
reading .frm file.
MakeGuiObjects() scans .frm file and call a global CreateWidget()
that scans a table that maps class names with
each static CreateWidget() in widget class.
*/
bool
CLASS::CreateWidget(VDKBGuiForm* owner, 
			      char* buffer,VDKBParser& parser)
{
char obj_name[128];
char obj_parent[128];
char obj_caption[256];
char arg[64];
CLASS* label;
// get name, and parent
  if ( !parser.GetParam(obj_name,buffer,"this:") ||
       !parser.GetParam(obj_parent,buffer,"parent:") 
       )
    return false;
  // no parent widget , error
  if(!strcmp(obj_parent,NIHIL_PROP))
    return false;
  // get caption
    if(parser.GetParam(obj_caption,buffer,"Caption:") && 
       strcmp(obj_caption,NIHIL_PROP))
      ;
    else
      strcpy(obj_caption," ");
  VDKObject* p = owner->ChildWithName(obj_parent);
  VDKBEventContainer* container = p ? 
    dynamic_cast<VDKBEventContainer*>(p) : (VDKBEventContainer*) NULL;
  if(container)
    {
      label = new CLASS(obj_name,owner,obj_caption);
      /////////////////////////////////////////
      // call ancestor to set common properties
      VDKBObject::CreateWidget(label,buffer,parser);
      // prepares packing args based on those loaded by 
      // VDKBObject::CreateWidget()
      int justification = l_justify;
      int expand=0,fill=0,padding=0;
      char* p = (char*)  label->GetProp(JUSTIFY_INTERNAL);
      justification = atoi(p);
      p = (char*) label->GetProp( EXPAND_INTERNAL);
      expand = atoi(p);
      p = (char*) label->GetProp("_Fill");
      fill = atoi(p);
      p = (char*) label->GetProp("_Padding");
      padding = atoi(p);
      if(parser.GetParam(arg,buffer,"CaptionWrap:") && strcmp(arg,NIHIL_PROP))
	{
	label->CaptionWrap = !strcmp(arg,CHECK_TRUE);
	label->SetPropValue("CaptionWrap",arg);
	}
      if(parser.GetParam(arg,buffer,"Justify:") && strcmp(arg,NIHIL_PROP))
	{
	  label->Justify =  (GtkJustification) atoi(arg);
	  label->SetPropValue("Justify",arg);
	}
      // last arg make  AddWidget() to use packing args.
      // By default all new widgets are added to a form
      // with l_justify,false,false,false.
      container->AddWidget(label,justification,expand,fill,padding,true);
    }
  else
    return false;
return true;
}
/////////////////////////////////////////////////////
//           OBJECT INSPECTOR MANAGEMENT
////////////////////////////////////////////////////
/*
 */
static char *aligns[] = 
{ "l_justify","r_justify","c_justify",0 };
/*
 */
VDKObjectContainer* 
CLASS::ExtraWidget(VDKBObjectInspector* isp) 
{ 
  inspector = isp;
  VDKFrame* bframe = new VDKFrame(inspector,NULL,v_box,shadow_etched_in);
  // FIX ME: lang support
  VDKTable* table = new VDKTable(inspector,2,3,false);
  table->SetSize(219,-1);
  VDKLabel* label = new VDKLabel(inspector,"Caption");
  table->AddToCell(label,0,0);
  VDKString c = (char*) GetProp("Caption");
  caption = new VDKEntry(inspector,0,
			 strcmp((char*) c,NIHIL_PROP) ? (char*) c : NULL);
  caption->SetSize(100,-1);
  table->AddToCell(caption,0,1);
  caption->Parent(this);
  SignalConnect(caption,"activate",&CLASS::OnSetCaption);

  wrap = new VDKCheckButton(inspector,"Caption wrap");
  table->AddToCell(wrap,1,1);
  wrap->Parent(this);
  SignalConnect(wrap,"toggled",&CLASS::OnSetCaptionWrap);

  set = new VDKLabelButton(inspector,"Set justify");
  table->AddToCell(set,2,0);
  set->Parent(this);
  SignalConnect(set,"clicked",&CLASS::OnSetJustify);
  int r = atoi(GetProp("Justify"));
  align = new VDKCombo(inspector,NULL);
  align->SetSize(100,-1);
  StringList sl;
  int t = 0;
  for(;aligns[t];t++)
    sl.add(VDKString(aligns[t]));
  align->PopdownStrings = sl;
  align->SelectItem(r);
  table->AddToCell(align,2,1);

  bframe->Add(table,false,false,false);
  return bframe; 
}
//////////////////////////////////////////////////////
// These response methods actually change both 
// properties on widget and gui widget properties
//////////////////////////////////////////////////////
/*
 */
bool
CLASS::OnSetCaptionWrap(VDKObject*)
{
  SetPropValue("CaptionWrap", wrap->Checked ? CHECK_TRUE : CHECK_FALSE);
  CaptionWrap = wrap->Checked ? true : false;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetJustify(VDKObject*)
{
  int sel = align->Selected;
  sprintf(buff,"%d", sel >= 0 ? sel : 0);
  int r = atoi(buff);
  SetPropValue("Justify",buff);
  Justify =  (GtkJustification) r;
  inspector->FormNeedToBeChanged();
  return true;
}
/*
 */
bool
CLASS::OnSetCaption(VDKObject*)
{
  if(strlen(caption->Text)>0)
    sprintf(buff,"%s",(char*) caption->Text);
  else
    sprintf(buff,"%s",NIHIL_PROP);
  SetPropValue("Caption",buff);
  Caption = caption->Text;
  inspector->FormNeedToBeChanged();
  return true;
}








