/*
 * ===========================
 * VDK Builder
 * Version 0.1.1
 * Revision 0.0
 * March 1999
 * ===========================
 *
 * Copyright (C) 1998, Mario Motta
 * Developed by Mario Motta <mmotta@guest.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-130
 */ 
#include <vdkb_widpopmenu.h>
#include <vdkb_dlgs.h>
#include <vdkb_form.h>
#include <vdkb_prjman.h>
DEFINE_SIGNAL_LIST(VDKBWidgetPopMenu,VDKMenu);
/*
 */
bool
VDKBWidgetPopMenu::MakeInspector(VDKObject*)
{
VDKBObject* vdkobj = dynamic_cast<VDKBObject*>(parent_widget);
if(vdkobj)
  vdkobj->PopObjectBrowser();
return true;
}
/*
 */
bool
VDKBWidgetPopMenu::SelectParentContainer(VDKObject*)
{
  VDKBEventContainer* container = 
    dynamic_cast<VDKBEventContainer*>(parent_widget->Parent());
  if(container)
    {
      VDKBGuiForm* ownerform = 
	dynamic_cast<VDKBGuiForm*>(parent_widget->Owner());
      if(ownerform)
	{
	  ownerform->Active->ClearMark();
	  ownerform->Active = container;
	  ownerform->Active->Mark();
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(ownerform->Owner());
	  if(prjman && prjman->objInspector)
	    prjman->objInspector->SetActive(container);
	}
    }
  return true;
}
/*
 */
bool
VDKBWidgetPopMenu::DelWidget(VDKObject*)
{
  VDKBObject*  parent;
  VDKBEventContainer* container = 
    dynamic_cast<VDKBEventContainer*>(parent_widget->Parent());
  if(container)
    {
      VDKBGuiForm* ownerform = 
	dynamic_cast<VDKBGuiForm*>(parent_widget->Owner());
      if(ownerform)
	{
	  parent = dynamic_cast<VDKBObject*>(parent_widget);
	  if(ownerform->Active == parent)
	    ownerform->Active = NULL;
	  ownerform->Changed = true;
	  // notify to inspector that object is to be deleted
	  // so inspector will be disabled
	  // and (eventually) signal disconnected.
	  VDKBProjectManager* prjman = 
	    dynamic_cast<VDKBProjectManager*>(ownerform->Owner());
	  if(prjman && prjman->objInspector)
	    {
	      ownerform->DisconnectWidget(parent);
	      prjman->objInspector->SetActive(NULL);
	    }
	  
	}
      container->boxlist.remove(parent_widget);
      container->RemoveObject(parent_widget);
    }
  return true;
}
/*
 */
bool 
VDKBWidgetPopMenu::SetWidgetSize(VDKObject*)
{
  char buff[128];
  VDKBGuiForm* ownerform = 
    dynamic_cast<VDKBGuiForm*>(parent_widget->Owner());
  if(ownerform && ownerform->Active)
    {
      sprintf(buff,"Setting %s size", (char*) ownerform->Active->Name());
      VDKBPropSizeDialog *dlg = 
	new VDKBPropSizeDialog(ownerform,
			       ownerform->Active,
			       buff);
      dlg->Setup();
      dlg->ShowModal();
      ownerform->Changed = true;
    }
  return true;
}



////////////////////////////////////////////////////
/*
 */
VDKBWidgetPopMenu::VDKBWidgetPopMenu(VDKObject* widget):
  VDKMenu(widget->Owner()),parent_widget(widget)
  
{
  VDKMenuItem *nop = new VDKMenuItem(this,"Nop");
  nop->Enabled = false;
  setsize = new VDKMenuItem(this,"Set min size");
  Separator();
  selectparent = new VDKMenuItem(this,"Select parent container");
  Separator();
  delwidget = new VDKMenuItem(this,"Remove widget");
  SignalConnect(delwidget,"activate",&VDKBWidgetPopMenu::DelWidget);
  SignalConnect(setsize,"activate",&VDKBWidgetPopMenu::SetWidgetSize);
  SignalConnect(selectparent,"activate",
		&VDKBWidgetPopMenu::SelectParentContainer);
  /*  
      better add it to owner, so will be surely
      destroyed even if never popped
  */
  parent_widget->Owner()->AddItem(this);
}







